@@ -1,8 +1,16 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2023 Rumma & Ko Ltd
+# Copyright 2011-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
+# from lino.modlib.linod.choicelists import background_task, schedule_often, schedule_daily
+from lino.core.roles import SiteStaff, SiteUser, SiteAdmin, login_required
+from django.utils import translation
+from importlib import import_module
+from lino.modlib.system.choicelists import Genders, PeriodEvents, YesNo
+from lino.utils.mldbc.fields import BabelCharField, LanguageField
+from lino.utils.mldbc.fields import BabelTextField
+from lino.utils.format_date import fdl as dtosl
+from lino.utils.fieldutils import get_fields, fields_help
 from lino import logger
 
 # import logging ; logger = logging.getLogger(__name__)
@@ -26,6 +34,7 @@ from lino.core.utils import obj2str
 from lino.core.utils import obj2unicode
 from lino.core.utils import range_filter
 from lino.core.utils import inrange_filter
+from lino.core.utils import overlap_range_filter
 from lino.core.utils import full_model_name
 
 from lino.core.model import Model
@@ -53,11 +62,13 @@ from lino.core.actions import ShowSlaveTable
 from lino.core.actions import ShowTable, ShowDetail
 from lino.core.actions import ShowInsert, DeleteSelected
 from lino.core.actions import SubmitDetail, SubmitInsert
+from lino.core.actions import ShowEditor
 
 from lino.core.choicelists import ChoiceList, Choice
 from lino.core.workflows import State, Workflow, ChangeStateAction
 
-from lino.core.fields import fields_list, ImportedFields
+from lino.core.fields import ImportedFields
+from lino.core.atomizer import fields_list
 from lino.core.fields import Dummy, DummyField
 from lino.core.fields import TimeField
 from lino.core.fields import TableRow
@@ -70,16 +81,19 @@ from lino.core.fields import CustomField
 from lino.core.fields import RecurrenceField
 from lino.core.fields import IncompleteDateField
 from lino.core.fields import DatePickerField
+
 # from lino.core.fields import NullCharField
 from lino.core.fields import PasswordField
 from lino.core.fields import MonthField
 from lino.core.fields import PercentageField
+
 # ~ from lino.core.fields import LinkedForeignKey
 from lino.core.fields import QuantityField
 from lino.core.fields import DurationField
 from lino.core.fields import HtmlBox, PriceField, RichTextField
 
 from lino.core.fields import DisplayField, displayfield, htmlbox, delayedhtmlbox
+
 # from lino.core.fields import DisplayField, displayfield, htmlbox
 from lino.core.fields import VirtualField, virtualfield
 from lino.core.fields import VirtualBooleanField
@@ -91,13 +105,16 @@ from lino.core.fields import CharField
 # from lino_xl.lib.appypod.mixins import PrintTableAction
 
 from lino.core.utils import babelkw
+
 # from lino.core.utils import babelattr
-from lino.core.utils import babel_values  # alias for babelkw for backward compat
+# alias for babelkw for backward compat
+from lino.core.utils import babel_values
 
 from lino.utils.choosers import chooser, action_chooser
 
 # from lino.core.layouts import FormLayout
-from lino.core.layouts import DetailLayout, InsertLayout, Panel
+from lino.core.utils import Panel
+from lino.core.layouts import DetailLayout, InsertLayout
 from lino.core.layouts import ParamsLayout, ActionParamsLayout
 from lino.core.layouts import DummyPanel
 
@@ -111,8 +128,10 @@ from lino.core.signals import auto_create
 from lino.core.signals import pre_merge
 from lino.core.signals import pre_add_child
 from lino.core.signals import pre_remove_child
+from lino.core.signals import pre_ui_save
+from lino.core.signals import post_ui_save
 from lino.core.signals import pre_ui_build
-from lino.core.signals import post_ui_build
+# from lino.core.signals import post_ui_build
 from lino.core.signals import post_delete, pre_delete
 
 from django.db.models.signals import pre_save, post_save
@@ -124,7 +143,6 @@ from django.db.backends.signals import connection_created
 from django.dispatch import receiver
 # ~ from lino.core import signals
 
-
 from django.db.models.fields import NOT_PROVIDED
 
 from lino.core.inject import inject_action
@@ -140,7 +158,7 @@ from lino.utils import IncompleteDate, read_exception
 
 from lino.utils.format_date import fdm, fdl, fdf, fdmy
 from lino.utils.format_date import fds as fds_
-from lino.utils.format_date import ftl
+from lino.utils.format_date import fdtl, fdtf
 
 
 def fds(d):
@@ -151,40 +169,26 @@ def fds(d):
 
 # backward compatibility
 dtos = fds
-from lino.utils.format_date import fdl as dtosl
 
 babelitem = settings.SITE.babelitem
 field2kw = settings.SITE.field2kw
 # urlkwargs = settings.SITE.urlkwargs
 
-from lino.utils.mldbc.fields import BabelTextField
-from lino.utils.mldbc.fields import BabelCharField, LanguageField
-
-from lino.modlib.system.choicelists import Genders, PeriodEvents, YesNo
-
-from importlib import import_module
 
 decfmt = settings.SITE.decfmt
 str2kw = settings.SITE.str2kw
 str2dict = settings.SITE.str2dict
-
-
-def today(*args, **kwargs):
-    # make it serializable for Django migrations
-    return settings.SITE.today(*args, **kwargs)
-
-
-# today = settings.SITE.today
+now = settings.SITE.now
+# today = settings.SITE.today  # see below
 strftime = settings.SITE.strftime
 demo_date = settings.SITE.demo_date
 is_abstract_model = settings.SITE.is_abstract_model
 is_installed = settings.SITE.is_installed
+is_hidden_plugin = settings.SITE.is_hidden_plugin
 resolve_plugin = settings.SITE.resolve_plugin
 get_plugin_setting = settings.SITE.get_plugin_setting
-# get_db_overview_rst = settings.SITE.get_db_overview_rst
 add_welcome_handler = settings.SITE.add_welcome_handler
 build_media_url = settings.SITE.build_media_url
-build_site_cache_url = settings.SITE.build_site_cache_url
 build_static_url = settings.SITE.build_static_url
 get_default_language = settings.SITE.get_default_language
 get_language_info = settings.SITE.get_language_info
@@ -193,42 +197,21 @@ babelattr = settings.SITE.babelattr
 plugins = settings.SITE.plugins
 format_currency = settings.SITE.format_currency
 
-from django.utils import translation
+
+def today(*args, **kwargs):
+    # make it serializable for Django migrations
+    return settings.SITE.today(*args, **kwargs)
+
 
 get_language = translation.get_language
 
-from lino.core.roles import SiteStaff, SiteUser, SiteAdmin, login_required
 
-
-def schedule_often(every=10):
-    def decorator(func):
-        if True:  # settings.SITE.use_linod:
-            from lino.modlib.linod.choicelists import Procedures
-            Procedures.add_item(func, every_unit='secondly', every=every)
-            # Procedures.add_item(func.__name__, func.__name__, func.__name__,
-            #     every_unit='secondly', every_value=every, func=func)
-
-    return decorator
-
-
-def schedule_daily():
-    def decorator(func):
-        if True:  # settings.SITE.use_linod:
-            from lino.modlib.linod.choicelists import Procedures
-            Procedures.add_item(func, every_unit='daily', every=1)
-            # Procedures.add_item(func.__name__, func.__name__, func.__name__,
-            #     every_unit='daily', every_value=1, func=func)
-        return func
-
-    return decorator
-
-def background_task(**kwargs):
-    def decorator(func):
-        if True:  # settings.SITE.use_linod:
-            from lino.modlib.linod.choicelists import Procedures
-            Procedures.add_item(func, **kwargs)
-            # Procedures.add_item(func.__name__, func.__name__, func.__name__,
-            #     func=func, **kwargs)
-        return func
-
-    return decorator
+def auto_height(n):
+    """
+    When specifying a `window_size`, the `height` should often be ``'auto'``,
+    but extjs dopesn't support auto  height with text editor widget.
+    """
+    if settings.SITE.default_ui == "lino_react.react":
+        return "auto"
+    else:
+        return n
