@@ -1,68 +1,109 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2015-2022 Rumma & Ko Ltd
+# Copyright 2015-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
+"""
+A selection of names to be used in tested documents as follows:
+
+>>> from lino.api.doctest import *
+
+This is by convention everything we want to have in the global namespace of a
+tested document. It includes
+
+- well-known Python standard modules like os, sys, datetime and collections.
+- A variable :data:`test_client`
 
-"""
-A selection of names to be used in tested documents.
 """
 
+from lino.mixins.clonable import Clonable
+from lino.utils.fieldutils import get_fields, fields_help
+from lino.core.boundaction import BoundAction
+from lino.core.tables import AbstractTable
+from lino.core.actions import Action
+from django.db.models import Model
+from lino.core.actions import ShowTable
+from lino.core.menus import Menu
+from lino.utils.html import html2text
+from lino.utils import dbhash
+from lino.core.utils import get_models
+from lino.core.utils import full_model_name
+from lino.core.utils import full_model_name as fmn
+from lino.utils.diag import visible_for
+from lino.sphinxcontrib.actordoc import menuselection_text
+from lino import logger
+from lino.core.menus import find_menu_item
+from lino.core import constants
+from lino.core import actors, kernel
+from lino.utils.sql import sql_summary
+from lino.utils import diag
+from lino.utils.diag import analyzer
+from lino.utils.html import E, tostring, to_rst
+from lino.utils import i2d
+from lino.utils import AttrDict
+from rstgen.utils import unindent, rmu, sixprint
+from rstgen import attrtable
+import rstgen
+import pytest
+from django.db import connection, reset_queries as reset_sql_queries
+from django.test import Client
+from django.utils.encoding import force_str
+from django.utils import translation
+from lino.api.shell import *
+from lino.core.constants import *
+import os
+import sys
+import datetime
+import collections
 import six  # TODO: remove here and then run all doctests
 import logging
+import sqlparse
+import json
+import textwrap
+
+from bs4 import BeautifulSoup
+from pprint import pprint, pformat
+from urllib.parse import urlencode
+
 import django
 django.setup()
-from lino.api.shell import *
-from django.utils import translation
-from django.utils.encoding import force_str
-from django.test import Client
-from django.db import connection, reset_queries as reset_sql_queries
-import json
-from bs4 import BeautifulSoup
-import textwrap
-from pprint import pprint, pformat
 
-from rstgen import table, ul
-import rstgen
-from rstgen import attrtable
-from rstgen.utils import unindent, rmu, sixprint
 
-from lino.utils import AttrDict
-from lino.utils import i2d
-from etgen.html import E, tostring, to_rst
-from lino.utils.diag import analyzer
-from lino.utils import diag
-from lino.utils.sql import sql_summary
-from lino.core import actors, kernel
-from lino.core.menus import find_menu_item
-from lino import logger
-from lino.sphinxcontrib.actordoc import menuselection_text
-from lino.utils.diag import visible_for
-from lino.core.utils import full_model_name
+# from rstgen import table, ul
+
+
 # from lino.core.utils import PseudoRequest
-from lino.utils import html2text
-from lino.core.menus import Menu
-from lino.core.actions import ShowTable
-from lino.core.actions import register_params
-from lino.core.layouts import BaseLayout
 
 test_client = Client()
-# naming it simply "client" caused conflict with a
-# `lino_welfare.pcsw.models.Client`
+"""An instance of :class:`django.test.Client`.
+
+N.B. Naming it simply "client" caused conflict with a
+:class:`lino_welfare.pcsw.models.Client`
+
+"""
 
-import collections
 HttpQuery = collections.namedtuple(
-    'HttpQuery',
-    ['username', 'url_base', 'json_fields', 'expected_rows', 'kwargs'])
+    "HttpQuery", ["username", "url_base", "json_fields", "expected_rows", "kwargs"]
+)
 
-settings.SITE.is_testing = True
+# settings.SITE.is_testing = True
 
-def get_json_dict(username, uri, an='detail', **kwargs):
-    url = '/api/{0}?fmt=json&an={1}'.format(uri, an)
-    for k, v in kwargs.items():
-        url += "&{}={}".format(k, v)
+ADMIN_FRONT_END = settings.SITE.kernel.editing_front_end
+
+
+def get_json_dict(username, uri, an="detail", **kwargs):
+    url = "/api/{0}?fmt=json&an={1}".format(uri, an)
+    if kwargs:
+        url += "&" + urlencode(kwargs, True)
+    # for k, v in kwargs.items():
+    #     url += "&{}={}".format(k, v)
     test_client.force_login(rt.login(username).user)
     res = test_client.get(url, REMOTE_USER=username)
-    assert res.status_code == 200
-    return json.loads(res.content.decode())
+    if res.status_code != 200:
+        raise Exception(f"GET {url} got status code {res.status_code}")
+    s = res.content.decode()
+    try:
+        return json.loads(s)
+    except Exception as e:
+        raise Exception(f"GET {url} got non-JSON response:\n{s}") from None
 
 
 def get_json_soup(username, uri, fieldname, **kwargs):
@@ -71,7 +112,7 @@ def get_json_soup(username, uri, fieldname, **kwargs):
 
     """
     d = get_json_dict(username, uri, **kwargs)
-    html = d['data'][fieldname]
+    html = d["data"][fieldname]
     return beautiful_soup(html)
 
 
@@ -81,13 +122,13 @@ def beautiful_soup(html):
     This supports URIs that return a `delayed_value`.
     """
     if isinstance(html, dict):
-        uri = "/" + html['delayed_value_url']
+        uri = "/" + html["delayed_value_url"]
         res = test_client.get(uri)
         assert res.status_code == 200
         d = json.loads(res.content.decode())
-        html = d['data']
+        html = d["data"]
         # raise Exception("20230710 {}".format(html))
-    return BeautifulSoup(html, 'lxml')
+    return BeautifulSoup(html, "lxml")
 
 
 def post_json_dict(username, url, data, **extra):
@@ -100,12 +141,13 @@ def post_json_dict(username, url, data, **extra):
     test_client.force_login(rt.login(username).user)
     res = test_client.post(url, data, REMOTE_USER=username, **extra)
     if res.status_code != 200:
-        raise Exception("{} gave status code {} instead of 200".format(
-            url, res.status_code))
+        raise Exception(
+            "{} gave status code {} instead of 200".format(url, res.status_code)
+        )
     return AttrDict(json.loads(res.content.decode()))
 
 
-def check_json_result(response, expected_keys=None, msg=''):
+def check_json_result(response, expected_keys=None, msg=""):
     """Checks the result of response which is expected to return a
     JSON-encoded dictionary with the expected_keys.
 
@@ -114,15 +156,18 @@ def check_json_result(response, expected_keys=None, msg=''):
     if response.status_code != 200:
         raise Exception(
             "Response status ({0}) was {1} instead of 200".format(
-                msg, response.status_code))
+                msg, response.status_code
+            )
+        )
     try:
         result = json.loads(response.content.decode())
     except ValueError as e:
         raise Exception("{0} in {1}".format(e, response.content))
     if expected_keys is not None:
         if set(result.keys()) != set(expected_keys.split()):
-            raise Exception("'{0}' != '{1}'".format(
-                ' '.join(list(result.keys())), expected_keys))
+            raise Exception(
+                "'{0}' != '{1}'".format(" ".join(list(result.keys())), expected_keys)
+            )
     return result
 
 
@@ -133,24 +178,25 @@ def demo_get(username, url_base, json_fields=None, expected_rows=None, **kwargs)
     url = str(settings.SITE.buildurl(case.url_base, **case.kwargs))
     # print(20160329, url)
     if False:
-        msg = 'Using remote authentication, but no user credentials found.'
+        msg = "Using remote authentication, but no user credentials found."
         try:
             response = test_client.get(url)
             raise Exception("Expected '%s'" % msg)
         except Exception:
             pass
-            #~ self.tc.assertEqual(str(e),msg)
-            #~ if str(e) != msg:
-                    #~ raise Exception("Expected %r but got %r" % (msg,str(e)))
+            # ~ self.tc.assertEqual(str(e),msg)
+            # ~ if str(e) != msg:
+            # ~ raise Exception("Expected %r but got %r" % (msg,str(e)))
 
     if False:
         # removed 20161202 because (1) it was relatively useless and
         # (2) caused a PermissionDenied warning
-        response = test_client.get(url, REMOTE_USER=str('foo'))
+        response = test_client.get(url, REMOTE_USER=str("foo"))
         if response.status_code != 403:
             raise Exception(
-                "Status code %s other than 403 for anonymous on GET %s" % (
-                    response.status_code, url))
+                "Status code %s other than 403 for anonymous on GET %s"
+                % (response.status_code, url)
+            )
     ses = rt.login(username)
     test_client.force_login(ses.user)
     response = test_client.get(url, REMOTE_USER=username)
@@ -170,9 +216,8 @@ def demo_get(username, url_base, json_fields=None, expected_rows=None, **kwargs)
         else:
             if not isinstance(num, tuple):
                 num = [num]
-            if result['count'] not in num:
-                msg = "%s got %s rows instead of %s" % (
-                    url, result['count'], num)
+            if result["count"] not in num:
+                msg = "%s got %s rows instead of %s" % (url, result["count"], num)
                 raise Exception(msg)
 
     # except Exception as e:
@@ -189,7 +234,7 @@ def show_menu_path(spec, language=None):
     automatically sets the language of the user and works for any user
     type.
     """
-    user_type = rt.models.users.UserTypes.get_by_value('900')
+    user_type = rt.models.users.UserTypes.get_by_value("900")
     mi = user_type.find_menu_item(spec)
     if mi is None:
         raise Exception("Invalid spec {0}".format(spec))
@@ -214,7 +259,7 @@ def noblanklines(s):
     itself.
 
     """
-    return '\n'.join([ln for ln in s.splitlines() if ln.strip()])
+    return "\n".join([ln for ln in s.splitlines() if ln.strip()])
 
 
 def show_choices(username, url, show_count=False):
@@ -228,122 +273,65 @@ def show_choices(username, url, show_count=False):
     if response.status_code != 200:
         raise Exception(
             "Response status ({0}) was {1} instead of 200".format(
-                url, response.status_code))
+                url, response.status_code
+            )
+        )
 
     result = json.loads(response.content.decode())
-    for r in result['rows']:
-        print(r['text'])
+    for r in result["rows"]:
+        print(r["text"])
         # print(r['value'], r['text'])
     if show_count:
-        print("{} rows".format(result['count']))
+        print("{} rows".format(result["count"]))
 
 
-from django.db.models import Model
-from lino.core.actions import Action
-from lino.core.tables import AbstractTable
-from lino.core.boundaction import BoundAction
-
 def show_workflow(actions, all=False, language=None):
     """
     Show the given actions as a table.  Usage example in
     :ref:`avanti.specs.cal`.
 
     """
+
     def doit():
         cells = []
-        cols = ["Action name", "Verbose name", "Help text",
-                "Target state", "Required states"]  # , "Required roles"]
+        cols = [
+            "Action name",
+            "Verbose name",
+            "Help text",
+            "Target state",
+            "Required states",
+        ]  # , "Required roles"]
         for a in actions:
-            ht = a.help_text or ''
+            ht = a.help_text or ""
             if ht or all:
                 # required_roles = ' '.join(
                 #     [str(r) for r in a.required_roles])
                 cells.append(
-                    [a.action_name, a.label, unindent(ht),
-                     a.target_state, a.required_states or '',
-                     # required_roles
-                    ])
-        print(table(cols, cells).strip())
+                    [
+                        a.action_name,
+                        a.label,
+                        unindent(ht),
+                        a.target_state,
+                        a.required_states or "",
+                        # required_roles
+                    ]
+                )
+        print(rstgen.table(cols, cells).strip())
 
     if language:
         with translation.override(language):
             return doit()
     return doit()
 
-def get_fields(model, fieldnames=None, columns=None):
-
-    if isinstance(model, BoundAction):
-        get_field = model.action.parameters.get
-        if fieldnames is None:
-            fieldnames = model.action.params_layout
-    elif isinstance(model, Action):
-        get_field = model.parameters.get
-        if fieldnames is None:
-            fieldnames = model.params_layout.main
-    elif issubclass(model, Model):
-        get_field = model._meta.get_field
-        # get_field = model.get_data_elem
-        if fieldnames is None:
-            fieldnames = [f.name for f in model._meta.get_fields()]
-    elif issubclass(model, AbstractTable):
-        if columns:
-            get_field = model.get_data_elem
-            if fieldnames is None:
-                fieldnames = model.column_names
-                # get_handle().list_layout.main.columns
-        else:
-            get_field = model.parameters.get
-            if fieldnames is None:
-                if not isinstance(model.params_layout, BaseLayout):
-                    register_params(model)
-                fieldnames = model.params_layout.main
-    if fieldnames is None:
-        return
-    if isinstance(fieldnames, str):
-        fieldnames = fieldnames.split()
-    for n in fieldnames:
-        yield get_field(n)
-
-
-def fields_help(model, fieldnames=None, columns=False, all=None):
-    """
-    Print an overview description of the specified fields of the
-    specified model.
-
-    If model is an action or table, print the parameter fields of that
-    action or table.
-
-    If model is a table and you want the columns instead of the
-    parameter fields, then specify `columns=True`.
-
-    By default this shows only fields that have a help text.  If you
-    specify `all=True`, then also fields that have no help text will
-    be shown.
-    """
-    if all is None:
-        all = fieldnames is not None
-
-    cells = []
-    # cols = ["Internal name", "Verbose name", "Help text"]
-    for fld in get_fields(model, fieldnames, columns):
-        if fld is not None and hasattr(fld, 'verbose_name'):
-            ht = fld.help_text or ''
-            if ht or all:
-                cells.append([fld.name,
-                              fld.verbose_name,
-                              unindent(ht)])
-
-    # return table(cols, cells).strip()
-    items = ["{} ({}) : {}".format(row[1], row[0], row[2]) for row in cells]
-    return ul(items).strip()
 
 def show_fields(*args, **kwargs):
     print(fields_help(*args, **kwargs))
 
+
 def show_fields_by_type(fldtype):
-    """Print a list of all fields (in all models) that have the specified type.
-    """
-    from lino.core.utils import (sorted_models_list)
+    """Print a list of all fields (in all models) that have the specified type."""
+    from lino.core.utils import sorted_models_list
+
     items = []
     for model in sorted_models_list():
         flds = []
@@ -355,15 +343,14 @@ def show_fields_by_type(fldtype):
                 flds.append(txt)
         if len(flds):
             txt = "{model} : {fields}".format(
-                model=full_model_name(model), fields=", ".join(flds))
+                model=full_model_name(model), fields=", ".join(flds)
+            )
             items.append(txt)
     print(rstgen.ul(items))
 
 
 def show_columns(*args, **kwargs):
-    """Like :func:`show_fields` but with `columns` defaulting to True.
-
-    """
+    """Like :func:`show_fields` but with `columns` defaulting to True."""
     kwargs.update(columns=True)
     return show_fields(*args, **kwargs)
 
@@ -372,16 +359,24 @@ def py2rst(x, doctestfmt=True):
     return diag.py2rst(x, doctestfmt)
 
 
-def show_dialog_actions():
-    return analyzer.show_dialog_actions(True)
+# def show_dialog_actions():
+#     return analyzer.show_dialog_actions(True)
+
+
+# def show_db_overview():
+#     print(analyzer.show_db_overview())
 
 
 def walk_menu_items(username=None, severe=True):
-    """Walk through all menu items which run a :class:`ShowTable
-    <lino.core.actions.ShowTable>` action, showing how many data rows
-    the grid contains.
+    """
+
+    Print a list of all :term:`application menu` items and, for items that open
+    a :term:`data table`, the number of data rows the grid contains.
 
     """
+
+    renderer = settings.SITE.kernel.default_renderer
+
     def doit(ar):
         if ar is None:
             user_type = None
@@ -391,32 +386,58 @@ def walk_menu_items(username=None, severe=True):
         mnu = settings.SITE.get_site_menu(user_type)
         items = []
         for mi in mnu.walk_items():
-          if mi.bound_action:
-            item = menuselection_text(mi) + " : "
-            if isinstance(mi.bound_action.action, ShowTable) and not mi.params:
-                mt = mi.bound_action.actor
-                url = 'api/{}/{}'.format(mt.app_label, mt.__name__)
-                url = str(settings.SITE.buildurl(url, fmt='json'))
-                try:
-                    response = test_client.get(url, REMOTE_USER=str(username))
-                    result = check_json_result(
-                        response, None,
-                        "GET %s for user %s" % (url, username))
-                    item += str(result['count'])
-                except Exception as e:
-                    if severe:
-                        raise
+            if ba := mi.bound_action:
+                item = menuselection_text(mi)
+                # item += " ({})".format(mi)
+                item += " : "
+                if isinstance(ba.action, ShowTable) and not mi.params:
+                    # url = settings.SITE.kernel.default_ui.renderer.request_handler()
+                    # sar = ba.request(parent=ar)
+                    if False:
+                        url = ar.get_permalink(ba, mi.instance)
                     else:
-                        item += str(e)
-            else:
-                item += "(not tested)"
-            items.append(item)
+                        mt = ba.actor
+                        baseurl = "api/{}/{}".format(mt.app_label, mt.__name__)
+                        kwargs = dict(fmt="json")
+                        # url = settings.SITE.buildurl(baseurl, **kwargs)
+                        url = ADMIN_FRONT_END.build_plain_url(baseurl, **kwargs)
+
+                    try:
+                        response = test_client.get(url)
+                        # response = test_client.get(url,
+                        #                            REMOTE_USER=str(username))
+                        result = check_json_result(
+                            response, None, f"GET {url} for user {username}")
+                        item += str(result["count"])
+                        # Also ask in display_mode "list" to cover assert_safe() bugs.
+                        # But not e.g. UserRoles because it's not on a model and doesn't have a list mode
+                        if mt.model:
+                            kwargs[
+                                constants.URL_PARAM_DISPLAY_MODE
+                            ] = constants.DISPLAY_MODE_LIST
+                            url = ADMIN_FRONT_END.build_plain_url(baseurl, **kwargs)
+                            response = test_client.get(url)
+                            result = check_json_result(
+                                response, None, f"GET {url} for user {username}"
+                            )
+                            # if ar is not None:
+                            #     sar = mt.request(parent=ar)
+                            #     print(renderer.show_table(sar, display_mode="list"))
+                            #     # sar.show(mt, display_mode="list")
+                    except Exception as e:
+                        if severe:
+                            raise
+                        else:
+                            item += str(e)
+                else:
+                    item += "(not tested)"
+                items.append(item)
 
         s = rstgen.ul(items)
         print(s)
 
     if settings.SITE.user_types_module:
-        ar = settings.SITE.login(username)
+        ar = rt.login(username)
         with translation.override(ar.user.language):
             doit(ar)
     else:
@@ -430,8 +451,8 @@ def show_sql_queries():
     Usage example: :ref:`specs.noi.sql`.
     """
     for qry in connection.queries:
-        sql = qry['sql'].strip()
-        print(sql.replace('"', ''))
+        sql = qry["sql"].strip()
+        print(sql.replace('"', ""))
     # reset_sql_queries()
 
 
@@ -442,6 +463,7 @@ def show_sql_summary(**kwargs):
     Usage example: :ref:`specs.tera.sql`.
 
     """
+
     def func():
         for qry in connection.queries:
             try:
@@ -460,11 +482,14 @@ def add_call_logger(owner, name):
     """
     func = getattr(owner, name)
     msg = "{}() on {} was called".format(name, owner)
+
     def w(*args, **kwargs):
         print(msg)
         return func(*args, **kwargs)
+
     setattr(owner, name, w)
 
+
 def str2languages(txt):
     """
     Return a list of all translations for the given translatable text.
@@ -475,6 +500,7 @@ def str2languages(txt):
             lst.append(str(txt))
     return lst
 
+
 def show_choicelist(cls):
     """
     Similar to :func:`rt.show`, but the `text` is shown in all
@@ -485,23 +511,43 @@ def show_choicelist(cls):
     for i in cls.get_list_items():
         row = [i.value, i.name] + str2languages(i.text)
         rows.append(row)
-    print(table(headers, rows))
+    print(rstgen.table(headers, rows))
+
 
 def show_choicelists():
     """
     Show all the choicelists defined in this application.
     """
-    headers = ["name", "#items", "preferred_width"] + [lng.name for lng in settings.SITE.languages]
+    headers = ["name", "#items", "preferred_width"] + [
+        lng.name for lng in settings.SITE.languages
+    ]
     rows = []
     for i in sorted(kernel.CHOICELISTS.values(), key=lambda s: str(s)):
-        row = [str(i), len(i.choices), i.preferred_width] + str2languages(i.verbose_name_plural)
+        row = [str(i), len(i.choices), i.preferred_width] + str2languages(
+            i.verbose_name_plural
+        )
         rows.append(row)
-    print(table(headers, rows))
+    print(rstgen.table(headers, rows))
+
+
+def show_parent_layouts():
+    """
+    Show all actors having a parent layout.
+    """
+    headers = ["actor", "is used in"]
+    rows = []
+    for a in actors.actors_list:
+        if issubclass(a, AbstractTable) and a.parent_layout:
+            parent = a.parent_layout._datasource
+            row = [str(a), str(parent)]  # , parent.__module__]
+            rows.append(row)
+    print(rstgen.table(headers, sorted(rows)))
 
 
 def show_permissions(*args):
     print(visible_for(*args))
 
+
 def show_translations(things, fmt, languages=None):
     if languages is None:
         languages = [lng.name for lng in settings.SITE.languages]
@@ -517,13 +563,19 @@ def show_translations(things, fmt, languages=None):
                 x, txt = fmt(thing)
                 cells.append(txt)
         rows.append(cells)
-    print(table(headers, rows))
+    print(rstgen.table(headers, rows))
+
 
 def show_model_translations(*models, **kwargs):
     def fmt(m):
-        return (full_model_name(m), "{0.verbose_name}\n{0.verbose_name_plural}".format(m._meta))
+        return (
+            full_model_name(m),
+            "{0.verbose_name}\n{0.verbose_name_plural}".format(m._meta),
+        )
+
     show_translations(models, fmt, **kwargs)
 
+
 def show_field_translations(model, fieldnames=None, wrap_width=40, **kwargs):
     """
     Print a table with the verbose name and the help text of the specified
@@ -535,62 +587,61 @@ def show_field_translations(model, fieldnames=None, wrap_width=40, **kwargs):
     :term:`language distribution` of this :term:`Lino site`.
 
     """
+
     def fmt(fld):
-        if fld.help_text:
+        if getattr(fld, "help_text", None):
             txt = "{} : {}".format(fld.verbose_name, fld.help_text)
         else:
-            txt = str(fld.verbose_name)
-        txt = '\n'.join(textwrap.wrap(txt, wrap_width))
+            txt = str(getattr(fld, "verbose_name", fld.__class__))
+        txt = "\n".join(textwrap.wrap(txt, wrap_width))
         return (fld.name, txt)
+
     show_translations(get_fields(model, fieldnames), fmt, **kwargs)
 
 
-
-
 def show_quick_search_fields(*args):
     for m in args:
         print(str(m._meta.verbose_name_plural))
         for fld in m.quick_search_fields:
             print("- {} ({})".format(fld.verbose_name, fld.name))
 
+
 def pprint_json_string(s):
     """
     Used to doctest json values and have them be python 2/3 passable.
     :param s: json string
 
     """
-    print(json.dumps(json.loads(s),
-                     indent=2,
-                     sort_keys=True,
-                     separators=(",", ": ")))
+    print(json.dumps(json.loads(s), indent=2, sort_keys=True, separators=(",", ": ")))
 
 
-def show_dashboard(username, **options):
+def show_dashboard(username, show_urls=False, **options):
     """Show the dashboard of the given user.
 
     Useful options:
 
+    - show_urls=False
     - ignore_links=True
 
     For more options, see
     https://pypi.org/project/html2text/ and
     https://github.com/Alir3z4/html2text/blob/master/docs/usage.md
 
-    Note that this is currently not much used because the result is difficult to
-    maintain.  One reason for this is that :func:`naturaltime` (from
-    :mod:`django.contrib.humanize.templatetags.humanize`) ignores demo_date and
-    therefore produces results that depend on the current date/time.
+    This is currently not much used because the result is difficult to maintain.
+    One reason for this is that :func:`naturaltime` (from
+    :mod:`django.contrib.humanize.templatetags.humanize`) ignores
+    :attr:`lino.core.site.Site.the_demo_date` and therefore produces results that
+    depend on the current date/time.
 
     """
-    # request = PseudoRequest(username)
-    rnd = settings.SITE.kernel.text_renderer
-    ar = rt.login(username, renderer=rnd)
-    html = settings.SITE.get_main_html(ar) #, front_end=rnd.front_end)
+    renderer = settings.SITE.kernel.text_renderer
+    ar = rt.login(username, renderer=renderer, show_urls=show_urls)
+    html = settings.SITE.get_main_html(ar)
     print(html2text(html, **options).strip())
 
+
 def menu2rst(mnu, level=1):
-    """Recursive utility used by :func:`show_menu`.
-    """
+    """Recursive utility used by :func:`show_menu`."""
     if not isinstance(mnu, Menu):
         return str(mnu.label)
 
@@ -600,36 +651,42 @@ def menu2rst(mnu, level=1):
             has_submenus = True
     items = [menu2rst(mi, level + 1) for mi in mnu.items]
     if has_submenus:
-        s = rstgen.ul(items).strip() + '\n'
+        s = rstgen.ul(items).strip() + "\n"
         if mnu.label is not None:
-            s = str(mnu.label) + ' :\n\n' + s
+            s = str(mnu.label) + " :\n\n" + s
     else:
-        s = ', '.join(items)
+        s = ", ".join(items)
         if mnu.label is not None:
-            s = str(mnu.label) + ' : ' + s
+            s = str(mnu.label) + " : " + s
     return s
 
 
-def show_menu(username, language=None, stripped=True, level=1):
+def show_menu(username=None, language=None, stripped=True, level=1):
     """
 
-    Render the main menu for the given user as a reStructuredText formatted
+    Print the main menu for the given user as a reStructuredText-formatted
     bullet list.
 
+    :username: the username of the user to select. If this is not specified,
+               anonymous user will be used.
     :language: explicitly select another language than that
                specified in the requesting user's :attr:`language
                <lino.modlib.users.models.User.language>` field.
-    :stripped: remove lots of blanklines which are necessary for
+    :stripped: remove lots of blanklines that are necessary for
                reStructuredText but disturbing in a doctest
                snippet.
 
     """
-    ar = rt.login(username)
-    user = ar.get_user()
-    if language is None:
-        language = user.language
+    if username is None:
+        user_type = None
+    else:
+        ar = rt.login(username)
+        user = ar.get_user()
+        user_type = user.user_type
+        if language is None:
+            language = user.language
     with translation.override(language):
-        mnu = settings.SITE.get_site_menu(user.user_type)
+        mnu = settings.SITE.get_site_menu(user_type)
         s = menu2rst(mnu, level)
         if stripped:
             for ln in s.splitlines():
@@ -652,7 +709,7 @@ def walk_store_fields(only_detail_fields=False):
             if a.get_handle_name is not None:
                 continue  # debts.PrintEntriesByBudget
             ah = a.get_handle()
-            if not hasattr(ah, 'store'):
+            if not hasattr(ah, "store"):
                 raise Exception("{} has no store".format(ah.actor))
             if only_detail_fields:
                 lst = ah.store.detail_fields
@@ -665,8 +722,108 @@ def walk_store_fields(only_detail_fields=False):
                 yield a, sf
 
 
-# settings.SITE.kernel.check_virgin()
 def set_log_level(level):
     logger.setLevel(level)
     for handler in logger.handlers:
         handler.setLevel(level)
+
+
+def printsql(qs):
+    """
+    Print the SQL query of the given Django queryset.
+    Uses sqlparse for formatting, after removing quotes around field names for
+    readability.
+    """
+    sql = str(qs.query).replace('"', "")
+    print(sqlparse.format(sql, reindent=True, keyword_case="upper"))
+
+
+def show_change_watchers():
+    """
+    Show information about the change watchers that have been installed using
+    :func:`lino.modlib.changes.utils.watch_changes`.
+    """
+    headers = ["model", "master_key", "ignored_fields"]
+    rows = []
+    for m in get_models():
+        ws = m.change_watcher_spec
+        if ws:
+            rows.append(
+                [fmn(m), ws.master_key, " ".join(sorted(ws.ignored_fields))]
+            )
+    print(rstgen.table(headers, rows, max_width=40))
+
+
+def show_display_modes():
+    """
+    Show the availble display modes per actor.
+    """
+    dml = sorted(constants.DISPLAY_MODES - constants.BASIC_DISPLAY_MODES)
+    headers = ["actor"]
+    headers += dml
+    rows = []
+    for a in sorted(actors.actors_list, key=str):
+        if not a.is_abstract():
+            rows.append(
+                [str(a)] + [
+                    ("x" if dm in a.extra_display_modes else "")
+                    for dm in dml]
+            )
+    print(rstgen.table(headers, rows))
+
+
+def show_choosers():
+    """
+    Show the availble choosers per actor.
+    """
+    headers = ["field"]
+    headers = ["field", "context_fields", "can_create_choice"]
+    rows = []
+    # for a in sorted(actors.actors_list, key=str):
+    for m in sorted(get_models(), key=full_model_name):
+        if (cd := getattr(m, "_choosers_dict", None)) is None:
+            continue
+        for fld in get_fields(m):
+            if (c := cd.get(fld.name, None)):
+                cf = ", ".join([cf.name for cf in c.context_fields])
+                rows.append([
+                    f"{full_model_name(m)}.{fld.name}",
+                    str(cf),
+                    str(c.can_create_choice)])
+    print(rstgen.table(headers, rows))
+
+
+def checkdb(m, num):
+    """
+    Raise an exception if the database doesn't contain the specified number of
+    rows of the specified model.
+
+    This is for usage in :xfile:`startup.py` scripts.
+
+    """
+    if m.objects.count() != num:
+        raise Exception(
+            f"Model {m} should have {num} rows but has {m.objects.count()}")
+
+
+def show_clonables():
+    """
+    Print a list of all :class:`Clonable <lino.mixins.clonable.Clonable>`
+    models, together with their related slaves, i.e. the data that will be
+    cloned in cascade with their master.
+    """
+    items = []
+    for m in get_models():
+        if issubclass(m, Clonable):
+            rels = []
+            if (obj := m.objects.first()) is not None:
+                new, related = obj.duplication_plan()
+                for fk, qs in related:
+                    rels.append(f"{fmn(qs.model)}.{fk.name}")
+            if len(rels):
+                x = ", ".join(rels)
+                items.append(f"{fmn(m)} : {x}")
+            else:
+                items.append(fmn(m))
+    items = sorted(items)
+    print(rstgen.ul(items).strip())
