@@ -23,17 +23,23 @@ class ModelBackend(object):
     """
 
     def authenticate(self, request, username=None, password=None, **kwargs):
+        # print("20210212 authenticate()", username)
         if settings.SITE.use_ipdict:
             ipdict = settings.SITE.plugins.ipdict
-            rec = ipdict.get_ip_record(request, 'anonymous')
+            rec = ipdict.get_ip_record(request)
+            # print("20210212 authenticate()", rec)
             if rec.blacklisted_since is not None:
                 since = datetime.now() - rec.blacklisted_since
+                # print("20210212 blacklisted since {} ({} < {}?)".format(
+                #     rec.blacklisted_since, since, ipdict.max_blacklist_time))
+                rec.blacklisted_since = datetime.now()
                 if since < ipdict.max_blacklist_time:
-                    msg = _("Blacklisted IP {} : contact your "
-                            "system administrator")
+                    msg = _("Too many authentication failures from {}").format(rec.addr)
                     # block this request, don't ask other backends
-                    raise PermissionDenied(msg.format(rec.addr))
+                    # print("20210212", msg)
+                    raise Warning(msg)
                     # return msg.format(rec.addr)
+                    # return
 
         UserModel = get_user_model()
 
@@ -54,16 +60,17 @@ class ModelBackend(object):
         Reject users with is_active=False. Custom user models that don't have
         that attribute are allowed.
         """
-        is_active = getattr(user, 'is_active', None)
+        is_active = getattr(user, "is_active", None)
         return is_active or is_active is None
 
     def _get_user_permissions(self, user_obj):
         return user_obj.user_permissions.all()
 
     def _get_group_permissions(self, user_obj):
-        user_groups_field = get_user_model()._meta.get_field('groups')
-        user_groups_query = 'group__%s' % user_groups_field.related_query_name()
+        user_groups_field = get_user_model()._meta.get_field("groups")
+        user_groups_query = "group__%s" % user_groups_field.related_query_name()
         from django.contrib.auth.models import Permission
+
         return Permission.objects.filter(**{user_groups_query: user_obj})
 
     def _get_permissions(self, user_obj, obj, from_name):
@@ -76,14 +83,19 @@ class ModelBackend(object):
             return set()
 
         from django.contrib.auth.models import Permission
-        perm_cache_name = '_%s_perm_cache' % from_name
+
+        perm_cache_name = "_%s_perm_cache" % from_name
         if not hasattr(user_obj, perm_cache_name):
             if user_obj.is_superuser:
                 perms = Permission.objects.all()
             else:
-                perms = getattr(self, '_get_%s_permissions' % from_name)(user_obj)
-            perms = perms.values_list('content_type__app_label', 'codename').order_by()
-            setattr(user_obj, perm_cache_name, set("%s.%s" % (ct, name) for ct, name in perms))
+                perms = getattr(self, "_get_%s_permissions" % from_name)(user_obj)
+            perms = perms.values_list("content_type__app_label", "codename").order_by()
+            setattr(
+                user_obj,
+                perm_cache_name,
+                set("%s.%s" % (ct, name) for ct, name in perms),
+            )
         return getattr(user_obj, perm_cache_name)
 
     def get_user_permissions(self, user_obj, obj=None):
@@ -91,19 +103,19 @@ class ModelBackend(object):
         Returns a set of permission strings the user `user_obj` has from their
         `user_permissions`.
         """
-        return self._get_permissions(user_obj, obj, 'user')
+        return self._get_permissions(user_obj, obj, "user")
 
     def get_group_permissions(self, user_obj, obj=None):
         """
         Returns a set of permission strings the user `user_obj` has from the
         groups they belong.
         """
-        return self._get_permissions(user_obj, obj, 'group')
+        return self._get_permissions(user_obj, obj, "group")
 
     def get_all_permissions(self, user_obj, obj=None):
         if not user_obj.is_active or user_obj.is_anonymous or obj is not None:
             return set()
-        if not hasattr(user_obj, '_perm_cache'):
+        if not hasattr(user_obj, "_perm_cache"):
             user_obj._perm_cache = self.get_user_permissions(user_obj)
             user_obj._perm_cache.update(self.get_group_permissions(user_obj))
         return user_obj._perm_cache
@@ -120,7 +132,7 @@ class ModelBackend(object):
         if not user_obj.is_active:
             return False
         for perm in self.get_all_permissions(user_obj):
-            if perm[:perm.index('.')] == app_label:
+            if perm[: perm.index(".")] == app_label:
                 return True
         return False
 
@@ -172,9 +184,9 @@ class RemoteUserBackend(ModelBackend):
         # instead we use get_or_create when creating unknown users since it has
         # built-in safeguards for multiple threads.
         if self.create_unknown_user:
-            user, created = UserModel._default_manager.get_or_create(**{
-                UserModel.USERNAME_FIELD: username
-            })
+            user, created = UserModel._default_manager.get_or_create(
+                **{UserModel.USERNAME_FIELD: username}
+            )
             if created:
                 user = self.configure_user(user)
         else:
