@@ -1,13 +1,18 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2016-2021 Rumma & Ko Ltd
+# Copyright 2016-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """Defines the :class:`DashboardItem` class.
 
 """
 
-from lino.api import _
+# from django.conf import settings
+# from lino.api import _
 from lino.core.permissions import Permittable
-from etgen.html import E, tostring
+from lino.core.requests import ActionRequest
+from lino.core.tables import AbstractTable
+# from lino.core.actions import ShowTable
+from lino.utils.html import E, tostring, assert_safe
+from django.utils.html import mark_safe
 
 
 class DashboardItem(Permittable):
@@ -51,8 +56,6 @@ class DashboardItem(Permittable):
         This is a helper function for shared use by :class:`ActorItem`
         and :class:`RequestItem`.
         """
-        from lino.core.tables import TableRequest
-        from lino.core.requests import ActionRequest
         assert ar.user is sar.user
         # T = sar.actor
         # print("20210112 render_request()", sar.actor, sar)
@@ -60,46 +63,48 @@ class DashboardItem(Permittable):
             if sar.get_total_count() < self.min_count:
                 # print("20180212 render no rows in ", sar)
                 return
-        yield '<div class="dashboard-item">'
+        yield mark_safe('<div class="dashboard-item">')
         if self.header_level is not None:
             buttons = sar.plain_toolbar_buttons()
-            buttons.append(sar.open_in_own_window_button())
+            # 20250713 Maybe add the ⏏ button already in plain_toolbar_buttons()
+            # buttons.append(sar.open_in_own_window_button())
             elems = []
             for b in buttons:
                 elems.append(b)
-                elems.append(' ')
-            yield tostring(E.h2(str(sar.actor.get_title_base(sar)), ' ', *elems))
+                elems.append(" ")
+            yield tostring(E.h2(str(sar.actor.get_title_base(sar)), " ", *elems))
 
         assert sar.renderer is not None
-        if isinstance(sar, TableRequest):
-
+        if isinstance(sar, ActionRequest):
             # TODO 20220930 until now, dashboard was always acting as if
-            # display_mode was DISPLAY_MODE_TABLE
-            # if sar.actor.display_mode == constants.DISPLAY_MODE_TABLE
-
-            for e in sar.renderer.table2story(sar, **kwargs):
-                yield tostring(e)
-        elif isinstance(sar, ActionRequest):
-            # example : courses.StatusReport in dashboard
-            for e in sar.renderer.show_story(ar, sar.actor.get_story(None, ar), **kwargs):
-                yield tostring(e)
+            # display_mode was DISPLAY_MODE_GRID
+            if issubclass(sar.actor, AbstractTable):
+                # if isinstance(sar.bound_action.action, ShowTable):
+                for e in sar.renderer.table2story(sar, **kwargs):
+                    # assert_safe(tostring(e))
+                    yield tostring(e)
+            else:
+                # example : courses.StatusReport in dashboard
+                yield sar.renderer.show_story(
+                    ar, sar.actor.get_story(None, ar), **kwargs)
+                # for e in sar.renderer.show_story(
+                #     ar, sar.actor.get_story(None, ar), **kwargs):
+                #     # assert_safe(tostring(e))
+                #     yield tostring(e)
         else:
-            yield "Cannot render {}".format(sar)
-        yield '</div>'
-
+            raise Exception("20240908 Cannot render {}".format(sar))
+            # yield "Cannot render {}".format(sar)
+        yield mark_safe("</div>")
 
     def serialize(self):
-        return dict(
-            name=self.name,
-            header_level=self.header_level
-        )
+        return dict(name=self.name, header_level=self.header_level)
 
     def __repr__(self):
         return f"{self.__class__.__module__}.{self.__class__.__name__}({self.name},header_level={self.header_level},min_count={self.min_count})"
 
 
 class ActorItem(DashboardItem):
-    """A dashboard item which simply renders a given actor.
+    """A dashboard item that simply renders a given actor.
     The actor should be a table, other usage is untested.
 
     Usage examples:
@@ -111,11 +116,15 @@ class ActorItem(DashboardItem):
         The header level.
 
     """
+
     def __init__(self, actor, **kwargs):
         self.actor = actor
-        super(ActorItem, self).__init__(str(actor), **kwargs)
+        super().__init__(str(actor), **kwargs)
 
     def get_view_permission(self, user_type):
+        # if settings.SITE.is_hidden_plugin(self.actor.app_label):
+        if self.actor.abstract:
+            return False
         return self.actor.default_action.get_view_permission(user_type)
         # rv = self.actor.default_action.get_view_permission(user_type)
         # print("20210112 get_view_permission", self.actor, rv)
@@ -141,35 +150,36 @@ class ActorItem(DashboardItem):
         # from lino.core.tables import AbstractTable
         T = self.actor
         # if isinstance(T, AbstractTable):
-        sar = T.request(limit=T.preview_limit, parent=ar)
+        # , renderer=settings.SITE.kernel.default_renderer)
+        sar = T.create_request(limit=T.preview_limit, parent=ar)
         # sar = ar.spawn(T, limit=T.preview_limit)
         # sar = ar.spawn_request(actor=T, limit=T.preview_limit)
         # raise Exception("20230331 {}".format(ar.subst_user))
 
+        # print("20250714 render()", sar.limit)
         # print("20210112 render()", ar, sar, ar.get_user(), sar.get_user())
 
         for i in self.render_request(ar, sar, **kwargs):
             yield i
-        # return
-        # for e in T.get_story(None, ar):
-        #     yield tostring(e)
 
     def serialize(self):
-        d = super(ActorItem, self).serialize()
+        d = super().serialize()
         d.update(actor=self.actor.actor_id)
         return d
 
 
-class RequestItem(DashboardItem):
+# class RequestItem(DashboardItem):
+class RequestItem(ActorItem):
     """
     Experimentally used in `lino_book.projects.events`.
     """
+
     def __init__(self, sar, **kwargs):
         self.sar = sar
-        super(RequestItem, self).__init__(None, **kwargs)
+        super().__init__(sar.actor, **kwargs)
 
     def get_view_permission(self, user_type):
-        return  self.sar.get_permission()
+        return self.sar.get_permission()
         # rv = self.sar.get_permission()
         # print("20210112 get_view_permission", self.sar, rv)
         # return rv
