@@ -1,10 +1,21 @@
-# Copyright 2009-2022 Rumma & Ko Ltd
+# Copyright 2009-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 This defines the :class:`Table` class.
 """
 
+from lino.core.gfks import ContentType, GenericForeignKey
+from lino.core.tables import AbstractTable
+from lino.core.utils import resolve_model, get_field, UnresolvedModel
+from .utils import models_by_base
+from lino.core.choicelists import ChoiceListField
+from lino.core import constants
+from lino.core import actors
+from lino.core.model import Model
+from lino.core import actions
+from lino.core import fields
+from lino.core.atomizer import fields_list
+
 from lino import logger
 
 import datetime
@@ -13,37 +24,25 @@ from django.conf import settings
 from django.core.exceptions import FieldDoesNotExist
 from django.db.models.fields import NOT_PROVIDED
 from django.utils.translation import gettext_lazy as _
+from django.utils.html import format_html, mark_safe, SafeString
 
 from django.db import models
 from django.db.models.query import QuerySet
 
-from django.apps import apps ; get_models = apps.get_models
+from django.apps import apps
 
-from etgen.html import tostring
+get_models = apps.get_models
 
-from lino.core import fields
-from lino.core import actions
-from lino.core.model import Model
-from lino.core import actors
-from lino.core import constants
-from lino.core.tablerequest import TableRequest
 
-from lino.core.choicelists import ChoiceListField
-from .utils import models_by_base
 # from .fields import get_data_elem_from_model
 
 
-from lino.core.utils import resolve_model, get_field, UnresolvedModel
-from lino.core.tables import AbstractTable, TableRequest
-from lino.core.gfks import ContentType, GenericForeignKey
-
-
 INVALID_MK = "Invalid master_key '{0}' in {1} : {2}"
 
 
 def base_attrs(cl):
     # if cl is Table or len(cl.__bases__) == 0:
-        # return
+    # return
     # myattrs = set(cl.__dict__.keys())
     for b in cl.__bases__:
         for k in base_attrs(b):
@@ -55,7 +54,7 @@ def base_attrs(cl):
 def add_gridfilters(qs, gridfilters):
     """Converts a `filter` request in the format used by
     :extux:`Ext.ux.grid.GridFilters` into a `Django field lookup
-    <https://docs.djangoproject.com/en/5.0/ref/models/querysets/#field-lookups>`_
+    <https://docs.djangoproject.com/en/5.2/ref/models/querysets/#field-lookups>`_
     on a :class:`django.db.models.query.QuerySet`.
 
     :param qs: the queryset to be modified.
@@ -64,21 +63,22 @@ def add_gridfilters(qs, gridfilters):
 
     """
     if not isinstance(qs, QuerySet):
-        raise NotImplementedError('TODO: filter also simple lists')
+        raise NotImplementedError("TODO: filter also simple lists")
     q = models.Q()
     # logger.info("20160610 %s", gridfilters)
     # raise Exception("20160610 %s" % gridfilters)
     for flt in gridfilters:
-        field = get_field(qs.model, flt.get('field', None) or flt.get("property"))
-        flttype = flt['type']
+        field = get_field(qs.model, flt.get("field", None) or flt.get("property"))
+        flttype = flt["type"]
         kw = {}
-        if flttype == 'string':
-            if isinstance(field, models.CharField):
-                kw[field.name + "__icontains"] = flt['value']
+        if flttype == "string":
+            if isinstance(field, (models.CharField, models.FileField)):
+                kw[field.name + "__icontains"] = flt["value"]
                 q = q & models.Q(**kw)
             elif isinstance(field, models.ForeignKey):
                 qf = field.remote_field.model.quick_search_filter(
-                    flt['value'], prefix=field.name + "__")
+                    flt["value"], prefix=field.name + "__"
+                )
                 # logger.info("20160610 %s %s", field.remote_field.model, qf)
                 q = q & qf
                 # rq = models.Q()
@@ -86,34 +86,34 @@ def add_gridfilters(qs, gridfilters):
                 # for search_field in field.remote_field.model.quick_search_fields:
                 # search_field = getattr(field.rel.model,'grid_search_field',None)
                 # if search_field is not None:
-                    # rq = rq | models.Q(**{field.name+"__%s__icontains" % search_field : flt['value']})
+                # rq = rq | models.Q(**{field.name+"__%s__icontains" % search_field : flt['value']})
                 # q = q & rq
             else:
                 raise NotImplementedError(repr(flt))
-        elif flttype == 'numeric':
-            cmp = str(flt.get('comparison', None) or flt.get('operator', None))
-            if cmp == 'eq':
-                cmp = 'exact'
-            kw[field.name + "__" + cmp] = flt['value']
+        elif flttype == "numeric":
+            cmp = str(flt.get("comparison", None) or flt.get("operator", None))
+            if cmp == "eq":
+                cmp = "exact"
+            kw[field.name + "__" + cmp] = flt["value"]
             q = q & models.Q(**kw)
-        elif flttype == 'boolean':
-            kw[field.name] = flt['value']
+        elif flttype == "boolean":
+            kw[field.name] = flt["value"]
             # kw[field.name + "__equals"] = flt['value']
             q = q & models.Q(**kw)
-        elif flttype == 'date':
-            v = datetime.date(*settings.SITE.parse_date(flt['value']))
+        elif flttype == "date":
+            v = datetime.date(*settings.SITE.parse_date(flt["value"]))
             # v = parse_js_date(flt['value'],field.name)
-            cmp = str(flt.get('comparison', None) or flt.get('operator', None))
-            if cmp == 'eq':
-                cmp = 'exact'
+            cmp = str(flt.get("comparison", None) or flt.get("operator", None))
+            if cmp == "eq":
+                cmp = "exact"
             kw[field.name + "__" + cmp] = v
             q = q & models.Q(**kw)
             # print kw
-        elif flttype == 'list':
+        elif flttype == "list":
             if isinstance(field, ChoiceListField):
                 choices = []
                 for x in field.choices:
-                    if x[1] in flt['value']:
+                    if x[1] in flt["value"]:
                         choices.append(x[0])
                 kw[field.name + "__in"] = choices
                 q = q & models.Q(**kw)
@@ -125,19 +125,19 @@ def add_gridfilters(qs, gridfilters):
 
 
 def rc_name(rptclass):
-    return rptclass.app_label + '.' + rptclass.__name__
+    return rptclass.app_label + "." + rptclass.__name__
 
 
-
-def has_fk(rr, name):
-    if isinstance(rr, TableRequest):
-        return rr.actor.master_key == name
+def has_fk(ar, name):
+    if isinstance(ar, ActionRequest) and ar.actor is not None:
+        return ar.actor.master_key == name
     return False
 
 
 def model2actor(m):
     def f(table, *args, **kwargs):
         return m(*args, **kwargs)
+
     return classmethod(f)
 
 
@@ -173,7 +173,7 @@ class Table(AbstractTable):
 
     show_detail_navigator = True
 
-    screenshot_profiles = ['admin']
+    screenshot_profiles = ["admin"]
     """
     The user user_type(s) for which we want a screenshot of this table.
     """
@@ -184,6 +184,13 @@ class Table(AbstractTable):
 
     """
 
+    live_panel_update = False
+    """
+    Whether the :term:`front end` should update panels on this table when  the
+    underlying database has been modified. This doesn't work unless there is a
+    :manage:`linod` running. Currently works only under the React front end.
+    """
+
     expand_memos = False
     """(No longer used; see :srcref:`docs/tickets/44`).  Whether multi-line text
     fields in Grid views should be expanded in by default or not.
@@ -194,27 +201,22 @@ class Table(AbstractTable):
     """If when in a detail view will override default param values with the detail's pv values.
     """
 
-    react_responsive = True
-    """If viewing the grid view on a mobile, should the grid use responsive mode
-    Default: True"""
-
-    #react_big_search = False
+    # react_big_search = False
     """If True will position the quick search input to the bottom of the header and have it full width.
     Default: False"""
 
-    @classmethod
-    def init_layouts(cls):
-        # if cls.list_layout is None:
-        if cls.model is not None and issubclass(cls.model, Model):
-            if not cls.abstract:
-                for tbl in cls.__mro__:
-                    # print("20210629", tbl)
-                    if 'list_layout' in tbl.__dict__:
-                        break
-                    if issubclass(tbl, actors.Actor):
-                        cls.list_layout = 'list_item'
-                        break
-        super(Table, cls).init_layouts()
+    # @classmethod
+    # def init_layouts(cls):
+    #     if cls.model is not None and issubclass(cls.model, Model):
+    #         if not cls.abstract:
+    #             for tbl in cls.__mro__:
+    #                 # print("20210629", tbl)
+    #                 if "list_layout" in tbl.__dict__:
+    #                     break
+    #                 if issubclass(tbl, actors.Actor):
+    #                     cls.list_layout = "list_item"
+    #                     break
+    #     super().init_layouts()
 
     @classmethod
     def add_quick_search_filter(cls, qs, search_text):
@@ -227,21 +229,13 @@ class Table(AbstractTable):
             return qs.model.objects.none()
         return qs.filter(flt)
 
-    @classmethod
-    def get_chooser_for_field(self, fieldname):
-        ch = super(Table, self).get_chooser_for_field(fieldname)
-        if ch is not None:
-            return ch
-        if self.model is not None:
-            return self.model.get_chooser_for_field(fieldname)
-
     # @classmethod
-    # def request(self, master_instance=None, **kw):  # 20130327
-    #     kw.update(actor=self)
-    #     if master_instance is not None:
-    #         kw.update(master_instance=master_instance)
-    #     kw.setdefault('action', self.default_action)
-    #     return TableRequest(**kw)
+    # def get_chooser_for_field(self, fieldname):
+    #     ch = super().get_chooser_for_field(fieldname)
+    #     if ch is not None:
+    #         return ch
+    #     if self.model is not None:
+    #         return self.model.get_chooser_for_field(fieldname)
 
     @classmethod
     def column_choices(self):
@@ -258,18 +252,22 @@ class Table(AbstractTable):
 
         profiles2user = dict()
         for u in settings.SITE.user_model.objects.filter(language=language):
-            if u.user_type and u.user_type.name in self.screenshot_profiles and not u.user_type in profiles2user:
+            if (
+                u.user_type
+                and u.user_type.name in self.screenshot_profiles
+                and not u.user_type in profiles2user
+            ):
                 profiles2user[u.user_type] = u
         for user in list(profiles2user.values()):
             # if user.profile.name != 'admin': return
-            # yield self.default_action.request(user=user)
+            # yield self.default_action.create_request(user=user)
             # and self.default_action is not self.detail_action:
             if self.detail_action:
-                yield self.detail_action.request(user=user)
+                yield self.detail_action.create_request(user=user)
 
     # @classmethod
-    # def elem_filename_root(cls,elem):
-        # return elem._meta.app_label + '.' + elem.__class__.__name__ + '-' + str(elem.pk)
+    # def elem_get_printable_target_stem(cls,elem):
+    # return elem._meta.app_label + '.' + elem.__class__.__name__ + '-' + str(elem.pk)
     @classmethod
     def get_detail_sets(self):
         """
@@ -277,41 +275,26 @@ class Table(AbstractTable):
         should try to create a Detail Set.
         """
         if self.model is not None:
+
             def yield_model_detail_sets(m):
                 for b in m.__bases__:
                     if issubclass(b, models.Model) and b is not models.Model:
                         for ds in yield_model_detail_sets(b):
                             yield ds
-                yield m._meta.app_label + '/' + m.__name__
+                yield m._meta.app_label + "/" + m.__name__
 
             for ds in yield_model_detail_sets(self.model):
                 yield ds
 
-        for s in super(Table, self).get_detail_sets():
+        for s in super().get_detail_sets():
             yield s
 
     # @classmethod
     # def find_field(cls,model,name):
-        # for vf in cls.model._meta.virtual_fields:
-            # if vf.name == name:
-                # return vf
-        # return cls.model._meta.get_field(name)
-
-    @classmethod
-    def get_table_story(cls, obj, ar):
-        sar = cls.request(parent=ar, master_instance=obj, is_on_main_actor=False)
-        html = ""
-        for i, obj in enumerate(sar.data_iterator):
-            if i == cls.preview_limit:
-                break
-            html += obj.as_story_item(sar)
-        if cls.insert_action is not None:
-            if not cls.editable:
-                return html
-            ir = cls.insert_action.request_from(sar)
-            if ir.get_permission():
-                html = tostring(ir.ar2button()) + html
-        return html
+    # for vf in cls.model._meta.virtual_fields:
+    # if vf.name == name:
+    # return vf
+    # return cls.model._meta.get_field(name)
 
     @classmethod
     def get_pk_field(self):
@@ -326,13 +309,28 @@ class Table(AbstractTable):
         Note: `ar` may not be None.
 
         """
-        try:
-            return self.model.get_user_queryset(ar.get_user()).get(pk=pk)
-            # return self.get_queryset(ar).get(pk=pk)
-        except ValueError:
-            return None
-        except self.model.DoesNotExist:
-            return None
+        # return self.model.objects.get(pk=pk)
+        # raise Exception("20240929")
+        qs = self.get_request_queryset(ar)
+        # qs = self.model.get_request_queryset(ar)
+        # str(qs.query)
+
+        if not settings.SITE.catch_layout_exceptions:
+            return qs.get(pk=pk)
+        else:
+            try:
+                return qs.get(pk=pk)
+            except self.model.DoesNotExist:
+                # sql = qs.query
+                # import sqlparse
+                # sql = str(sql).replace('"', '')
+                # sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
+                # raise self.model.DoesNotExist(f"No row {pk} in {ar} ({sql})")
+                raise self.model.DoesNotExist(f"No row {pk} in {ar}") from None
+                # from django.core.exceptions import ObjectDoesNotExist
+                # assert isinstance(exc, ObjectDoesNotExist)
+                # raise exc
+                # return None
 
     # @classmethod
     # def disabled_actions(self, ar, obj):  # no longer used since 20170909
@@ -366,20 +364,17 @@ class Table(AbstractTable):
             # return self._label or self.__name__
             return super().get_actor_label()
         if isinstance(self.model, str):
-            print("20231104", settings.SITE.is_hidden_app(self.app_label))
+            # print("20231104", settings.SITE.is_hidden_app(self.app_label))
             raise Exception("20231104 {}.model is {!r}".format(self, self.model))
         return self._label or self.model._meta.verbose_name_plural
 
     @classmethod
     def class_init(self):
-
         if self.model is not None:
             model = resolve_model(self.model, self.app_label)
             if isinstance(model, UnresolvedModel):
                 # if settings.SITE.is_installed(self.app_label):
-                raise Exception(
-                    "Invalid model {} on table {}".format(
-                        self.model, self))
+                raise Exception("Invalid model {} on table {}".format(self.model, self))
             self.model = model
 
         if isinstance(self.model, UnresolvedModel):
@@ -388,12 +383,14 @@ class Table(AbstractTable):
         if self.model is not None:
             if isinstance(self.hidden_columns, str):
                 self.hidden_columns = frozenset(
-                    fields.fields_list(self.model, self.hidden_columns))
+                    fields_list(self.model, self.hidden_columns)
+                )
             self.hidden_columns |= self.model.hidden_columns
 
             if isinstance(self.active_fields, str):
                 self.active_fields = frozenset(
-                    fields.fields_list(self.model, self.active_fields))
+                    fields_list(self.model, self.active_fields)
+                )
             self.active_fields |= self.model.active_fields
             self.hidden_elements |= self.model.hidden_elements
 
@@ -412,19 +409,21 @@ class Table(AbstractTable):
                                 pass
                                 # logger.info("%s disables model action '%s'",self,k)
                             else:
-                                if not isinstance(
-                                        existing_value, actions.Action):
+                                if not isinstance(existing_value, actions.Action):
                                     raise Exception(
                                         "%s cannot install model action %s "
                                         "because name is already used "
-                                        "for %r" % self, k, existing_value)
+                                        "for %r" % self,
+                                        k,
+                                        existing_value,
+                                    )
 
-            for name in ('workflow_state_field', 'workflow_owner_field'):
+            for name in ("workflow_state_field", "workflow_owner_field"):
                 if getattr(self, name) is None:
                     setattr(self, name, getattr(self.model, name))
 
             for name in (  # 'disabled_fields',
-                'handle_uploaded_files',
+                "handle_uploaded_files",
                 # 'get_row_permission',
                 # 'disable_editing',
             ):
@@ -436,8 +435,15 @@ class Table(AbstractTable):
                         # 'dictproxy' object does not support item assignment:
                         # self.__dict__[name] = model2actor(m)
 
-            if self.master_key:
+            # accounting.MovementsByProject has a master_key 'project', which is a
+            # DummyField when the site has no project_model. In that case
 
+            # if self.master is not None:
+            #     self.master = resolve_model(self.master)
+
+            # For abstract tables  we don't check the master key, e.g.
+            # tickets.TicketsByGroup
+            if self.master_key and not self.abstract:
                 master_model = None
                 try:
                     fk = self.model.get_data_elem(self.master_key)
@@ -446,14 +452,17 @@ class Table(AbstractTable):
                     # assert direct
                     # assert not m2m
                     if fk is None:
-                        if hasattr(self.model, '_meta'):
-                            candidates = ', '.join(
-                                [f.name for f in self.model._meta.get_fields()])
+                        if hasattr(self.model, "_meta"):
+                            candidates = ", ".join(
+                                [f.name for f in self.model._meta.get_fields()]
+                            )
                         else:
                             candidates = "N/A"
                         raise Exception(
                             "Invalid master_key '{}' for {} ({} has fields {})".format(
-                                self.master_key, self, self.model, candidates))
+                                self.master_key, self, self.model, candidates
+                            )
+                        )
                     if fk.remote_field:
                         master_model = fk.remote_field.model
                     elif isinstance(fk, ChoiceListField):
@@ -465,7 +474,9 @@ class Table(AbstractTable):
                     else:
                         raise Exception(
                             "Unsupported master_key {0}.{1} ({2})".format(
-                                self, fk, fk.__class__))
+                                self, fk, fk.__class__
+                            )
+                        )
                 except FieldDoesNotExist:
                     for vf in self.model._meta.private_fields:
                         if vf.name == self.master_key:
@@ -476,18 +487,17 @@ class Table(AbstractTable):
                     df = getattr(self.model, self.master_key, None)
                     if df is None:
                         msg = "No field '{0}' in {1}".format(
-                            self.master_key, self.model)
-                        raise Exception(INVALID_MK.format(
-                            self.master_key, self, msg))
+                            self.master_key, self.model
+                        )
+                        raise Exception(INVALID_MK.format(self.master_key, self, msg))
                     elif isinstance(df, fields.DummyField):
                         self.abstract = True
                     else:
                         msg = "Cannot handle master key {0}".format(df)
                         msg += " (20150820 virtual fields: {0})".format(
-                            [vf.name for vf in
-                             self.model._meta.private_fields])
-                        raise Exception(INVALID_MK.format(
-                            self.master_key, self, msg))
+                            [vf.name for vf in self.model._meta.private_fields]
+                        )
+                        raise Exception(INVALID_MK.format(self.master_key, self, msg))
                         # raise Exception(
                         #     "%s : invalid master class for master_key "
                         #     "%r in %s" % (
@@ -497,36 +507,39 @@ class Table(AbstractTable):
                     self.master_field = fk
                     # self.hidden_columns |= set([fk.name])
 
-        super(Table, self).class_init()
+        super().class_init()
 
         if self.order_by is not None:
             if not isinstance(self.order_by, (list, tuple)):
-                raise Exception("%s.order_by is %r (must be a list or tuple)" %
-                                (self, self.order_by))
+                raise Exception(
+                    "%s.order_by is %r (must be a list or tuple)"
+                    % (self, self.order_by)
+                )
             if False:
                 # good idea, but doesn't yet work for remote fields,
                 # e.g. order_by = ['content_type__app_label']
                 for fieldname in self.order_by:
-                    if fieldname.startswith('-'):
+                    if fieldname.startswith("-"):
                         fieldname = fieldname[1:]
                     try:
                         fk, remote, direct, m2m = self.model._meta.get_field_by_name(
-                            fieldname)
+                            fieldname
+                        )
                         assert direct
                         assert not m2m
                     except FieldDoesNotExist:
-                        raise Exception("Unknown fieldname %r in %s.order_by" %
-                                        (fieldname, self))
+                        raise Exception(
+                            "Unknown fieldname %r in %s.order_by" % (fieldname, self)
+                        )
 
     @classmethod
     def do_setup(self):
-
-        super(Table, self).do_setup()
+        super().do_setup()
         # AbstractTable.do_setup(self)
         if self.model is None:
             return
 
-        if hasattr(self.model, '_lino_slaves'):
+        if hasattr(self.model, "_lino_slaves"):
             self._slaves = list(self.model._lino_slaves.values())
         else:
             self._slaves = []
@@ -541,7 +554,7 @@ class Table(AbstractTable):
 
     @classmethod
     def make_disabled_fields(cls, obj, ar):
-        s = super(Table, cls).make_disabled_fields(obj, ar)
+        s = super().make_disabled_fields(obj, ar)
 
         if obj is not None and ar is not None:
             s |= obj.disabled_fields(ar)
@@ -578,45 +591,22 @@ class Table(AbstractTable):
         # logger.info("20130225 dbtables.disable_delete")
         if self.delete_action is None:
             return "No delete_action"
-        if not self.get_row_permission(obj, ar, self.get_row_state(obj), self.delete_action):
+        if not self.get_row_permission(
+                obj, ar, self.get_row_state(obj), self.delete_action):
             # print "20130222 ar is %r" % ar
             # logger.info("20130225 dbtables.disable_delete no permission")
             return _("You have no permission to delete this row.")
         return obj.disable_delete(ar)
 
-    # @classmethod
-    # def get_data_elem(self, name):
-    #     """
-    #     Adds the possibility to specify
-    #     :class:`remote fields <lino.core.fields.RemoteField>`
-    #     in a layout template.
-    #     """
-    #     # cc = AbstractTable.get_data_elem(self,name)
-    #
-    #     # if self.model is not None:
-    #     if isinstance(self.model, type) and issubclass(self.model, models.Model):
-    #         # logger.info("20120202 Table.get_data_elem found nothing")
-    #         for m in models_by_base(self.model):
-    #             de = m.get_data_elem(name)
-    #             if de is not None:
-    #                 return de
-    #     return super(Table, self).get_data_elem(name)
-    #
     @classmethod
     def get_request_queryset(self, ar, **filter):
-        """
-        Return the iterable of Django database objects for the specified
-        action request.
+        # **filter is allowed only on abstract models.
 
-        The default implementation calls :meth:`get_user_queryset` and then
-        applies request parameters.
-        """
-        # print("20181121b get_request_queryset", self)
-        if not isinstance(ar, TableRequest):
-            ar = self.request(parent=ar)
+        # raise Exception("20240321 get_request_queryset", self)
+        # if not isinstance(ar, TableRequest):
+        #     ar = self.create_request(parent=ar)
 
         def apply(qs):
-
             # print("20160329 {}".format(qs.query))
             if qs is None:
                 return self.model.objects.none()
@@ -627,24 +617,30 @@ class Table(AbstractTable):
                 qs = qs.filter(**kw)
 
             if ar.exclude:
-                qs = qs.exclude(**ar.exclude)
+                qs = qs.exclude(ar.exclude)
                 # qs = qs.exclude(ar.exclude)
 
-            # 20200425
-            spv = dict()
-            for k in self.simple_parameters:
-                v = getattr(ar.param_values, k)
-                # if "room" in k:
-                # print("20200423", k, v, self.simple_parameters, ar.param_values)
-                if v == constants.CHOICES_BLANK_FILTER_VALUE:
-                    spv[k+"__isnull"] = True
-                elif v == constants.CHOICES_NOT_BLANK_FILTER_VALUE:
-                    spv[k+"__isnull"] = False
-                elif v is not None:
-                    spv[k] = v
+            if ar.param_values is not None:
+                spv = dict()
+                for k in self.simple_parameters:
+                    v = getattr(ar.param_values, k)
+                    # if "room" in k:
+                    # print("20200423", k, v, self.simple_parameters, ar.param_values)
+                    if v == constants.CHOICES_BLANK_FILTER_VALUE:
+                        spv[k + "__isnull"] = True
+                    elif v == constants.CHOICES_NOT_BLANK_FILTER_VALUE:
+                        spv[k + "__isnull"] = False
+                    # elif v is not None:
+                    elif v not in {None, ''}:
+                        spv[k] = v
+                        # print("20240506 {} = {}".format(k, v))
+                        # e.g. in MyTickets 'user' should be an obvious field
+                        # 20241004 caused #5768:
+                        ar.known_values[k] = v  # make it an obvious field
 
-            qs = self.model.add_param_filter(qs, **spv)
-            # qs = self.model.add_param_filter(qs, **ar.param_values)
+                qs = self.model.add_param_filter(qs, **spv)
+                # qs = self.model.add_param_filter(qs, **ar.param_values)
+                # print(f"20251017 {ar.known_values} {qs.query}")
 
             if self.filter:
                 qs = qs.filter(self.filter)
@@ -655,7 +651,7 @@ class Table(AbstractTable):
             if ar.known_values:
                 # logger.info("20120111 known values %r", ar.known_values)
                 d = {}
-                for k, v in list(ar.known_values.items()):
+                for k, v in ar.known_values.items():
                     if v is None:
                         d[k + "__isnull"] = True
                     else:
@@ -664,8 +660,7 @@ class Table(AbstractTable):
                     qs = qs.filter(**d)
 
             if self.exclude:
-                qs = qs.exclude(**self.exclude)
-                # TODO: use Q object instead of dict
+                qs = qs.exclude(self.exclude)
 
             if ar.quick_search:
                 qs = self.add_quick_search_filter(qs, ar.quick_search)
@@ -684,48 +679,38 @@ class Table(AbstractTable):
             return qs
 
         if self.model._meta.abstract:
-            # Django's UNION feature is a pseudo feature because
-            # count() doesn't work and because it doesn't work on
-            # mysql (Django ticket #28281).  So here we emulate a
-            # union and return an iteration over the different models
-            # which implement the abstract model. One limitation is of
-            # course that they are ordered only within their model.
-            # There is no known in production usage of this feature.
+            # Django's UNION doesn't support count()
+            # (Django ticket `#28281 <https://code.djangoproject.com/ticket/28281>`__)
+            # and doesn't work on mysql (is this still true?).
+            # So here we emulate a union and return an iteration over the
+            # different models that implement the abstract model. One limitation
+            # is of course that they are ordered only within their model. Used
+            # for vat.IntracomPurchases and vat.IntracomSales
             def func():
                 for m in models_by_base(self.model):
                     qs = m.get_request_queryset(ar, **filter)
                     qs = apply(qs)
                     for obj in qs:
                         yield obj
+
             return func()
+
         assert not filter
-        qs = self.get_queryset(ar)
+        # qs = self.get_queryset(ar)
+        qs = self.model.get_request_queryset(ar)
         return apply(qs)
 
-    @classmethod
-    def get_queryset(self, ar, **filter):
-        """Return the Django Queryset processed by this table.
+    # @classmethod
+    # def get_queryset(self, ar, **filter):
+    #     # raise Exception("20240321 get_queryset", self)
+    #     return self.model.get_request_queryset(ar, **filter)
 
-        The default implementation forwards the call to the model's
-        :meth:`get_request_queryset
-        <lino.core.model.Model.get_request_queryset>`.
-
-        Override this to use e.g. select_related() or to return a list.
-
-        Example::
-
-          def get_queryset(self, **ar):
-              return self.model.objects.select_related('country', 'city')
-
-        """
-        return self.model.get_request_queryset(ar, **filter)
-
-    @classmethod
-    def get_title_tags(self, ar):
-        for t in super(Table, self).get_title_tags(ar):
-            yield t
-        for t in self.model.get_title_tags(ar):
-            yield t
+    # @classmethod
+    # def get_title_tags(self, ar):
+    #     for t in super(Table, self).get_title_tags(ar):
+    #         yield t
+    #     for t in self.model.get_title_tags(ar):
+    #         yield t
 
     @classmethod
     def create_instance(self, ar, **kw):
@@ -757,8 +742,8 @@ class Table(AbstractTable):
 
     # @classmethod
     # def ajax_update(self,request):
-        # print request.POST
-        # return HttpResponse("1", mimetype='text/x-json')
+    # print request.POST
+    # return HttpResponse("1", mimetype='text/x-json')
 
 
 def table_factory(model):
@@ -770,10 +755,10 @@ def table_factory(model):
     # logger.info('table_factory(%s)',model.__name__)
     bases = (Table,)
     for b in model.__bases__:
-        rpt = getattr(b, '_lino_default_table', None)
+        rpt = getattr(b, "_lino_default_table", None)
         if rpt is not None:
             if issubclass(model, rpt.model):
-            # if issubclass(rpt.model,model):
+                # if issubclass(rpt.model,model):
                 bases = (rpt,)
                 # bases = (rpt.__class__,)
     # logger.info('table_factory(%s) : bases is %s',model.__name__,bases)
