@@ -7,6 +7,7 @@ import copy
 from django.conf import settings
 from django.db.models.signals import class_prepared
 from django.core.exceptions import FieldDoesNotExist
+
 # from django.db.models.fields import FieldDoesNotExist
 from django.db import models
 from django.dispatch import receiver
@@ -16,13 +17,13 @@ from lino.core.signals import pre_analyze
 from .utils import resolve_model
 
 from django.apps import apps
+
 get_models = apps.get_models
 
 PENDING_INJECTS = dict()
 PREPARED_MODELS = dict()
 
 
-
 def fix_field_cache(model):
     """
     Remove duplicate entries in the field cache of the specified model
@@ -48,7 +49,7 @@ def fix_field_cache(model):
             pass  # msg(f.attname)
         else:
             new_cache.append(f)
-        #~ raise Exception("20131110 %r" % (model._meta._field_cache,))
+        # ~ raise Exception("20131110 %r" % (model._meta._field_cache,))
     model._meta.local_fields = new_cache
     # model._meta._expire_cache()
     # print(model._meta.fields)
@@ -60,7 +61,6 @@ def fix_field_cache(model):
     # django.core.exceptions.AppRegistryNotReady: Models aren't loaded yet.
 
 
-
 @receiver(class_prepared)
 def on_class_prepared(sender, **kw):
     """This is Lino's general `class_prepared` handler.
@@ -80,15 +80,15 @@ def on_class_prepared(sender, **kw):
     # collect_virtual_fields() first time because virtual fields might
     # get updated
     # collect_virtual_fields(model)
-    k = model._meta.app_label + '.' + model.__name__
+    k = model._meta.app_label + "." + model.__name__
     PREPARED_MODELS[k] = model
-    #~ logger.info("20120627 on_class_prepared %r = %r",k,model)
+    # ~ logger.info("20120627 on_class_prepared %r = %r",k,model)
     todos = PENDING_INJECTS.pop(k, None)
     if todos is not None:
         for func, caller in todos:
             func(model)
-        #~ for k,v in injects.items():
-            #~ model.add_to_class(k,v)
+        # ~ for k,v in injects.items():
+        # ~ model.add_to_class(k,v)
 
     fix_field_cache(model)
 
@@ -96,12 +96,13 @@ def on_class_prepared(sender, **kw):
     # # might have been injected
     # collect_virtual_fields(model)
 
+
 def fmt(func_caller):
     f, caller = func_caller
-    #~ ln = inspect.getsourcelines(f)[1]
-    #~ return "%s in %s:%d" % (f.__name__,inspect.getsourcefile(f),ln)
-    #~ return "%s in %s:%d" % (f.__name__,caller.filename,caller.line_no)
-    #~ return "%s in %s" % (f.__name__,caller)
+    # ~ ln = inspect.getsourcelines(f)[1]
+    # ~ return "%s in %s:%d" % (f.__name__,inspect.getsourcefile(f),ln)
+    # ~ return "%s in %s:%d" % (f.__name__,caller.filename,caller.line_no)
+    # ~ return "%s in %s" % (f.__name__,caller)
     return "called from %s" % caller
 
 
@@ -109,25 +110,31 @@ def fmt(func_caller):
 def check_pending_injects(sender, models_list=None, **kw):
     # raise Exception(20150304)
     # called from kernel.analyze_models()
-    #~ logger.info("20130212 check_pending_injects()...")
+    # ~ logger.info("20130212 check_pending_injects()...")
+    # if PENDING_INJECTS:
+    #     for spec, todos in PENDING_INJECTS.items():
+    #         model = resolve_model(spec, strict=True)
+    #         if todos is not None:
+    #             for func, caller in todos:
+    #                 func(model)
+
     if PENDING_INJECTS:
-        msg = ''
-        for spec, funcs in list(PENDING_INJECTS.items()):
-            msg += spec + ': '
-            msg += ', '.join([fmt(f) for f in funcs])
-            #~ msg += '\n'.join([str(dir(func)) for func in funcs])
-            #~ msg += '\n'.join([str(func.func_code.co_consts) for func in funcs])
-            #~ msg += str(funcs)
-        raise Exception("Oops, there are pending injects: %s" % msg)
-        #~ logger.warning("pending injects: %s", msg)
-
-    #~ logger.info("20131110 no pending injects")
+        msg = ""
+        for spec, funcs in PENDING_INJECTS.items():
+            msg += spec + ": "
+            msg += ", ".join([fmt(f) for f in funcs])
+            # ~ msg += '\n'.join([str(dir(func)) for func in funcs])
+            # ~ msg += '\n'.join([str(func.func_code.co_consts) for func in funcs])
+            # ~ msg += str(funcs)
+        raise Exception(f"Oops, there are pending injects: {msg}")
+        # ~ logger.warning("pending injects: %s", msg)
 
+    # ~ logger.info("20131110 no pending injects")
     """
     20130106
-    now we loop a last time over each model and fill it's _meta._field_cache
-    otherwise if some application module used inject_field() on a model which
-    has subclasses, then the new field would not be seen by subclasses
+    now we loop a last time over each model and fill its _meta._field_cache
+    otherwise if some plugin used inject_field() on a model that
+    has subclasses, the new field would not be seen by subclasses
     """
     for model in models_list:
         model._meta._expire_cache()
@@ -144,14 +151,14 @@ def do_when_prepared(todo, *model_specs):
 
     If a model_spec is not a string, the function `todo` is called immediately.
     """
-    #~ caller = inspect.stack()[2]
+    # ~ caller = inspect.stack()[2]
     caller = inspect.getouterframes(inspect.currentframe())[2]
-    #~ print 20131111, caller
+    # ~ print 20131111, caller
     caller = "%s:%d" % (caller[1], caller[2])
 
-    #~ caller = inspect.getframeinfo(caller)
-    #~ caller = inspect.getframeinfo(inspect.currentframe().f_back)[2]
-    #~ caller = inspect.getframeinfo(caller.f_back)[2]
+    # ~ caller = inspect.getframeinfo(caller)
+    # ~ caller = inspect.getframeinfo(inspect.currentframe().f_back)[2]
+    # ~ caller = inspect.getframeinfo(caller.f_back)[2]
 
     # logger.info("20200622 test %s for %s", caller, model_specs)
     for model_spec in model_specs:
@@ -166,15 +173,15 @@ def do_when_prepared(todo, *model_specs):
             if model is None:
                 injects = PENDING_INJECTS.setdefault(k, [])
                 injects.append((todo, caller))
-                #~ d[name] = field
-                #~ if model_spec == "system.SiteConfig":
+                # ~ d[name] = field
+                # ~ if model_spec == "system.SiteConfig":
                 # logger.info("20200622 Defer %s for %s", caller, model_spec)
                 continue
         else:
             model = model_spec
-            #~ k = model_spec._meta.app_label + '.' + model_spec.__name__
-        #~ if model._meta.abstract:
-            #~ raise Exception("Trying do_when_prepared on abstract model %s" % model)
+            # ~ k = model_spec._meta.app_label + '.' + model_spec.__name__
+        # ~ if model._meta.abstract:
+        # ~ raise Exception("Trying do_when_prepared on abstract model %s" % model)
         # logger.info("20200622 Run %s for %s", caller, model)
         # from lino.core.model import Model
         # Model.django2lino(model)
@@ -188,8 +195,10 @@ def when_prepared(*model_specs):
     If the model has already been prepared, the function is executed
     immediately.
     """
+
     def decorator(fn):
         return do_when_prepared(fn, *model_specs)
+
     return decorator
 
 
@@ -201,20 +210,25 @@ def inject_action(model_spec, **actions):
     choosers.
 
     """
+
     def todo(model):
         model.define_action(**actions)
+
     return do_when_prepared(todo, model_spec)
 
+
 def update_model(model_spec, **actions):
     """
     Replace the specified attributes in the specified model.
     """
+
     def todo(model):
         for k, v in actions.items():
             if not hasattr(model, k):
                 raise Exception(
                     "%s has no attribute %s to update." % (model, k))
             setattr(model, k, v)
+
     if isinstance(model_spec, models.Model):
         return todo(model_spec)
     return do_when_prepared(todo, model_spec)
@@ -256,8 +270,7 @@ def inject_field(model_spec, name, field, doc=None, active=False):
         if True:  # 20181023
             try:
                 model._meta.get_field(name)
-                raise Exception("Duplicate field {} on {}".format(
-                    name, model))
+                raise Exception("Duplicate field {} on {}".format(name, model))
             except FieldDoesNotExist:
                 pass
         model.add_to_class(name, field)
@@ -293,12 +306,14 @@ def update_field(model_spec, name, **kw):
     field initialization.
 
     """
+
     # if name == "overview":
     #     if 'verbose_name' in kw:
     #         if kw['verbose_name'] is None:
     #             raise Exception("20181022")
     def todo(model):
         from lino.core import actors
+
         model.collect_virtual_fields()
         # if issubclass(model, models.Model):
         #     collect_virtual_fields(model)
@@ -368,13 +383,15 @@ def inject_quick_add_buttons(model, name, target):
 
     def fn(self, ar):
         if ar is None:
-            return ''
-        return ar.renderer.quick_add_buttons(
-            ar.spawn(target, master_instance=self))
+            return ""
+        return ar.renderer.quick_add_buttons(ar.spawn(target, master_instance=self))
+
     tm = resolve_model(target.model)
-    inject_field(model, name,
-                 fields.VirtualField(fields.DisplayField(
-                     tm._meta.verbose_name_plural), fn))
+    inject_field(
+        model,
+        name,
+        fields.VirtualField(fields.DisplayField(
+            tm._meta.verbose_name_plural), fn))
 
 
 # def django_patch():
