@@ -1,82 +1,56 @@
 # Copyright 2009-2020 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """See :doc:`/dev/layouts/index`.
 
 """
 
-from lino import logger
 import re
 
 from django.conf import settings
 from django.utils.translation import gettext_lazy as _
+from django.utils.functional import Promise
 from django.db.models.fields import NOT_PROVIDED
 from django.db.models.fields.related import ForeignObject
 # from django.contrib.contenttypes.fields import GenericRelation
 
+from lino import logger
+from lino.utils import jsgen
 from lino.core import constants
-from lino.core.fields import fields_list, VirtualField, wildcard_data_elems
+from lino.core import atomizer
+from lino.core import fields
+from lino.core import store
 from lino.core.plugin import Plugin
+from lino.core.utils import Panel
 from lino.modlib.users.utils import get_user_profile
 
 
 class LayoutError(RuntimeError):
     pass
 
-LABEL_ALIGN_TOP = 'top'
-LABEL_ALIGN_LEFT = 'left'
-LABEL_ALIGN_RIGHT = 'right'
+
+LABEL_ALIGN_TOP = "top"
+LABEL_ALIGN_LEFT = "left"
+LABEL_ALIGN_RIGHT = "right"
 
 ALLOW_DUPLICATE_ELEMS = True
 
 
 def DEBUG_LAYOUTS(lo):
-    #~ if lo._table.__name__ == 'Users':
-        #~ return True
+    # ~ if lo._table.__name__ == 'Users':
+    # ~ return True
     return False
 
 
-class DummyPanel(object):
+class DummyPanel:
     """
-    A layout panel which does not exist in the current configuration
+    A layout panel that does not exist in the current configuration
     but might exist as a real panel in some other configuration.
     """
+
     pass
 
 
-class Panel(object):
-
-    """
-    To be used when a panel cannot be expressed using a simple
-    template string because it requires one or more options. These
-    `options` parameters can be:
-
-    - label
-    - required_roles
-    - window_size
-    - label_align
-
-    Unlike a :class:`BaseLayout` it cannot have any child panels
-    and cannot become a tabbed panel.
-    """
-
-    def __init__(self, desc, label=None, **options):
-        # assert not 'required' in options
-        self.desc = desc
-        if label is not None:
-            options.update(label=label)
-        self.options = options
-
-    def replace(self, *args, **kw):
-        """
-        Calls the standard :meth:`string.replace`
-        method on this Panel's template.
-        """
-        self.desc = self.desc.replace(*args, **kw)
-
-
-
-class LayoutHandle(object):
+class LayoutHandle:
     """
     A layout handle analyzes a layout (an instance of :class:`BaseLayout` or a some
     subclass thereof) and holds the resulting metadata, especially the layout
@@ -100,7 +74,7 @@ class LayoutHandle(object):
         assert isinstance(layout, BaseLayout)
         assert isinstance(ui, Plugin)
         self.layout = layout
-        self.ui = ui
+        self.ui = ui  # TODO: rename ui to front_end
         self.hidden_elements = layout.hidden_elements
         self._data_elems = []
         self._names = {}
@@ -110,24 +84,23 @@ class LayoutHandle(object):
         #     main = layout.main_m or layout.main
         # else:
         #     main = layout.main
-        main = layout.main
-        self.define_panel('main', main)
 
-        self.main = self._names.get('main')
+        self.define_panel("main", layout.main)
+
+        self.main = self._names.get("main")
         if self.main is None:
             raise Exception(
-                "Failed to create main element %r for %s." % (
-                    layout.main, layout))
+                "Failed to create main element %r for %s." % (layout.main, layout)
+            )
 
         self.width = self.main.width
         self.height = self.main.height
-
-        self.layout.setup_handle(self)
         for k, v in self.layout._labels.items():
             if k not in self._names:
                 raise Exception(
-                    "%s has no attribute %r (layout.main is %r)" %
-                    (self, k, layout.main))
+                    "%s has no attribute %r (layout.main is %r)"
+                    % (self, k, layout.main)
+                )
             self._names[k].set_label(v)
 
     def desc2elem(self, elemname, desc, **kwargs):
@@ -147,44 +120,45 @@ class LayoutHandle(object):
             # if 'label_align' in kwargs:
             #     print("20170921 desc2elem", elemname, desc, kwargs)
 
-        if not isinstance(desc, str):
-            raise Exception("{} is {} (must be a string)".format(
-                elemname, desc))
+        if isinstance(desc, Promise):
+            desc = str(desc)
+        elif not isinstance(desc, str):
+            raise Exception("{} is {} (must be a string)".format(elemname, desc))
 
         # flatten continued lines:
-        desc = desc.replace('\\\n', '')
+        desc = desc.replace("\\\n", "")
 
         # expand aliases (configurable fields)
         for alias, repl in self.layout._datasource.get_layout_aliases():
             desc = re.sub(r"\b" + alias + r"\b", repl, desc)
 
         # expand wildcards
-        if '*' in desc:
-            assert elemname == 'main'
+        if "*" in desc:
+            assert elemname == "main"
             explicit_specs = set()
             remote_wildcards = []
             for spec in desc.split():
-                if '*' not in spec:
-                    name, kwargs = self.splitdesc(spec)
+                if "*" not in spec:
+                    name, kwargs2 = self.splitdesc(spec)
                     # if 'hide_sum' in kwargs:
-                    #     raise Exception("20180210a")
+                    #     print("20240213 desc2elem {}".format(spec))
+                    #     # raise Exception("20180210a {}".format(spec))
                     explicit_specs.add(name)
                 elif len(spec) > 1:
                     remote_wildcards.append(spec)
 
             for rwspec in remote_wildcards:
-                assert rwspec.endswith('__*')
+                assert rwspec.endswith("__*")
                 rmodel = rwspec[:-3]
                 rwprefix = rwspec[:-1]
 
                 fld = self.get_data_elem(rmodel)
                 if fld is None:
-                    raise Exception(
-                        "Invalid remote wildcard %s" % rw)
+                    raise Exception("Invalid remote wildcard %s" % rw)
                 rwmodel = fld.remote_field.model
                 rwnames = []
-                for de in wildcard_data_elems(rwmodel):
-                    if isinstance(de, (VirtualField, ForeignObject)):
+                for de in fields.wildcard_data_elems(rwmodel):
+                    if isinstance(de, (fields.VirtualField, ForeignObject)):
                         continue
                     if self.use_as_wildcard(de):
                         k = rwprefix + de.name
@@ -201,15 +175,15 @@ class LayoutHandle(object):
                 if de.name not in explicit_specs:
                     if self.use_as_wildcard(de):
                         wildcard_names.append(de.name)
-                        if len(explicit_specs) or isinstance(de, VirtualField):
+                        if len(explicit_specs) or (
+                                isinstance(de, fields.FakeField) and de.hide_unless_explicit):
                             self.hidden_elements.add(de.name)
             wildcard_str = self.layout.join_str.join(wildcard_names)
-            desc = desc.replace('*', wildcard_str)
+            desc = desc.replace("*", wildcard_str)
 
             mk = self.layout._datasource.master_key
-            if mk and mk not in explicit_specs \
-               and mk not in self.hidden_elements:
-                desc += ' ' + mk
+            if mk and mk not in explicit_specs and mk not in self.hidden_elements:
+                desc += " " + mk
                 self.hidden_elements.add(mk)
 
         if "\n" in desc:
@@ -225,7 +199,7 @@ class LayoutHandle(object):
                 x = x.strip()
                 if len(x) > 0 and not x.startswith("# "):
                     i += 1
-                    e = self.desc2elem(elemname + '_' + str(i), x)
+                    e = self.desc2elem(elemname + "_" + str(i), x)
                     if e is not None:
                         elems.append(e)
         else:
@@ -246,14 +220,15 @@ class LayoutHandle(object):
                         elems.append(e)
         if len(elems) == 0:
             return None
-        if len(elems) == 1 and elemname != 'main':
+        if len(elems) == 1 and elemname != "main":
             elems[0].setup(**kwargs)
             return elems[0]
-        from lino.core.elems import create_layout_panel
-        return create_layout_panel(
-            self, elemname, vertical, elems, **kwargs)
 
-    def define_panel(self, name, desc, **kw):
+        from lino.core.elems import create_layout_panel
+
+        return create_layout_panel(self, elemname, vertical, elems, **kwargs)
+
+    def define_panel(self, name, desc, **kwargs):
         if not desc:
             return
         if name in self._names:
@@ -261,9 +236,10 @@ class LayoutHandle(object):
                 return self._names[name]
             else:
                 raise Exception(
-                    'Duplicate element definition %s = %r in %s'
-                    % (name, desc, self.layout))
-        e = self.desc2elem(name, desc, **kw)
+                    "Duplicate element definition %s = %r in %s"
+                    % (name, desc, self.layout)
+                )
+        e = self.desc2elem(name, desc, **kwargs)
         if e is None:
             return
         self._names[name] = e
@@ -271,7 +247,8 @@ class LayoutHandle(object):
 
     def create_element(self, desc_name):
         from lino.core.elems import create_layout_element
-        #~ logger.debug("create_element(%r)", desc_name)
+
+        # ~ logger.debug("create_element(%r)", desc_name)
         name, options = self.splitdesc(desc_name)
         # if 'hide_sum' in options:
         #     raise Exception("20180210b")
@@ -280,8 +257,9 @@ class LayoutHandle(object):
                 return self._names[name]
             else:
                 raise Exception(
-                    'Duplicate element usage %s = %r in %s'
-                    % (name, desc_name, self.layout))
+                    "Duplicate element usage %s = %r in %s"
+                    % (name, desc_name, self.layout)
+                )
         desc = getattr(self.layout, name, None)
         if desc is not None:
             return self.define_panel(name, desc, **options)
@@ -298,7 +276,7 @@ class LayoutHandle(object):
             else:
                 e.hidden = True
 
-        self.ui.setup_layout_element(e)
+        # self.ui.setup_layout_element(e)
         self.layout.setup_element(self, e)
         self._names[name] = e
         return e
@@ -306,8 +284,8 @@ class LayoutHandle(object):
     def splitdesc(self, picture, **options):
         """Parse the given element descriptor and return a tuple `(name,
         options)` where `name` is the element name and `options` is a
-        `dict` with keyword arguments to be forwarded to the widget
-        constructor (:class:`LayoutElement`).
+        `dict` with :term:`widget options` (keyword arguments to be forwarded to the widget
+        constructor (:class:`LayoutElement`)).
 
         """
         a = picture.split(":", 1)
@@ -315,10 +293,10 @@ class LayoutHandle(object):
             name = picture
         elif len(a) == 2:
             name = a[0]
-            if ':' in a[1]:
-                aas = a[1].split(':')
+            if ":" in a[1]:
+                aas = a[1].split(":")
                 for v in aas:
-                    m = re.match(r'^(\d+)(x(\d+))?$', v)
+                    m = re.match(r"^(\d+)(x(\d+))?$", v)
                     if m is not None:
                         a = v
                         continue
@@ -345,6 +323,8 @@ class LayoutHandle(object):
         return self._names[name]
 
     def add_data_elem(self, field):
+        # if field in self._data_elems:
+        #     raise Exception(f"20251007 {field} {self}")
         self._data_elems.append(field)
 
     def get_title(self, ar):
@@ -357,7 +337,7 @@ class LayoutHandle(object):
         return self.main.ext_lines(request)
 
     def use_as_wildcard(self, de):
-        if de.name.endswith('_ptr'):
+        if de.name.endswith("_ptr"):
             return False
         # if isinstance(de, VirtualField):
         #     return False
@@ -371,10 +351,10 @@ class LayoutHandle(object):
         # precedence over those defined in the datasource.
 
         # if not name in ('__str__', '__unicode__', 'name', 'label'):
-        if name not in ('name', 'label'):
+        if name not in ("name", "label"):
             value = getattr(self.layout, name, NOT_PROVIDED)
-            # if name == 'ledger':
-            #     logger.info("20150610 'ledger' in instance of %s is %r",
+            # if name == 'accounting':
+            #     logger.info("20150610 'accounting' in instance of %s is %r",
             #                 self.layout.__class__, value)
             if value is not NOT_PROVIDED:
                 return value
@@ -382,11 +362,11 @@ class LayoutHandle(object):
 
     def get_choices_url(self, *args, **kw):
         return self.layout.get_choices_url(
-            settings.SITE.kernel.default_renderer.front_end,
-            *args, **kw)
+            settings.SITE.kernel.default_renderer.front_end, *args, **kw
+        )
 
 
-class BaseLayout(object):
+class BaseLayout:
     """
     Base class for all Layouts (:class:`FormLayout`, :class:`ColumnsLayout`
     and  :class:`ParamsLayout`).
@@ -409,7 +389,7 @@ class BaseLayout(object):
     required_roles = None
     label_align = LABEL_ALIGN_TOP
 
-    window_size = None
+    window_size: None | tuple[int, str] = None
     """A tuple `(width, height)` that specifies the size of the window to
     be used for this layout.
 
@@ -439,8 +419,7 @@ class BaseLayout(object):
     #
     # """
 
-    def __init__(self, main=None, datasource=None,
-                 hidden_elements=None, **kw):
+    def __init__(self, main=None, datasource=None, hidden_elements=None, **kw):
         """
         datasource is either an actor or an action.
         """
@@ -451,10 +430,9 @@ class BaseLayout(object):
         self._element_options = dict()
         if main is not None:
             self.main = main
-        #~ elif not hasattr(self,'main'):
+        # ~ elif not hasattr(self,'main'):
         elif self.main is None:
-            raise Exception(
-                "Cannot instantiate %s without `main`." % self.__class__)
+            raise Exception("Cannot instantiate %s without `main`." % self.__class__)
         self.set_datasource(datasource)
         for k, v in kw.items():
             # The following test is deactivated because it is possible
@@ -471,8 +449,8 @@ class BaseLayout(object):
 
     def __str__(self):
         return "{}.{} on {!r}".format(
-            self.__class__.__module__, self.__class__.__name__,
-            self._datasource)
+            self.__class__.__module__, self.__class__.__name__, self._datasource
+        )
 
     def add_datasource(self, ds):
         if ds in self._other_datasources:
@@ -483,12 +461,12 @@ class BaseLayout(object):
         self._datasource = ds
         if ds is not None:
             if isinstance(self.hidden_elements, str):
-                self.hidden_elements = set(fields_list(
-                    ds, self.hidden_elements))
+                self.hidden_elements = set(
+                    atomizer.fields_list(ds, self.hidden_elements))
             self.hidden_elements |= ds.hidden_elements
             # self.editable = not ds.hide_editing(user_type)
-            #~ if str(ds).endswith('Partners'):
-                #~ print "20130124 set_datasource ", self,self.hidden_elements
+            # ~ if str(ds).endswith('Partners'):
+            # ~ print "20130124 set_datasource ", self,self.hidden_elements
 
     @property
     def editable(self):
@@ -515,18 +493,13 @@ class BaseLayout(object):
         Removes specified element names from this layout's `main` template.
         """
         for name in args:
-            self.main = self.main.replace(name, '')
-
-    def setup_handle(self, lh):
-        pass
+            self.main = self.main.replace(name, "")
 
     def setup_element(self, lh, e):
         pass
 
     def update(self, **kw):
-        """Update the template of one or more panels.
-
-        """
+        """Update the template of one or more panels."""
         for k, v in list(kw.items()):
             if DEBUG_LAYOUTS(self):
                 msg = """\
@@ -535,7 +508,7 @@ In %s, updating attribute %r:
 %s
 --- after:
 %s
----""" % (self, k, getattr(self, k, '(undefined)'), v)
+---""" % (self, k, getattr(self, k, "(undefined)"), v)
                 logger.info(msg)
             setattr(self, k, v)
 
@@ -550,14 +523,14 @@ In %s, updating attribute %r:
         - `label` an optional label
         - any further keyword are passed as options to the new panel
         """
-        #~ if hasattr(self,'_extjs3_handle'):
-            #~ raise Exception("Cannot update for layout after UI has been set up.")
-        if '\n' in name:
+        # ~ if hasattr(self,'_extjs3_handle'):
+        # ~ raise Exception("Cannot update for layout after UI has been set up.")
+        if "\n" in name:
             raise Exception("name may not contain any newline")
-        if ' ' in name:
+        if " " in name:
             raise Exception("name may not contain any whitespace")
-        #~ if getattr(self,name,None) is not None:
-            #~ raise Exception("name %r already defined in %s" % (name,self))
+        # ~ if getattr(self,name,None) is not None:
+        # ~ raise Exception("name %r already defined in %s" % (name,self))
         self._add_panel(name, tpl, label, options)
 
     def _add_panel(self, name, tpl, label, options):
@@ -591,20 +564,23 @@ Adding panel %r to %s ---:
         - `tpl` the template string
         - `label` an optional label
         """
-        #~ print "20120526 add_detail_tab", self, name
-        #~ if hasattr(self,'_extjs3_handle'):
-            #~ raise Exception("Cannot update form layout after UI has been set up.")
-        if '\n' in name:
+        # ~ print "20120526 add_detail_tab", self, name
+        # ~ if hasattr(self,'_extjs3_handle'):
+        # ~ raise Exception("Cannot update form layout after UI has been set up.")
+        if "\n" in name:
             raise Exception("name may not contain any newline")
-        if ' ' in name:
+        if " " in name:
             raise Exception("name may not contain any whitespace")
-        if '\n' in self.main:
-            if hasattr(self, 'general'):
-                raise NotImplementedError("""\
-%s has both a vertical `main` and a panel called `general`.""" % self)
+        if "\n" in self.main:
+            if hasattr(self, "general"):
+                raise NotImplementedError(
+                    """\
+%s has both a vertical `main` and a panel called `general`."""
+                    % self
+                )
             self.general = self.main
             self.main = "general " + name
-            self._labels['general'] = _("General")
+            self._labels["general"] = _("General")
             if DEBUG_LAYOUTS(self):
                 msg = """\
 add_tabpanel() on %s moving content of vertical 'main' panel to 'general'.
@@ -616,18 +592,18 @@ New 'main' panel is %r"""
                 msg = """\
 add_tabpanel() on %s horizontal 'main' panel %r."""
                 logger.info(msg, self, self.main)
-        #~ if tpl is not None:
+        # ~ if tpl is not None:
         self._add_panel(name, tpl, label, options)
-            #~ self._add_panel(name,tpl)
-            #~ setattr(self,name,tpl)
-            # ~ self._added_panels[name] = tpl # 20120914c
-        #~ if label is not None:
-            #~ self._labels[name] = label
-        #~ self._element_options[name] = options
-        #~ if kw:
-            #~ print 20120525, self, self.detail_layout._element_options
+        # ~ self._add_panel(name,tpl)
+        # ~ setattr(self,name,tpl)
+        # ~ self._added_panels[name] = tpl # 20120914c
+        # ~ if label is not None:
+        # ~ self._labels[name] = label
+        # ~ self._element_options[name] = options
+        # ~ if kw:
+        # ~ print 20120525, self, self.detail_layout._element_options
 
-    def get_layout_handle(self, front_end=None):
+    def get_layout_handle(self):  # , front_end=None):
         """
         Return the LayoutHandle for this layout.
         Create a LayoutHandle instance if this is the first call.
@@ -639,12 +615,11 @@ add_tabpanel() on %s horizontal 'main' panel %r."""
         changed my mind.
 
         """
-        if front_end is None:
-            front_end = settings.SITE.kernel.editing_front_end
+        # if front_end is None:
+        front_end = settings.SITE.kernel.editing_front_end
         hname = front_end.ui_handle_attr_name
         if hname is None:
-            raise Exception(
-                "{0} has no `ui_handle_attr_name`!".format(front_end))
+            raise Exception(f"{front_end} has no `ui_handle_attr_name`!")
 
         # we do not want any inherited handle
         h = self.__dict__.get(hname, None)
@@ -667,7 +642,9 @@ add_tabpanel() on %s horizontal 'main' panel %r."""
             "choices",
             self._datasource.app_label,
             self._datasource.__name__,
-            field.name, **kw)
+            field.name,
+            **kw,
+        )
 
     def to_rst(self, user_type=None, name=None):
         lh = self.get_layout_handle()
@@ -707,11 +684,21 @@ class FormLayout(FieldLayout):
     :attr:`insert_layout <lino.core.actors.Actor.insert_layout>`.
 
     """
+
     join_str = "\n"
 
 
 class DetailLayout(FormLayout):
     pass
+    # def __repr__(self):
+    #     args = repr(self.main)
+    #     kwargs = {}
+    #     if self.window_size:
+    #         kwargs.update(window_size=self.window_size)
+    #     if kwargs:
+    #         kwargs = ", ".join(["{}={}".format(*i) for i in kwargs.items()])
+    #         return "dd.DetailLayout({}, {})".format(args, kwargs)
+    #     return "dd.DetailLayout({})".format(args)
 
 
 class InsertLayout(FormLayout):
@@ -727,6 +714,7 @@ class ColumnsLayout(FieldLayout):
     <lino.core.tables.AbstractTable.column_names>` attribute.
 
     """
+
     join_str = " "
 
     # def __init__(self, main=None, datasource=None,
@@ -746,8 +734,8 @@ class ColumnsLayout(FieldLayout):
             if name == actor.model.previewable_field:
                 pf_index = i
                 break
-        short_preview = actor.model.previewable_field + '_short_preview'
-        full_preview = actor.model.previewable_field + '_full_preview'
+        short_preview = actor.model.previewable_field + "_short_preview"
+        full_preview = actor.model.previewable_field + "_full_preview"
 
         def remove_preview(preview, hidden_elements):
             if preview in columns:
@@ -768,13 +756,13 @@ class ColumnsLayout(FieldLayout):
         columns.insert(pf_index + 1, short_preview)
         columns.insert(pf_index + 2, full_preview)
 
-        main = ' '.join(columns)
+        main = " ".join(columns)
         return main, hidden_elements
 
     def set_datasource(self, ds):
         if ds is None:
             raise Exception("20130327 No datasource for %r" % self)
-        super(ColumnsLayout, self).set_datasource(ds)
+        super().set_datasource(ds)
 
 
 class ParamsLayout(BaseLayout):
@@ -786,24 +774,32 @@ class ParamsLayout(BaseLayout):
     based on that actor's
     :attr:`params_layout <lino.core.actors.Actor.params_layout>`.
     """
+
     join_str = " "
     url_param_name = constants.URL_PARAM_PARAM_VALUES
-    params_store = None
+    _params_store = None
 
     def get_data_elem(self, name):
         de = self._datasource.get_param_elem(name)
         if de is None:
-            de = self._datasource.get_data_elem(name)
+            # no longer return a data element when no param element exists:
+            raise Exception(f"No parameter '{name}' in {self._datasource}")
+            # de = self._datasource.get_data_elem(name)
         de.editable = True  # 20200425
         # if "__" in name:
         #     print("20200425 ParamsLayout.get_data_elem()", name, de)
         return de
 
-    def setup_handle(self, lh):
-        # if str(self._datasource) == 'courses.Pupils':
-        #     print("20160329 setup_handle")
-        from lino.core.store import ParameterStore
-        self.params_store = ParameterStore(lh, self.url_param_name)
+    @property
+    def params_store(self):
+        if self._params_store is None:
+            k = settings.SITE.kernel.editing_front_end.ui_handle_attr_name
+            lh = getattr(self, k, None)
+            if lh is None:
+                # raise Exception(f"20250523 {self} has no lh")
+                return None
+            self._params_store = store.ParameterStore(lh, self.url_param_name)
+        return self._params_store
 
 
 class ActionParamsLayout(ParamsLayout):
@@ -815,16 +811,16 @@ class ActionParamsLayout(ParamsLayout):
 
     A subclass of :class:`ParamsLayout`.
     """
+
     join_str = "\n"
-    window_size = (50, 'auto')
+    window_size = (50, "auto")
     url_param_name = constants.URL_PARAM_FIELD_VALUES
 
     def setup_element(self, lh, e):
-        from lino.utils import jsgen
         e.declare_type = jsgen.DECLARE_THIS
 
     def get_choices_url(self, ui, field, **kw):
-        return settings.SITE.kernel.web_front_ends[0].build_plain_url(
+        return settings.SITE.kernel.editing_front_end.build_plain_url(
             "apchoices",
             self._datasource.defining_actor.app_label,
             self._datasource.defining_actor.__name__,
