@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Defines the classes :class:`MenuItem` and :class:`Menu`
 """
@@ -15,7 +14,6 @@ from lino.core.actions import WrappedAction
 
 
 class QuickLinksList:
-
     def __init__(self):
         self.items = []
 
@@ -30,26 +28,32 @@ class MenuItem(object):
     A menu item. Note that this is subclassed by :class:`Menu`: a menu
     is also a menu item.
     """
-    HOTKEY_MARKER = '~'
+
+    HOTKEY_MARKER = "~"
 
     name = None
     parent = None
 
-    def __init__(self,
-                 name=None, label=None, doc=None, enabled=True,
-                 action=None,
-                 #~ can_view=None,
-                 hotkey=None,
-                 params=None,
-                 help_text=None,
-                 #~ request=None,
-                 instance=None,
-                 javascript=None,
-                 href=None):
+    def __init__(
+        self,
+        name=None,
+        label=None,
+        doc=None,
+        enabled=True,
+        action=None,
+        # ~ can_view=None,
+        hotkey=None,
+        params=None,
+        help_text=None,
+        # ~ request=None,
+        instance=None,
+        javascript=None,
+        href=None,
+    ):
         if action is None:
             pass
-            #~ if instance is not None:
-                #~ action = instance.get_default_table().default_action
+            # ~ if instance is not None:
+            # ~ action = instance.get_default_table().default_action
         elif not isinstance(action, BoundAction):
             raise Exception("20121003 not a BoundAction: %r" % action)
         if instance is not None:
@@ -63,11 +67,12 @@ class MenuItem(object):
         self.javascript = javascript
         self.help_text = help_text
 
-        if label is None:
-            if instance is not None:
-                label = str(instance)
-            elif action is not None:
-                label = action.get_button_label()
+        if label is None and action is not None:
+            # if instance is not None:
+            #     label = str(instance)
+            # elif action is not None:
+            #     label = action.get_button_label()
+            label = action.get_button_label()
 
         if name is not None:
             self.name = name
@@ -79,8 +84,8 @@ class MenuItem(object):
         #     label = label.replace('~', '')
         self.label = label
 
-        #~ if self.label is None:
-            #~ raise Exception("20130907 %r has no label" % self)
+        # ~ if self.label is None:
+        # ~ raise Exception("20130907 %r has no label" % self)
 
     def compress(self):
         pass
@@ -103,28 +108,28 @@ class MenuItem(object):
     def __repr__(self):
         s = self.__class__.__name__ + "("
         attrs = []
-        for k in 'label instance bound_action href params name'.split():
+        for k in "label instance bound_action href params name".split():
             v = getattr(self, k)
             if v is not None:
                 attrs.append(k + "=" + obj2str(v))
-        s += ', '.join(attrs)
+        s += ", ".join(attrs)
         return s + ")"
 
-    #~ def interesting(self,**kw):
-        #~ l = []
-        #~ if self.label is not None:
-            #~ l.append(('label',self.label.strip()))
-        #~ if not self.enabled:
-            #~ l.append( ('enabled',self.enabled))
-        #~ return l
+    # ~ def interesting(self,**kw):
+    # ~ l = []
+    # ~ if self.label is not None:
+    # ~ l.append(('label',self.label.strip()))
+    # ~ if not self.enabled:
+    # ~ l.append( ('enabled',self.enabled))
+    # ~ return l
 
-    #~ def parents(self):
-        #~ l = []
-        #~ p = self.parent
-        #~ while p is not None:
-            #~ l.append(p)
-            #~ p = p.parent
-        #~ return l
+    # ~ def parents(self):
+    # ~ l = []
+    # ~ p = self.parent
+    # ~ while p is not None:
+    # ~ l.append(p)
+    # ~ p = p.parent
+    # ~ return l
 
     def get_url_path(self):
         if self.action:
@@ -134,7 +139,7 @@ class MenuItem(object):
             if len(s) and not s.endswith("/"):
                 s += "/"
         else:
-            s = '/'
+            s = "/"
         if self.name:
             return s + self.name
         return s
@@ -155,8 +160,7 @@ class MenuItem(object):
     #     return False
 
     def openui5Render(self):
-
-        ar = self.bound_action.request(**self.params)
+        ar = self.bound_action.create_request(**self.params)
         js = settings.SITE.kernel.default_renderer.request_handler(ar)
         return js
 
@@ -171,19 +175,16 @@ def create_item(unused, spec, action=None, help_text=None, **kw):
         assert help_text is None
         assert action is None
         kw.update(spec)
-        action_spec = spec.get('action', None)
+        action_spec = spec.get("action", None)
         if action_spec is not None:
             kw.update(action=resolve_action(action_spec))
     else:
-        a = resolve_action(spec, action)
+        ba = resolve_action(spec, action)
         # if str(spec).startswith("webshop"):
         #     print("20210319", spec, action, a)
-        kw.update(action=a)
+        kw.update(action=ba)
         if help_text is None:
-            if a is a.actor.default_action:
-                help_text = a.actor.help_text or a.action.help_text
-            else:
-                help_text = a.action.help_text
+            ba.get_help_text()
         if help_text is not None:
             kw.update(help_text=help_text)
     return MenuItem(**kw)
@@ -237,10 +238,10 @@ class Menu(MenuItem):
         Dynamically removes empty menu entries and useless separators.
         Collapses menu with only one item into their parent.
         """
-        while len(self.items) and self.items[-1].label.startswith('-'):
+        while len(self.items) and self.items[-1].label.startswith("-"):
             del self.items[-1]
 
-        while len(self.items) and self.items[0].label.startswith('-'):
+        while len(self.items) and self.items[0].label.startswith("-"):
             del self.items[0]
 
         for mi in self.items:
@@ -255,7 +256,7 @@ class Menu(MenuItem):
                     if not self.avoid_lonely_items:
                         newitems.append(mi)
                     elif len(mi.items) == 1:
-                        if not mi.items[0].label.startswith('-'):
+                        if not mi.items[0].label.startswith("-"):
                             if self.parent is None:
                                 newitems.append(mi)
                             else:
@@ -277,18 +278,24 @@ class Menu(MenuItem):
 
     def add_instance_action(self, obj, **kw):
         """
-        Add an action that displays the given database object instance in
-        a detail form for editing.
 
-        Used e.g. for the :menuselection`Configure --> System --> Site
-        configuration` command.  Or for the :guilabel:`[My settings]`
-        quicklink in :ref:`care`.
+        Add a menu item that will run the specified action on the given
+        :term:`database row`.
+
+        Default for ``action`` is to open the detail view on the database row.
+
+        Used for the :menuselection:`Configure --> System --> Site configuration`
+        command.
+
         """
         kw.update(instance=obj)
-        ba = kw.get('action', None)
+        ba = kw.get("action", None)
         if ba is None:
             ba = obj.get_default_table().detail_action
             kw.update(action=ba)
+            if "label" not in kw:
+                kw.update(label=str(obj))
+
         # elif isinstance(ba, str):
         #     ba = obj.get_default_table().get_action_by_name(ba)
         #     kw.update(action=ba)
@@ -298,23 +305,21 @@ class Menu(MenuItem):
         return self.add_item_instance(MenuItem(name, label, **kw))
 
     def add_separator(self, **kw):
-        if len(self.items) > 0 and not self.items[-1].label == '-':
-            return self.add_item_instance(MenuItem(None, '-', **kw))
+        if len(self.items) > 0 and not self.items[-1].label == "-":
+            return self.add_item_instance(MenuItem(None, "-", **kw))
 
     def add_menu(self, name, label, **kw):
-        return self.add_item_instance(Menu(
-            self.user_type, name, label, self, **kw))
+        return self.add_item_instance(Menu(self.user_type, name, label, self, **kw))
 
     def get_item(self, name):
         return self.items_dict[name]
 
-    #~ def add_url_button(self,url,label):
     def add_url_button(self, url, **kw):
         kw.update(href=url)
         return self.add_item_instance(MenuItem(**kw))
-        #~ self.items.append(dict(
-          #~ xtype='button',text=label,
-          #~ handler=js_code("function() {window.location='%s';}" % url)))
+        # ~ self.items.append(dict(
+        # ~ xtype='button',text=label,
+        # ~ handler=js_code("function() {window.location='%s';}" % url)))
 
     def add_item_instance(self, mi):
         """
@@ -335,15 +340,16 @@ class Menu(MenuItem):
             if old is not None:
                 if mi.label != old.label:
                     raise Exception(
-                        "Menu item %r labelled %s cannot override existing label %s" %
-                        (mi.name, mi.label, old.label))
+                        "Menu item %r labelled %s cannot override existing label %s"
+                        % (mi.name, mi.label, old.label)
+                    )
                 return old
             self.items_dict[mi.name] = mi
         self.items.append(mi)
         return mi
 
-    #~ def get(self,name):
-        #~ return self.items_dict.get(name)
+    # ~ def get(self,name):
+    # ~ return self.items_dict.get(name)
 
     def walk_items(self):
         yield self
@@ -356,12 +362,14 @@ class Toolbar(Menu):
     """
     A top-level menu.
     """
+
     pass
 
 
 def find_menu_item(bound_action):
     from lino.api import rt
-    user_type = rt.models.users.UserTypes.get_by_value('900')
+
+    user_type = rt.models.users.UserTypes.get_by_value("900")
     return user_type.find_menu_item(bound_action)
     # menu = settings.SITE.get_site_menu(settings.SITE.kernel, user_type)
     # return menu.find_item(bound_action)
