@@ -1,16 +1,14 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2022 Rumma & Ko Ltd
+# Copyright 2009-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the :class:`Model` class.
 
-See :ref:`dev.models`, :doc:`/dev/delete`, :doc:`/dev/disable`,
+See :doc:`/dev/models`, :doc:`/dev/delete`, :doc:`/dev/disable`,
 :doc:`/dev/hide`, :doc:`/dev/format`
 
 """
-
-from lino import logger
 import copy
+# from bs4 import BeautifulSoup
 
 from django.db import models
 from django.conf import settings
@@ -20,47 +18,44 @@ from django.utils.translation import gettext_lazy as _
 from django.db.models.signals import pre_delete
 from django.utils.text import format_lazy
 
-from etgen.html import E, forcetext, tostring, join_elems
+from lino import logger
+from lino.utils.html import E, tostring, join_elems
+from lino.utils.soup import sanitize
 
 from lino.core import fields
+from lino.core import atomizer
 from lino.core import signals
 from lino.core import actions
+from lino.core import inject
 
-from .fields import make_remote_field, RichTextField, displayfield
+from lino.core.atomizer import make_remote_field
+from lino.core.exceptions import ChangedAPI
+from .fields import RichTextField, displayfield
 from .utils import error2str
 from .utils import obj2str
 from .diff import ChangeWatcher
 from .utils import obj2unicode
 from .utils import class_dict_items
-from .signals import receiver, on_ui_created, pre_ui_delete, pre_ui_save
+from .signals import receiver, on_ui_created, pre_ui_delete, pre_ui_save, post_ui_save
 from .workflows import ChangeStateAction
-from .tablerequest import TableRequest, sliced_data_iterator
-
-try:
-    import bleach
-except ImportError:
-    bleach = None
+from .requests import ActionRequest, sliced_data_iterator
+from .tables import AbstractTable
 
 
 class Model(models.Model, fields.TableRow):
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
-    allow_cascaded_copy = frozenset()
     allow_cascaded_delete = frozenset()
-
+    allow_cascaded_copy = None
+    # suppress_cascaded_copy = frozenset()
     grid_post = actions.CreateRow()
     submit_insert = actions.SubmitInsert()
-
     allow_merge_action = False
-
     master_data_fields = None
-
     show_in_site_search = True
-
     quick_search_fields = None
-
     quick_search_fields_digit = None
 
     active_fields = frozenset()
@@ -85,13 +80,9 @@ class Model(models.Model, fields.TableRow):
 
     change_watcher_spec = None
     """
-    Internally used by :meth:`watch_changes`
+    Internally used by :mod:`lino.modlib.changes`.
     """
 
-    # disable_create_choice = False
-    # """Whether to disable any automatic creation by learning choosers.
-    # """
-
     _lino_tables = []
     _bleached_fields = []
 
@@ -122,8 +113,9 @@ class Model(models.Model, fields.TableRow):
         # print("201906274 collect_virtual_fields a ", model, model._meta.fields)
         # fieldnames = {f.name for f in model._meta.get_fields()}
         # inject_field() can call this when Models aren't loaded yet.
-        fieldnames = {f.name for f in
-                      model._meta.private_fields + model._meta.local_fields}
+        fieldnames = {
+            f.name for f in model._meta.private_fields + model._meta.local_fields
+        }
         # print("201906274 collect_virtual_fields b ", fieldnames)
         for m, k, v in class_dict_items(model):
             if isinstance(v, fields.VirtualField):
@@ -165,8 +157,7 @@ class Model(models.Model, fields.TableRow):
 
     @classmethod
     def get_data_elem(cls, name):
-
-        if not name.startswith('__'):
+        if not name.startswith("__"):
             rf = make_remote_field(cls, name)
             if rf:
                 return rf
@@ -189,8 +180,11 @@ class Model(models.Model, fields.TableRow):
         # MTI parents (other than self) whether they have a veto .
 
         for b in self.__class__.__bases__:
-            if issubclass(b, models.Model) \
-               and b is not models.Model and not b._meta.abstract:
+            if (
+                issubclass(b, models.Model)
+                and b is not models.Model
+                and not b._meta.abstract
+            ):
                 msg = b._lino_ddh.disable_delete_on_object(
                     self, [self.__class__])
                 if msg is not None:
@@ -198,7 +192,16 @@ class Model(models.Model, fields.TableRow):
         return self.__class__._lino_ddh.disable_delete_on_object(self)
 
     def disabled_fields(self, ar):
-        return set()
+        df = set()
+
+        state = None
+        if self.workflow_state_field is not None:
+            state = getattr(self, self.workflow_state_field.name)
+        for ba in ar.actor.get_actions():
+            if not ba.get_bound_action_permission(ar, self, state):
+                df.add(ba.action.action_name)
+
+        return df
 
     def delete(self, **kw):
         """
@@ -215,10 +218,12 @@ class Model(models.Model, fields.TableRow):
             "Cannot delete %(model)s %(self)s "
             "because %(count)d %(refs)s refer to it."
         ) % dict(
-            self=self, count=n,
+            self=self,
+            count=n,
             model=self._meta.verbose_name,
-            refs=m._meta.verbose_name_plural or m._meta.verbose_name + 's')
-        #~ print msg
+            refs=m._meta.verbose_name_plural or m._meta.verbose_name + "s",
+        )
+        # ~ print msg
         return msg
 
     @classmethod
@@ -236,7 +241,7 @@ class Model(models.Model, fields.TableRow):
         custom chooser.
 
         """
-        for k, v in list(kw.items()):
+        for k, v in kw.items():
             if k in cls.__dict__:
                 raise Exception("Tried to redefine %s.%s" % (cls, k))
             setattr(cls, k, v)
@@ -245,8 +250,8 @@ class Model(models.Model, fields.TableRow):
     def add_active_field(cls, names):
         if isinstance(cls.active_fields, str):
             cls.active_fields = frozenset(
-                fields.fields_list(cls, cls.active_fields))
-        cls.active_fields = cls.active_fields | fields.fields_list(cls, names)
+                atomizer.fields_list(cls, cls.active_fields))
+        cls.active_fields = cls.active_fields | atomizer.fields_list(cls, names)
 
     @classmethod
     def hide_elements(self, *names):
@@ -256,7 +261,7 @@ class Model(models.Model, fields.TableRow):
         self.hidden_elements = self.hidden_elements | set(names)
 
     @classmethod
-    def add_param_filter(cls, qs, lookup_prefix='', **kwargs):
+    def add_param_filter(cls, qs, lookup_prefix="", **kwargs):
         """Add filters to queryset using table parameter fields.
 
         This is called for every simple parameter.
@@ -275,7 +280,6 @@ class Model(models.Model, fields.TableRow):
 
     @classmethod
     def on_analyze(cls, site):
-
         if isinstance(cls.workflow_owner_field, str):
             cls.workflow_owner_field = cls.get_data_elem(
                 cls.workflow_owner_field)
@@ -298,13 +302,15 @@ class Model(models.Model, fields.TableRow):
         bleached_fields = []
         for f in cls._meta.get_fields():
             if isinstance(f, RichTextField):
-                if f.editable and \
-                    (f.bleached is True or
-                        f.bleached is None and settings.SITE.textfield_bleached):
+                if f.editable and (
+                    f.bleached is True
+                    or f.bleached is None and settings.SITE.textfield_bleached
+                ):
                     bleached_fields.append(f)
         cls._bleached_fields = tuple(bleached_fields)
-        if hasattr(cls, 'bleached_fields'):
-            raise ChangedAPI("Replace bleached_fields by bleached=True on each field")
+        if hasattr(cls, "bleached_fields"):
+            raise ChangedAPI(
+                "Replace bleached_fields by bleached=True on each field")
 
     @classmethod
     def create_from_choice(cls, text, ar=None, save=True):
@@ -314,14 +320,15 @@ class Model(models.Model, fields.TableRow):
         if values is None:
             raise ValidationError(
                 _("Cannot create {obj} from '{text}'").format(
-                obj=cls._meta.verbose_name, text=text))
+                    obj=cls._meta.verbose_name, text=text
+                )
+            )
         obj = cls(**values)
         obj.full_clean()
         if save:
             obj.save()
         return obj
 
-
     @classmethod
     def choice_text_to_dict(cls, text, ar):
         return None
@@ -329,9 +336,9 @@ class Model(models.Model, fields.TableRow):
     @classmethod
     def lookup_or_create(model, lookup_field, value, **known_values):
         """
-        Look up whether there is a model instance having
-        `lookup_field` with value `value`
-        (and optionally other `known_values` matching exactly).
+
+        Look up whether there is a row having value `value` for field
+        `lookup_field` (and optionally other `known_values` matching exactly).
 
         If it doesn't exist, create it and emit an
         :attr:`auto_create <lino.core.signals.auto_create>` signal.
@@ -351,7 +358,7 @@ class Model(models.Model, fields.TableRow):
                 lookup_field.name, value, **known_values)
         else:
             if isinstance(lookup_field, models.CharField):
-                fkw[lookup_field.name + '__iexact'] = value
+                fkw[lookup_field.name + "__iexact"] = value
             else:
                 fkw[lookup_field.name] = value
             flt = models.Q(**fkw)
@@ -361,7 +368,11 @@ class Model(models.Model, fields.TableRow):
             if qs.count() > 1:
                 logger.warning(
                     "%d %s instances having %s=%r (I'll return the first).",
-                    qs.count(), model.__name__, lookup_field.name, value)
+                    qs.count(),
+                    model.__name__,
+                    lookup_field.name,
+                    value,
+                )
             return qs[0]
         # ~ known_values[lookup_field.name] = value
         obj = model(**known_values)
@@ -369,14 +380,14 @@ class Model(models.Model, fields.TableRow):
         try:
             obj.full_clean()
         except ValidationError as e:
-            raise ValidationError("Failed to auto_create %s : %s" %
-                                  (obj2str(obj), e))
+            raise ValidationError(
+                "Failed to auto_create %s : %s" % (obj2str(obj), e))
         obj.save()
         signals.auto_create.send(obj, known_values=known_values)
         return obj
 
     @classmethod
-    def quick_search_filter(model, search_text, prefix=''):
+    def quick_search_filter(model, search_text, prefix=""):
         """Return the filter expression to apply when a quick search text is
         specified.
 
@@ -392,9 +403,14 @@ class Model(models.Model, fields.TableRow):
                 w = w[1:]
                 char_search = False
             if w.isdigit():
-                for fn in model.quick_search_fields_digit:
-                    kw = {prefix + fn.name: int(w)}
-                    q = q | models.Q(**kw)
+                try:
+                    i = int(w)
+                except ValueError:
+                    char_search = True
+                else:
+                    for fn in model.quick_search_fields_digit:
+                        kw = {prefix + fn.name: i}
+                        q = q | models.Q(**kw)
             if char_search:
                 for fn in model.quick_search_fields:
                     kw = {prefix + fn.name + "__icontains": w}
@@ -406,37 +422,30 @@ class Model(models.Model, fields.TableRow):
         pass
 
     def before_ui_save(self, ar, cw):
-        for f, old, new in self.fields_to_bleach():
+        for f, old, new in self.fields_to_bleach(save=True, ar=ar):
             setattr(self, f.name, new)
 
-    def fields_to_bleach(self):
+    def fields_to_bleach(self, **kwargs):
+        for f in self._bleached_fields:
+            old = getattr(self, f.name)
+            if old is None:
+                continue
 
-        if bleach:
-            for f in self._bleached_fields:
-                old = getattr(self, f.name)
-                if old is None:
-                    continue
-                # print("20190626", self, f, old)
-                if not old.startswith("<"):
-                    continue
-                try:
-                    new = bleach.clean(
-                        old, tags=settings.SITE.bleach_allowed_tags,
-                        attributes=settings.SITE.bleach_allowed_attributes,
-                        strip=True)
-                except TypeError as e:
-                    logger.warning(
-                        "Could not bleach %r : %s (%s)", old, e, self)
-                    continue
-                if old != new:
-                    logger.debug(
-                        "Bleaching %s from %r to %r", f.name, old, new)
-                    yield f, old, new
+            if getattr(f, "format") == "plain":
+                # assert BeautifulSoup(old, "html.parser").find() is None
+                new = old
+            else:
+                new = sanitize(old, **kwargs)
+            if old != new:
+                logger.debug("Bleaching %s from %r to %r", f.name, old, new)
+                yield f, old, new
 
+    updatable_panels = None
 
     def after_ui_save(self, ar, cw):
         # Invalidate disabled_fields cache
         self._disabled_fields = None
+        post_ui_save.send(sender=self.__class__, instance=self, ar=ar, cw=cw)
 
     def after_ui_create(self, ar):
         # print(19062017, "Ticket 1910")
@@ -449,7 +458,11 @@ class Model(models.Model, fields.TableRow):
         elem.before_ui_save(ar, None)
         elem.save(force_insert=True)
         # yes, `on_ui_created` comes *after* save()
-        on_ui_created.send(elem, request=ar.request)
+        if ar.request is not None:
+            # TODO: change on_ui_created API to accept ar instead of request.
+            # Because e.g. AddCommentField doesn't currently trigger
+            # on_ui_created although it should.
+            on_ui_created.send(elem, request=ar.request)
         ar.selected_rows.append(elem)
         elem.after_ui_create(ar)
         if ar and ar.actor:
@@ -457,14 +470,20 @@ class Model(models.Model, fields.TableRow):
         elem.after_ui_save(ar, None)
 
     def save_watched_instance(elem, ar, watcher):
+        # raise Exception("20250726")
         if watcher.is_dirty():
             # pre_ui_save.send(sender=elem.__class__, instance=elem, ar=ar)
             # elem.before_ui_save(ar, watcher)
             elem.save(force_update=True)
             watcher.send_update(ar)
             ar.success(_("%s has been updated.") % obj2unicode(elem))
-        else:
-            ar.success(_("%s : nothing to save.") % obj2unicode(elem))
+        # else:
+        #     ar.success(_("%s : nothing to save.") % obj2unicode(elem))
+        # 20250726 The "nothing to save" message is confusing e.g. in
+        # ratings.ResponsesByExam when setting score1 or score2. These virtual
+        # fields store the value in their respective ChallengeRating object but
+        # the examResponse remains unchanged.
+
         elem.after_ui_save(ar, watcher)
 
     def delete_instance(self, ar):
@@ -472,14 +491,14 @@ class Model(models.Model, fields.TableRow):
         self.delete()
 
     def get_row_permission(self, ar, state, ba):
-        """Returns True or False whether this database object gives permission
+        """Returns True or False whether this database row gives permission
         to the ActionRequest `ar` to run the specified action.
 
         """
         return ba.get_bound_action_permission(ar, self, state)
 
     def update_owned_instance(self, controllable):
-        #~ print '20120627 tools.Model.update_owned_instance'
+        # ~ print '20120627 tools.Model.update_owned_instance'
         pass
 
     def after_update_owned_instance(self, controllable):
@@ -521,6 +540,7 @@ class Model(models.Model, fields.TableRow):
         """
         watcher = ChangeWatcher(row)
         # assert hasattr(row, state_field.attname)
+        row.before_ui_save(ar, watcher)  # added 20250312 for #5976
         old = getattr(row, state_field.attname)
         target_state.choicelist.before_state_change(row, ar, old, target_state)
         row.before_state_change(ar, old, target_state)
@@ -541,24 +561,43 @@ class Model(models.Model, fields.TableRow):
         """
         pass
 
+    def as_ref_prefix(self, ar=None):
+        """
+        The return value is used in :class:`lino.mixins.Referrable` to create
+        a further distinction between :attr:`ref <lino.mixins.Referrable.ref>` (s) (references)
+
+        For example, when the plugin *ledgers* is installed to have multiple
+        ledgers (one per each company) available on a site the
+        references for the :class:`Journal <lino_xl.lib.accounting.Journal>` 's
+        such as **SLS** (for Sales journals) are made distinct by calling
+        this method on instances of :class:`Company <lino_xl.lib.contacts.Company>`
+        to create unique **ref** prefixes. So, for each company the **ref** for
+        sales journal will be like: "**C#XXX_SLS**", where *XXX* is the company ID.
+        """
+        return f"{self.__class__.__name__[0]}#{self.id}_"
+
     def as_search_item(self, ar):
         def _get_attr(attr):
             value = getattr(self, attr)
-            if value.startswith('<'):
+            if value.startswith("<"):
                 return value
             return tostring(E.p(value))
-        if hasattr(self, 'body_short_preview'):
-            content = _get_attr('body_short_preview')
-        elif hasattr(self, 'description'):
-            content = _get_attr('description')
+
+        if hasattr(self, "body_short_preview"):
+            content = _get_attr("body_short_preview")
+        elif hasattr(self, "description"):
+            content = _get_attr("description")
         else:
             content = ""
         return tostring(E.h4(ar.obj2html(self))) + ar.parse_memo(content)
 
     @property
     def edge_ngram_field(self):
-        values = [str(getattr(self, field.name)) for field in self.quick_search_fields + self.quick_search_fields_digit]
-        return ' '.join(values)
+        values = [
+            str(getattr(self, field.name))
+            for field in self.quick_search_fields + self.quick_search_fields_digit
+        ]
+        return " ".join(values)
 
     @property
     def haystack_rendered_field(self):
@@ -571,7 +610,7 @@ class Model(models.Model, fields.TableRow):
         database query and show this value as the search result.
 
         """
-        ar = self.__class__.get_default_table().request()
+        ar = self.__class__.get_default_table().create_request()
         return self.as_search_item(ar)
 
     # def summary_row(self, ar, **kw):
@@ -581,37 +620,30 @@ class Model(models.Model, fields.TableRow):
     def name_column(self, ar):
         return str(self)
 
-    # @fields.displayfield(_("Description"))
-    # def mobile_item(self, ar):
-    #     if ar is None:
-    #         return ''
-    #     return E.div(*forcetext(self.get_mobile_list_item_elems(ar)))
-    #
-    # def get_mobile_list_item_elems(self, ar):
-    #     return [self.obj2href(ar)]
-
     @fields.htmlbox()
     def navigation_panel(self, ar):
-        if not isinstance(ar, TableRequest):
+        # if not isinstance(ar, ActionRequest):
+        if ar is None or ar.actor is None or not issubclass(ar.actor, AbstractTable):
             return None
         items = []
         navinfo = ar.actor.get_navinfo(ar, self)
         if navinfo is None:
             return None
         nav_items = [
-            ar.goto_pk(navinfo['first'], "⏮ "), # 23EE
-            ar.goto_pk(navinfo['prev_page'], "⏪ "), # 23Ea
-            ar.goto_pk(navinfo['prev'], "◀ "),  # 25C0
-            navinfo['message'],
-            ar.goto_pk(navinfo['next'], " ▶"),  # 25B6
-            ar.goto_pk(navinfo['next_page'], " ⏩"), # 23E9
-            ar.goto_pk(navinfo['last'], " ⏭") # 0x23ED
-            ]
+            ar.goto_pk(navinfo["first"], "⏮ "),  # 23EE
+            ar.goto_pk(navinfo["prev_page"], "⏪ "),  # 23Ea
+            ar.goto_pk(navinfo["prev"], "◀ "),  # 25C0
+            navinfo["message"],
+            ar.goto_pk(navinfo["next"], " ▶"),  # 25B6
+            ar.goto_pk(navinfo["next_page"], " ⏩"),  # 23E9
+            ar.goto_pk(navinfo["last"], " ⏭"),  # 0x23ED
+        ]
         items.append(E.p(*nav_items))
         # print("20210325", navinfo)
         # raise Exception("20")
 
-        qs = sliced_data_iterator(ar.data_iterator, navinfo['offset'], ar.limit)
+        qs = sliced_data_iterator(
+            ar.data_iterator, navinfo["offset"], ar.limit)
         for obj in qs:
             if obj.pk == self.pk:
                 items.append(E.b(str(obj)))
@@ -621,14 +653,13 @@ class Model(models.Model, fields.TableRow):
         # return E.div(*items, **{'class': 'l-component'})
         # items = join_elems([E.p(i) for i in items])
         items = join_elems(items, sep=E.br)
-        kwargs = {'class': 'list-item', 'style': 'width:15em;'}
+        kwargs = {"class": "list-item", "style": "width:15em;"}
         return ar.html_text(E.div(*items, **kwargs))
 
-
     @displayfield(_("Workflow"))
     def workflow_buttons(self, ar):
         if ar is None:
-            return ''
+            return ""
         return self.get_workflow_buttons(ar)
 
     def get_workflow_buttons(obj, ar):
@@ -636,40 +667,42 @@ class Model(models.Model, fields.TableRow):
         actor = ar.actor
         # print(20170102, actor)
         state = actor.get_row_state(obj)
-        sep = ''
+        sep = ""
         show = True  # whether to show the state
+
         # logger.info('20161219 workflow_buttons %r', state)
 
         def show_state():
             l.append(sep)
-            #~ l.append(E.b(unicode(state),style="vertical-align:middle;"))
+            # ~ l.append(E.b(unicode(state),style="vertical-align:middle;"))
             if state.button_text:
-                l.append(E.b(u"{} {}".format(state.button_text, state)))
+                l.append(E.b("{} {}".format(state.button_text, state)))
             else:
                 l.append(E.b(str(state)))
             # l.append(E.b(str(state)))
-            #~ l.append(u" » ")
-            #~ l.append(u" \u25b8 ")
-            #~ l.append(u" \u2192 ")
-            #~ sep = u" \u25b8 "
+            # ~ l.append(u" » ")
+            # ~ l.append(u" \u25b8 ")
+            # ~ l.append(u" \u2192 ")
+            # ~ sep = u" \u25b8 "
 
         df = actor.get_disabled_fields(obj, ar)
         # print(20170909, df)
-        for ba in actor.get_actions():
-            assert ba.actor == actor  # 20170102
-            if ba.action.show_in_workflow:
-                # if actor.model.__name__ == 'Vote':
-                #     if ba.action.__class__.__name__ == 'MarkVoteAssigned':
-                #         print(20170115, actor, ar.get_user())
-                if ba.action.action_name not in df:
-                  if actor.get_row_permission(obj, ar, state, ba):
-                    if show and isinstance(ba.action, ChangeStateAction):
-                        show_state()
-                        sep = u" \u2192 "  # "→"
-                        show = False
-                    l.append(sep)
-                    l.append(ar.action_button(ba, obj))
-                    sep = ' '
+        if 'workflow_buttons' not in df:
+            for ba in actor.get_actions():
+                assert ba.actor == actor  # 20170102
+                if ba.action.show_in_workflow:
+                    # if actor.model.__name__ == 'Vote':
+                    #     if ba.action.__class__.__name__ == 'MarkVoteAssigned':
+                    #         print(20170115, actor, ar.get_user())
+                    if ba.action.action_name not in df:
+                        if actor.get_row_permission(obj, ar, state, ba):
+                            if show and isinstance(ba.action, ChangeStateAction):
+                                show_state()
+                                sep = " \u2192 "  # "→"
+                                show = False
+                            l.append(sep)
+                            l.append(ar.action_button(ba, obj))
+                            sep = " "
         if state and show:
             show_state()
         return E.span(*l)
@@ -693,59 +726,15 @@ class Model(models.Model, fields.TableRow):
     #     return self.preview(ar)
     #     #~ username = kw.pop('username',None)
 
-    def get_typed_instance(self, model):
-        """
-        Used when implementing :ref:`polymorphism`.
-        """
-        return self
-
-
-#     def is_attestable(self):
-#         """Override this to disable the :class:`lino_xl.lib.excerpts.CreateExcerpt`
-# action on individual instances.
-
-#         """
-#         return True
-
-    @classmethod
-    def get_chooser_for_field(cls, fieldname):
-        d = getattr(cls, '_choosers_dict', {})
-        # if fieldname.endswith("__municipality"):
-        #     print("20200425 Model.get_chooser_for_field", cls, fieldname, d)
-        return d.get(fieldname, None)
-
-    def filename_root(self):
-        return self._meta.app_label + '.' + self.__class__.__name__ \
-            + '-' + str(self.pk)
+    def get_printable_target_stem(self):
+        return self._meta.app_label + "." + self.__class__.__name__ + "-" + str(self.pk)
 
     @classmethod
     def get_request_queryset(cls, ar, **filter):
-        """Return the base queryset for tables on this object.
-
-        The optional `filter` keyword arguments, if present, are
-        applied as additional filter. This is used only in UNION
-        tables on abstract model mixins where filtering cannot be done
-        after the join.
-
-        """
-        qs = cls.get_user_queryset(ar.get_user(), **filter)
-        if ar.actor.only_fields is not None:
-            qs = qs.only(ar.actor.only_fields)
-        return qs
-
-    @classmethod
-    def get_user_queryset(cls, user, **filter):
-        """
-        Get the base queryset, used for user level row filtering in :class:`lino_xl.lib.tickets.Ticket`
-        """
         if filter:
             return cls.objects.filter(**filter)
         return cls.objects.all()
 
-    @classmethod
-    def get_title_tags(self, ar):
-        return []
-
     @classmethod
     def resolve_states(cls, states):
         """Convert the given string `states` into a set of state objects.
@@ -759,8 +748,7 @@ class Model(models.Model, fields.TableRow):
             return None
         elif isinstance(states, str):
             fld = cls.workflow_state_field
-            return set(
-                [fld.choicelist.get_by_name(x) for x in states.split()])
+            return set([fld.choicelist.get_by_name(x) for x in states.split()])
         elif isinstance(states, set):
             return states
         raise Exception(
@@ -782,7 +770,6 @@ class Model(models.Model, fields.TableRow):
         vfield_name = "pick_" + fldname
 
         class QuickSetChoice(actions.Action):
-
             label = cls.verbose_name
             icon_name = None
             show_in_toolbar = False
@@ -822,12 +809,18 @@ class Model(models.Model, fields.TableRow):
                     if fldname in ar.actor.get_disabled_fields(self, ar):
                         pass  # elems.append(label)
                     else:
-                        elems.append(ar.action_button(
-                            ba, self, label=label, request_kwargs=kw, title=v.text))
+                        elems.append(
+                            ar.action_button(
+                                ba, self, label=label, request_kwargs=kw, title=v.text
+                            )
+                        )
             return E.p(*join_elems(elems, sep=" | "))
 
         setattr(model, vfield_name, pick_choice)
 
+    @classmethod
+    def update_field(cls, *args, **kwargs):
+        inject.update_field(cls, *args, **kwargs)
 
     @classmethod
     def django2lino(cls, model):
@@ -849,7 +842,7 @@ class Model(models.Model, fields.TableRow):
 
         for k in LINO_MODEL_ATTRIBS:
             if not hasattr(model, k):
-                #~ setattr(model,k,getattr(dd.Model,k))
+                # ~ setattr(model,k,getattr(dd.Model,k))
                 # if k in cls.__dict__:
                 #     setattr(model, k, cls.__dict__[k])
                 # else:
@@ -859,8 +852,8 @@ class Model(models.Model, fields.TableRow):
                         setattr(model, k, b.__dict__[k])
                         break
                 # setattr(model, k, getattr(cls, k))
-                #~ model.__dict__[k] = getattr(dd.Model,k)
-                #~ logger.info("20121127 Install default %s for %s",k,model)
+                # ~ model.__dict__[k] = getattr(dd.Model,k)
+                # ~ logger.info("20121127 Install default %s for %s",k,model)
 
     @classmethod
     def get_subclasses_graph(self):
@@ -877,6 +870,7 @@ class Model(models.Model, fields.TableRow):
 
         """
         from lino.api import rt
+
         pairs = []
         collected = set()
 
@@ -894,12 +888,17 @@ class Model(models.Model, fields.TableRow):
                         pairs.append((p, c))
                         collected.add(c)
                     collect(c)
+
         collect(self)
-        s = '\n'.join(
-            ['    "%s" -> "%s"' % (
-                p._meta.verbose_name, c._meta.verbose_name)
-             for p, c in pairs])
-        return """
+        s = "\n".join(
+            [
+                '    "%s" -> "%s"' % (p._meta.verbose_name,
+                                      c._meta.verbose_name)
+                for p, c in pairs
+            ]
+        )
+        return (
+            """
 
 .. graphviz::
 
@@ -907,7 +906,9 @@ class Model(models.Model, fields.TableRow):
 %s
   }
 
-""" % s
+"""
+            % s
+        )
 
     @classmethod
     def print_subclasses_graph(self):
@@ -915,71 +916,85 @@ class Model(models.Model, fields.TableRow):
 
 
 LINO_MODEL_ATTRIBS = (
-    'collect_virtual_fields',
-    'define_action',
-    'get_workflow_buttons',
-    'workflow_buttons',
-    'delete_instance',
-    'setup_parameters',
-    'add_param_filter',
-    'save_new_instance',
-    'save_watched_instance',
-    'save_existing_instance',
-    '_widget_options',
-    'set_widget_options',
-    'get_widget_options',
-    'get_chooser_for_field',
-    'get_detail_action',
+    "collect_virtual_fields",
+    "define_action",
+    "get_workflow_buttons",
+    "workflow_buttons",
+    "delete_instance",
+    "setup_parameters",
+    "param_defaults",
+    "add_param_filter",
+    "save_new_instance",
+    "save_watched_instance",
+    "save_existing_instance",
+    "_widget_options",
+    "extra_display_modes",
+    "set_widget_options",
+    "get_widget_options",
+    "get_chooser_for_field",
+    "get_detail_action",
     # 'get_print_language',
-    'get_row_permission',
+    "get_row_permission",
     # 'get_excerpt_options',
     # 'is_attestable',
-    'as_summary_row',
-    'as_paragraph',
-    'as_page',
-    'get_data_elem',
-    'get_param_elem',
-    'after_ui_save',
-    'preferred_foreignkey_width',
-    'before_ui_save',
-    'allow_cascaded_delete',
-    'allow_cascaded_copy',
-    'workflow_state_field',
-    'workflow_owner_field',
-    'disabled_fields',
-    'get_choices_text',
+    # "as_summary_row",
+    "get_data_elem",
+    "get_param_elem",
+    "after_ui_save",
+    "preferred_foreignkey_width",
+    "before_ui_save",
+    "allow_cascaded_delete",
+    "allow_cascaded_copy",
+    # "suppress_cascaded_copy",
+    "workflow_state_field",
+    "workflow_owner_field",
+    "disabled_fields",
+    "get_choices_text",
     # 'summary_row',
-    'submit_insert',
-    'active_fields',
-    'hidden_columns',
-    'hidden_elements',
-    'get_simple_parameters',
-    'get_request_queryset',
-    'get_user_queryset',
-    'get_title_tags',
-    'get_default_table',
-    'get_default_table',
-    'get_layout_aliases',
-    'edge_ngram_field',
-    'as_search_item',
-    'as_story_item',
-    'haystack_rendered_field',
-    'get_related_project',
-    'obj2href',
-    'quick_search_fields',
-    'quick_search_fields_digit',
-    'change_watcher_spec',
-    'on_analyze',
-    'disable_delete',
-    'lookup_or_create',
-    'quick_search_filter',
-    'on_duplicate',
-    'on_create',
-    'error2str',
-    'get_typed_instance',
-    'print_subclasses_graph',
-    'grid_post', 'submit_insert', 'delete_veto_message', '_lino_tables',
-    'show_in_site_search', 'allow_merge_action')
+    "submit_insert",
+    "active_fields",
+    "hidden_columns",
+    "hidden_elements",
+    "get_simple_parameters",
+    "get_request_queryset",
+    "get_request_words",
+    "get_title_tags",
+    "get_default_table",
+    "get_default_table",
+    "get_layout_aliases",
+    "edge_ngram_field",
+    "as_str",
+    "get_str_words",
+    "as_summary_item",
+    "as_paragraph",
+    "as_page",
+    "as_story_item",
+    "as_ref_prefix",
+    "as_search_item",
+    "haystack_rendered_field",
+    "get_related_project",
+    "quick_search_fields",
+    "quick_search_fields_digit",
+    "change_watcher_spec",
+    "on_analyze",
+    "disable_delete",
+    "lookup_or_create",
+    "quick_search_filter",
+    "on_duplicate",
+    "on_create",
+    "error2str",
+    "print_subclasses_graph",
+    "disable_create",
+    "override_column_headers",
+    "grid_post",
+    "submit_insert",
+    "delete_veto_message",
+    "_lino_tables",
+    "show_in_site_search",
+    "allow_merge_action",
+    "get_overview_elems",
+    "get_parent_links",
+)
 
 
 @receiver(pre_delete)
@@ -994,7 +1009,7 @@ def pre_delete_handler(sender, instance=None, **kw):
 
     It seems that Django deletes *generic related objects* only if
     the object being deleted has a `GenericRelation
-    <https://docs.djangoproject.com/en/5.0/ref/contrib/contenttypes/#django.contrib.contenttypes.fields.GenericRelation>`_
+    <https://docs.djangoproject.com/en/5.2/ref/contrib/contenttypes/#django.contrib.contenttypes.fields.GenericRelation>`_
     field (according to `Why won't my GenericForeignKey cascade
     when deleting?
     <https://stackoverflow.com/questions/6803018/why-wont-my-genericforeignkey-cascade-when-deleting>`_).
@@ -1028,9 +1043,11 @@ def pre_delete_handler(sender, instance=None, **kw):
                     qs.model, qs.count()))
     for qs in must_cascade:
         if qs.count():
-            logger.info("Deleting %d %s before deleting %s",
-                        qs.count(),
-                        qs.model._meta.verbose_name_plural,
-                        obj2str(instance))
+            logger.info(
+                "Deleting %d %s before deleting %s",
+                qs.count(),
+                qs.model._meta.verbose_name_plural,
+                obj2str(instance),
+            )
             for obj in qs:
                 obj.delete()
