@@ -1,4 +1,4 @@
-    # -*- coding: UTF-8 -*-
+# -*- coding: UTF-8 -*-
 # Copyright 2012-2020 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """
@@ -26,15 +26,19 @@ class State(choicelists.Choice):
 
     transition = None  # how to get into this state
 
-    def add_transition(self, label=None,
-                       help_text=None,
-                       # notify=False,
-                       name=None,
-                       #~ icon_file=None,
-                       icon_name=None,
-                       debug_permissions=None,
-                       required_states=None,
-                       required_roles=None, **kwargs):
+    def add_transition(
+        self,
+        label=None,
+        help_text=None,
+        # notify=False,
+        name=None,
+        # ~ icon_file=None,
+        icon_name=None,
+        debug_permissions=None,
+        required_states=None,
+        required_roles=None,
+        **kwargs,
+    ):
         """
         Declare an transition action which makes an object enter this
         state.
@@ -79,8 +83,8 @@ class State(choicelists.Choice):
             #         "Cannot specify notify when using custom action")
             # if debug_permissions:
             #     kwargs.update(debug_permissions=debug_permissions)
-                # raise Exception(
-                #     "Cannot specify debug_permissions when using custom action")
+            # raise Exception(
+            #     "Cannot specify debug_permissions when using custom action")
             for a in workflow_actions:
                 if isinstance(a, label):
                     raise Exception("Duplicate transition class %s" % a)
@@ -100,12 +104,11 @@ class State(choicelists.Choice):
             a = ChangeStateAction(self, required_roles, label=label, **kwargs)
 
         if name is None:
-            name = 'wf' + str(i + 1)
+            name = "wf" + str(i + 1)
 
         for x in workflow_actions:
             if x.action_name == name:
-                raise Exception(
-                    "Duplicate transition name {0}".format(name))
+                raise Exception("Duplicate transition name {0}".format(name))
 
         if debug_permissions:
             a.debug_permissions = debug_permissions
@@ -115,8 +118,6 @@ class State(choicelists.Choice):
         self.transition = a
 
 
-
-
 class Workflow(choicelists.ChoiceList):
     """
     A workflow is a specialized choicelist used for defining the
@@ -137,6 +138,7 @@ class Workflow(choicelists.ChoiceList):
 
 
     """
+
     item_class = State
 
     verbose_name = _("State")
@@ -157,7 +159,7 @@ class Workflow(choicelists.ChoiceList):
         # logger.info("20150602 Workflow.on_analyze %s", cls)
         used_on_models = []
         for fld in cls._fields:
-            model = getattr(fld, 'model', None)
+            model = getattr(fld, "model", None)
             if model:
                 used_on_models.append(model)
                 # logger.info("20150602 %s, %s", model, cls.workflow_actions)
@@ -182,7 +184,6 @@ class Workflow(choicelists.ChoiceList):
                 cls.verbose_name_plural = _("States")
         cls.sort()
 
-
     @classmethod
     def before_state_change(cls, obj, ar, oldstate, newstate):
         pass
@@ -193,22 +194,18 @@ class Workflow(choicelists.ChoiceList):
 
     @classmethod
     def override_transition(cls, **kw):
-        """
-        """
+        """ """
         for name, cl in kw.items():
             found = False
             for i, a in enumerate(cls.workflow_actions):
                 if a.action_name == name:
-                    new = cl(
-                        a.target_state, a.required_roles,
-                        sort_index=a.sort_index)
+                    new = cl(a.target_state, a.required_roles, sort_index=a.sort_index)
                     new.attach_to_workflow(cls, name)
                     cls.workflow_actions[i] = new
                     found = True
                     break
             if not found:
-                raise Exception(
-                    "There is no workflow action named {0}".format(name))
+                raise Exception("There is no workflow action named {0}".format(name))
 
     @classmethod
     def clear_transitions(cls):
@@ -229,13 +226,13 @@ class ChangeStateAction(actions.Action):
     show_in_toolbar = False
     show_in_workflow = True
     readonly = False
+
     # sort_index = 200
 
-    def __init__(self, target_state, required_roles=None,
-                 help_text=None, **kw):
+    def __init__(self, target_state, required_roles=None, help_text=None, **kw):
         self.target_state = target_state
-        assert 'required' not in kw
-        assert 'required_roles' not in kw
+        assert "required" not in kw
+        assert "required_roles" not in kw
         new_required = set(self.required_roles)
         if required_roles is not None:
             new_required |= required_roles
@@ -250,7 +247,7 @@ class ChangeStateAction(actions.Action):
             kw.update(help_text=help_text)
 
         super(ChangeStateAction, self).__init__(**kw)
-        #~ logger.info('20120930 ChangeStateAction %s %s', actor,target_state)
+        # ~ logger.info('20120930 ChangeStateAction %s %s', actor,target_state)
         if self.label is None:
             self.label = target_state.text
 
@@ -258,7 +255,8 @@ class ChangeStateAction(actions.Action):
             self.button_text = target_state.button_text
 
         if self.icon_name:
-            self.help_text = format_lazy(u"{}. {}",self.label,self.help_text)
+            self.help_text = format_lazy(
+                "{}. {}", self.label, self.help_text)
 
     # def get_action_permission(self, ar, obj, state):
     #     if not super(ChangeStateAction, self).get_action_permission(ar, obj, state):
@@ -267,7 +265,6 @@ class ChangeStateAction(actions.Action):
     #         return False
     #     return True
 
-
     def run_from_ui(self, ar):
         # print(20170116, ar.action_param_values)
         for row in ar.selected_rows:
@@ -278,9 +275,7 @@ class ChangeStateAction(actions.Action):
     confirmation_msg_template = None
 
     def get_confirmation_msg_context(self, ar, obj, **kwargs):
-        kwargs.update(
-            user=ar.get_user(),
-            state=self.target_state)
+        kwargs.update(user=ar.get_user(), state=self.target_state)
         return kwargs
 
     def before_execute(self, ar, obj):
@@ -294,12 +289,7 @@ class ChangeStateAction(actions.Action):
     def execute(self, ar, obj):
         def doit(ar):
             self.before_execute(ar, obj)
-            obj.set_workflow_state(
-                ar,
-                ar.actor.workflow_state_field,
-                self.target_state)
-
-
+            obj.set_workflow_state(ar, ar.actor.workflow_state_field, self.target_state)
 
         if self.confirmation_msg_template is None:
             doit(ar)
