@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2021 Rumma & Ko Ltd
+# Copyright 2021-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
 from lino import logger
@@ -18,42 +18,48 @@ from lino.utils.test import DemoTestCase
 
 
 class TestCase(DemoTestCase):
-
-    tested_urls = ('/', '/?su=3', '/?su=1234')
+    tested_urls = ("/", "/?su=3", "/?su=123")
 
     def test_get(self):
-
         if False:
             # switched off because it causes a message "Not found: /foo"
             res = self.client.get("/foo")
             self.assertEqual(
-                res.status_code, 404,
-                "Status code {} other than 404".format(res.status_code))
+                res.status_code,
+                404,
+                "Status code {} other than 404".format(res.status_code),
+            )
 
         for url in self.tested_urls:
             # print("20200513", url)
             res = self.client.get(url)
             self.assertEqual(
-                res.status_code, 200,
-                "Status code %s other than 200 for anonymous on GET %s" % (
-                    res.status_code, url))
+                res.status_code,
+                200,
+                "Status code %s other than 200 for anonymous on GET %s"
+                % (res.status_code, url),
+            )
 
         if settings.SITE.user_model:
             for user in settings.SITE.user_model.objects.all():
                 if user.user_type:
-                    d = self.login(user.username, '1234')
+                    d = self.login(user.username, dd.plugins.users.demo_password)
                     # self.client.force_login(user)
                     for url in self.tested_urls:
                         # logger.debug("%s gets %s", user.username, url)
                         print("{} gets {}".format(user.username, url))
                         res = self.client.get(url)
                         self.assertEqual(
-                            res.status_code, 200,
+                            res.status_code,
+                            200,
                             "Status code {} for {} on GET {}".format(
-                                res.status_code, user, url))
+                                res.status_code, user, url
+                            ),
+                        )
 
     def test_ipdict(self):
         if not settings.SITE.use_ipdict:
+            # print("20250126 no test because use_ipdict is False")
             return
         ipdict = dd.plugins.ipdict
 
@@ -62,56 +68,58 @@ class TestCase(DemoTestCase):
 
         # For this test we reduce max_blacklist_time because we are going to
         # simulate a hacker who patiently waits:
-        ipdict.max_blacklist_time = timedelta(seconds=1)
+        ipdict.max_blacklist_time = timedelta(seconds=4)
 
         self.assertEqual(ipdict.ip_records, {})
 
-        def login(pwd):
-            d = self.login('robin', pwd)
-            return d.message
+        def login(pwd, expected):
+            d = self.login("robin", pwd)
+            if d.message != expected:
+                self.fail(f"Expected {expected} but got {d.message} ({d})")
 
-        self.assertEqual(login("bad"), 'Failed to sign in as robin.')
-        rec = ipdict.ip_records['127.0.0.1']
+        login("bad", "Failed to sign in as robin.")
+        rec = ipdict.ip_records["127.0.0.1"]
         self.assertEqual(rec.login_failures, 1)
-        self.assertEqual(login("bad"), 'Failed to sign in as robin.')
+        login("bad", "Failed to sign in as robin.")
         self.assertEqual(rec.login_failures, 2)
-        self.assertEqual(login("bad"), 'Failed to sign in as robin.')
+        login("bad", "Failed to sign in as robin.")
         self.assertEqual(rec.login_failures, 3)
-        self.assertEqual(login("bad"), 'Failed to sign in as robin.')
+        login("bad", "Failed to sign in as robin.")
         self.assertEqual(rec.login_failures, 4)
-        self.assertEqual(login("bad"), 'Too many authentication failures from 127.0.0.1')
+        login("bad", "Too many authentication failures from 127.0.0.1")
 
         # login_failures doesn't continue to increase when the ip is blacklisted:
         self.assertEqual(rec.login_failures, 4)
 
         # Even with the right password you cannot unlock a blacklisted ip
-        self.assertEqual(login("1234"), 'Too many authentication failures from 127.0.0.1')
+        login(dd.plugins.users.demo_password,
+              "Too many authentication failures from 127.0.0.1")
 
         # After max_blacklist_time, the IP gets removed from the blacklist, but
         # every new failure will now blacklist it again, the
         # max_failed_auth_per_ip no longer counts.
 
-        time.sleep(1)
-        self.assertEqual(login("bad"), 'Failed to sign in as robin.')
+        # time.sleep(1.5)
+        time.sleep(5)
+        login("bad", "Failed to sign in as robin.")
         self.assertEqual(rec.login_failures, 5)
-        self.assertEqual(login("bad"), 'Too many authentication failures from 127.0.0.1')
+        login("bad", "Too many authentication failures from 127.0.0.1")
         self.assertEqual(rec.login_failures, 5)
 
-        time.sleep(1)
-        self.assertEqual(login("1234"), 'Now signed in as Robin Rood')
+        time.sleep(5)
+        login(dd.plugins.users.demo_password, "Now signed in as Robin Rood")
 
         # Once you manage to authenticate, your ip address gets removed from the
         # blacklist, i.e. when you log out and in for some reason, you get again
         # max_failed_auth_per_ip attempts
 
         self.assertEqual(ipdict.ip_records, {})
-        self.assertEqual(login("bad"), 'Failed to sign in as robin.')
-        rec = ipdict.ip_records['127.0.0.1']
+        login("bad", "Failed to sign in as robin.")
+        rec = ipdict.ip_records["127.0.0.1"]
         self.assertEqual(rec.login_failures, 1)
 
 
 class Command(BaseCommand):
-
     help = "Run a series of standard read-only tests in this project."
 
     # requires_system_checks = []
