@@ -9,33 +9,28 @@ from lino.management.commands.initdb import CommandError
 
 
 class Command(BaseCommand):
-    """Create a dump for migration tests.
-    """
+    """Create a dump for migration tests."""
 
     def handle(self, *args, **options):
+        # : loop through all demo projects.
+        # skip if the project has no  :xfile:`test_restore.py` file.
+        # dumpdir = "tests/dumps/" + version
+        # python manage.py dump2py dumpdir
+        # error if
 
-
-          # : loop through all demo projects.
-          # skip if the project has no  :xfile:`test_restore.py` file.
-          # dumpdir = "tests/dumps/" + version
-          # python manage.py dump2py dumpdir
-          # error if
-
-
-        fixtures = options.get('fixtures', args)
+        fixtures = options.get("fixtures", args)
         if len(fixtures) > 0:
-            raise CommandError(
-                "This command takes no arguments (got %r)" % fixtures)
+            raise CommandError("This command takes no arguments (got %r)" % fixtures)
 
         if settings.SITE.readonly:
             settings.SITE.logger.info(
-                "Skipped `initdb` on readonly site '%s'.",
-                settings.SETTINGS_MODULE)
+                "Skipped `initdb` on readonly site '%s'.", settings.SETTINGS_MODULE
+            )
             return
 
         args = settings.SITE.demo_fixtures
         if isinstance(args, str):
             args = args.split()
-        options['fixtures'] = args
+        options["fixtures"] = args
         # super(Command, self).handle(*args, **options)
         super(Command, self).handle(**options)
