@@ -2,7 +2,6 @@
 # Copyright 2014 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
 from lino import logger
 
 from optparse import make_option
@@ -18,13 +17,19 @@ class Command(BaseCommand):
 
     def add_arguments(self, parser):
         parser.add_argument(
-            '--noinput', action='store_false',
-            dest='interactive', default=True,
-            help='Do not prompt for input of any kind.')
+            "--noinput",
+            action="store_false",
+            dest="interactive",
+            default=True,
+            help="Do not prompt for input of any kind.",
+        )
         parser.add_argument(
-            '--nochange', action='store_true',
-            dest='nochange', default=False,
-            help='Dry run. Do not actually change anything.')
+            "--nochange",
+            action="store_true",
+            dest="nochange",
+            default=False,
+            help="Dry run. Do not actually change anything.",
+        )
 
     def merge_module(self, name):
         m = import_module(name)
@@ -48,15 +53,14 @@ class Command(BaseCommand):
                     obj.full_clean()
                     if not self.nochange:
                         obj.save()
-                        tpl = 'New %s %s has been created.'
+                        tpl = "New %s %s has been created."
                     else:
-                        tpl = 'New %s %s would have been created.'
+                        tpl = "New %s %s would have been created."
                     logger.info(tpl, M._meta.verbose_name, obj)
                 elif len(stored) == 1:
                     logger.info("Cannot yet update existing %s", obj)
                 else:
-                    logger.info("Found %d existing rows for %s",
-                                len(stored), obj)
+                    logger.info("Found %d existing rows for %s", len(stored), obj)
             else:
                 logger.warning("Ignored non-empty primary key %s", obj)
 
@@ -64,8 +68,8 @@ class Command(BaseCommand):
         if len(args) == 0:
             raise CommandError("I need at least one argument.")
 
-        self.nochange = options['nochange']
-        self.interactive = options['interactive']
+        self.nochange = options["nochange"]
+        self.interactive = options["interactive"]
 
         for name in args:
             self.merge_module(name)
