@@ -5,38 +5,50 @@
 from django.core.management.base import BaseCommand
 from django.conf import settings
 
-
 import sys
-from PyQt5.QtWidgets import (QApplication, QWidget, QPushButton,
-                             QMessageBox, QDesktopWidget, QMainWindow,
-                             QAction, qApp, QTextEdit, QHBoxLayout,
-                             QVBoxLayout)
+from PyQt5.QtWidgets import (
+    QApplication,
+    QWidget,
+    QPushButton,
+    QMessageBox,
+    QDesktopWidget,
+    QMainWindow,
+    QAction,
+    qApp,
+    QTextEdit,
+    QHBoxLayout,
+    QVBoxLayout,
+)
+
 # from PyQt5.QtCore import QCoreApplication
 from PyQt5.QtGui import QIcon
 from lino.api import rt
 from lino.core.menus import Menu  # , MenuItem
 from pathlib import Path
-images_path = Path(settings.STATIC_ROOT) / 'static' / 'images' / 'mjames'
+
+images_path = Path(settings.STATIC_ROOT) / "static" / "images" / "mjames"
+
 
 class ItemCaller(object):
     def __init__(self, win, mi):
         self.mi = mi
-        self.win  = win
+        self.win = win
 
     def __call__(self, event):
         if False:
             QMessageBox.question(
-                self.win, str(self.mi.label),
+                self.win,
+                str(self.mi.label),
                 str(self.mi.help_text),
-                QMessageBox.Yes |
-                QMessageBox.No, QMessageBox.Yes)
+                QMessageBox.Yes | QMessageBox.No,
+                QMessageBox.Yes,
+            )
 
         self.frm = DetailForm(self.win, self.mi)
         self.frm.show()
 
 
 class DetailForm(QWidget):
-
     def __init__(self, win, mi):
         self.mi = mi
         super().__init__(win)
@@ -44,7 +56,6 @@ class DetailForm(QWidget):
         self.initUI()
 
     def initUI(self):
-
         okButton = QPushButton("OK")
         cancelButton = QPushButton("Cancel")
 
@@ -64,28 +75,26 @@ class DetailForm(QWidget):
 
 
 class LinoClient(QMainWindow):
-
     def __init__(self):
         super().__init__()
         self.initUI()
 
     def initUI(self):
-
         textEdit = QTextEdit()
         self.setCentralWidget(textEdit)
 
         self.setGeometry(300, 300, 300, 220)
         self.center()
-        self.setWindowTitle('qtclient.py')
-        self.setWindowIcon(QIcon('../../.static/logo.png'))
-        self.setToolTip('This is a <b>QWidget</b> widget')
+        self.setWindowTitle("qtclient.py")
+        self.setWindowIcon(QIcon("../../.static/logo.png"))
+        self.setToolTip("This is a <b>QWidget</b> widget")
         self.menubar = self.menuBar()
 
-        user_type = rt.models.users.UserTypes.get_by_value('900')
+        user_type = rt.models.users.UserTypes.get_by_value("900")
         menu = settings.SITE.get_site_menu(user_type)
         self.load_menu(menu, self.menubar)
         self.show()
-        self.statusBar().showMessage('Ready')
+        self.statusBar().showMessage("Ready")
 
     def load_menu(self, menu, menubar):
         for mi in menu.items:
@@ -93,8 +102,7 @@ class LinoClient(QMainWindow):
                 submenu = menubar.addMenu(str(mi.label))
                 self.load_menu(mi, submenu)
             else:
-                a = QAction(QIcon(str(images_path / 'cancel.png')),
-                            str(mi.label), self)
+                a = QAction(QIcon(str(images_path / "cancel.png")), str(mi.label), self)
                 if mi.hotkey:
                     a.setShortcut(mi.hotkey)
                 a.setStatusTip(str(mi.help_text))
@@ -102,7 +110,7 @@ class LinoClient(QMainWindow):
                 menubar.addAction(a)
 
         # fileMenu = menubar.addMenu('&File')
-        exitAction = QAction(QIcon('exit.png'), '&Exit', self)
+        exitAction = QAction(QIcon("exit.png"), "&Exit", self)
         # exitAction.setShortcut('Ctrl+Q')
         # exitAction.setStatusTip('Exit application')
         # exitAction.triggered.connect(qApp.quit)
@@ -126,15 +134,17 @@ class LinoClient(QMainWindow):
     #     self.detail_form.show()
 
     def closeEvent(self, event):
-
         if True:
             event.accept()
             return
 
-        reply = QMessageBox.question(self, 'MessageBox',
+        reply = QMessageBox.question(
+            self,
+            "MessageBox",
             "This will close the window! Are you sure?",
-                                     QMessageBox.Yes |
-                                     QMessageBox.No, QMessageBox.Yes)
+            QMessageBox.Yes | QMessageBox.No,
+            QMessageBox.Yes,
+        )
 
         if reply == QMessageBox.Yes:
             event.accept()
@@ -142,7 +152,6 @@ class LinoClient(QMainWindow):
             event.ignore()
 
     def center(self):
-
         qr = self.frameGeometry()
         cp = QDesktopWidget().availableGeometry().center()
         qr.moveCenter(cp)
@@ -153,7 +162,6 @@ class Command(BaseCommand):
     help = """Run a Qt client for this site."""
 
     def handle(self, *args, **options):
-
         app = QApplication(sys.argv)
         self.ex = LinoClient()
         # sys.exit(app.exec_())
