@@ -8,10 +8,9 @@ from rstgen.confparser import ConfigParser
 
 
 class Command(BaseCommand):
-
     def handle(self, **options):
         cfgp = ConfigParser()
-        ini_file = settings.SITE.project_dir / 'lino.ini'
+        ini_file = settings.SITE.project_dir / "lino.ini"
         cfgp.read(ini_file)
 
         def set_conf(gens):
@@ -25,7 +24,8 @@ class Command(BaseCommand):
                         cfgp.stringify_set(*conf)
                 else:
                     set_conf(conf)
+
         set_conf(settings.SITE.get_plugin_configs())
 
-        with ini_file.open('w+') as f:
+        with ini_file.open("w+") as f:
             cfgp.update_file(f)
