@@ -1,16 +1,15 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2014-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Defines the :class:`Dupable` model mixin and related functionality
 to assist users in finding unwanted duplicate database records.
 
-Don't mix up this module with :mod:`lino.mixins.duplicable`.  Models
-are "duplicable" if users may *want* to duplicate some instance
+Don't mix up this module with :mod:`lino.mixins.`.  Models
+are "" if users may *want* to duplicate some instance
 thereof, while "dupable" implies that the duplicates are *unwanted*.
 To dupe *somebody* means "to make a dupe of; deceive; delude; trick."
-(`reference.com <http://dictionary.reference.com/browse/dupe>`_), and
+(`reference.com <https://dictionary.reference.com/browse/dupe>`_), and
 to dupe *something* means to duplicate it (eventually in order to
 cheat somebody e.g. by making a cheap copy of a valuable object).
 
@@ -32,23 +31,23 @@ in Python
 by Frank Hofmann (2018-02-12)
 """
 
-import six
-from builtins import map
-
 from django.conf import settings
 from django.db import models
 
 from lino.api import dd, _
 from lino.core.actions import SubmitInsert
 from lino.utils import join_elems
-from etgen.html import E
+from lino.utils.html import E, tostring, mark_safe
 from lino.core import constants
+from lino.modlib.checkdata.choicelists import Checker
+
 
 class CheckedSubmitInsert(SubmitInsert):
     """Like the standard :class:`lino.core.actions.SubmitInsert`, but adds
     a confirmation if there is a possible duplicate record.
 
     """
+
     def run_from_ui(self, ar, **kw):
         obj = ar.create_instance_from_request()
 
@@ -60,20 +59,24 @@ class CheckedSubmitInsert(SubmitInsert):
         qs = list(obj.find_similar_instances(4))
         if len(qs) > 0:
             msg = _("There are %d similar %s:") % (
-                len(qs), obj._meta.verbose_name_plural)
+                len(qs),
+                obj._meta.verbose_name_plural,
+            )
             for other in qs:
-                msg += '<br/>\n' + str(other)
+                msg += "<br/>\n" + str(other)
 
-            msg += '<br/>\n'
-            msg += _("Are you sure you want to create a new "
-                     "%(model)s named %(name)s?") % dict(
-                model=obj._meta.verbose_name,
-                name=obj.get_full_name())
+            msg += "<br/>\n"
+            msg += _(
+                "Are you sure you want to create a new " "%(model)s named %(name)s?"
+            ) % dict(model=obj._meta.verbose_name, name=obj.get_full_name())
 
             ar.confirm(ok, msg)
         else:
             ok(ar)
 
+# TODO: remove duplicate code. PhoneticWordBase.reduce_word() is exactly the
+# same code as lino.modlib.dupable.models.PhoneticWord.reduce_word(). One of
+# them is used in welfare, the other in avanti.
 
 
 class PhoneticWordBase(dd.Model):
@@ -86,10 +89,11 @@ class PhoneticWordBase(dd.Model):
     of `PhoneticWordBase`.
 
     """
-    class Meta(object):
+
+    class Meta:
         abstract = True
 
-    allow_cascaded_delete = ['owner']
+    allow_cascaded_delete = ["owner"]
 
     word = models.CharField(max_length=100)
 
@@ -99,24 +103,25 @@ class PhoneticWordBase(dd.Model):
     # @classmethod
     # def on_analyze(cls, site):
     #     pass
-        # import metaphone as fuzzy
-        # cls._fuzzy_DMetaphone = fuzzy.doublemetaphone()
-        # import fuzzy
-        # cls._fuzzy_DMetaphone = fuzzy.DMetaphone()
+    # import metaphone as fuzzy
+    # cls._fuzzy_DMetaphone = fuzzy.doublemetaphone()
+    # import fuzzy
+    # cls._fuzzy_DMetaphone = fuzzy.DMetaphone()
 
     @classmethod
     def reduce_word(cls, s):
         # from metaphone.word import Word
         import metaphone as fuzzy
+
         # fuzzy.DMetaphone does not work with unicode strings, see
         # https://bitbucket.org/yougov/fuzzy/issue/2/fuzzy-support-for-unicode-strings-with
         # dm = fuzzy.doublemetaphone(s.encode('utf8'))
         dm = fuzzy.doublemetaphone(s)
         dms = dm[0] or dm[1]
         if dms is None:
-            return ''
-        if isinstance(dms, six.binary_type):
-            dms = dms.decode('utf8')
+            return ""
+        if isinstance(dms, bytes):
+            dms = dms.decode("utf8")
         return dms
 
 
@@ -137,7 +142,8 @@ class Dupable(dd.Model):
     is done by plugins like :mod:`lino_xl.lib.dupable_partners` or
     :mod:`lino_welfare.modlib.dupable_clients`
     """
-    class Meta(object):
+
+    class Meta:
         abstract = True
 
     submit_insert = CheckedSubmitInsert()
@@ -149,7 +155,7 @@ class Dupable(dd.Model):
     confirmation when necessary.
     """
 
-    dupable_words_field = 'name'
+    dupable_words_field = "name"
     """The name of a CharField on this model which holds the full-text
     description that is being tested for duplicates."""
 
@@ -170,7 +176,7 @@ class Dupable(dd.Model):
         # if not site.is_installed(cls._meta.app_label):
         #     cls.dupable_word_model = None
         #     return
-        site.setup_model_spec(cls, 'dupable_word_model')
+        site.setup_model_spec(cls, "dupable_word_model")
 
     def dupable_matches_required(self):
         """Return the minimum number of words that must sound alike before
@@ -192,8 +198,7 @@ class Dupable(dd.Model):
             return
         qs = self.dupable_word_model.objects.filter(owner=self)
         existing = [o.word for o in qs]
-        wanted = self.get_dupable_words(
-            getattr(self, self.dupable_words_field))
+        wanted = self.get_dupable_words(getattr(self, self.dupable_words_field))
         if existing == wanted:
             return
         if really:
@@ -208,8 +213,8 @@ class Dupable(dd.Model):
             self.update_dupable_words()
 
     def get_dupable_words(self, s):
-        for c in '-,/&+':
-            s = s.replace(c, ' ')
+        for c in "-,/&+":
+            s = s.replace(c, " ")
         return list(map(self.dupable_word_model.reduce_word, s.split()))
 
     def find_similar_instances(self, limit=None, **kwargs):
@@ -227,56 +232,43 @@ class Dupable(dd.Model):
         qs = self.__class__.objects.filter(**kwargs)
         if self.pk is not None:
             qs = qs.exclude(pk=self.pk)
-        parts = self.get_dupable_words(
-            getattr(self, self.dupable_words_field))
+        parts = self.get_dupable_words(getattr(self, self.dupable_words_field))
         qs = qs.filter(dupable_words__word__in=parts).distinct()
-        qs = qs.annotate(num=models.Count('dupable_words__word'))
+        qs = qs.annotate(num=models.Count("dupable_words__word"))
         qs = qs.filter(num__gte=self.dupable_matches_required())
-        qs = qs.order_by('-num', 'pk')
+        qs = qs.order_by("-num", "pk")
         # print("20150306 find_similar_instances %s" % qs.query)
         if limit is None:
             return qs
         return qs[:limit]
 
 
-from lino.modlib.checkdata.choicelists import Checker
-
-
 class DupableChecker(Checker):
     """Checks for the following repairable problem:
 
     - :message:`Must update phonetic words.`
 
     """
+
     verbose_name = _("Check for missing phonetic words")
     model = Dupable
 
-    def get_checkdata_problems(self, obj, fix=False):
+    def get_checkdata_problems(self, ar, obj, fix=False):
         msg = obj.update_dupable_words(fix)
         if msg:
             yield (True, msg)
 
+
 DupableChecker.activate()
 
 
 class SimilarObjects(dd.VirtualTable):
-    """Shows the other objects which are similar to this one."""
-    display_mode = ((None, constants.DISPLAY_MODE_SUMMARY), )
+    """Shows the other objects that are similar to this one."""
+
+    default_display_modes = {None: constants.DISPLAY_MODE_SUMMARY}
     master = dd.Model
     abstract = True
 
-    # class Row:
-
-    #     def __init__(self, master, other):
-    #         self.master = master
-    #         self.other = other
-
-    #     def summary_row(self, ar):
-    #         yield ar.obj2html(self.other)
-
-    #     def __unicode__(self):
-    #         return unicode(self.other)
-
     @classmethod
     def get_data_rows(self, ar):
         mi = ar.master_instance
@@ -284,7 +276,6 @@ class SimilarObjects(dd.VirtualTable):
             return
 
         for o in mi.find_similar_instances(4):
-            # yield self.Row(mi, o)
             yield o
 
     @dd.displayfield(_("Similar record"))
@@ -293,13 +284,14 @@ class SimilarObjects(dd.VirtualTable):
         return ar.obj2html(obj)
 
     @classmethod
-    def get_table_summary(self, obj, ar):
+    def table_as_summary(self, ar):
         chunks = []
-        for other in ar.spawn(self, master_instance=obj):
+        for other in ar:
             chunks.append(ar.obj2html(other))
         if len(chunks):
+            obj = ar.master_instance
             s = getattr(obj, obj.dupable_words_field)
-            words = ', '.join(obj.get_dupable_words(s))
+            words = ", ".join(obj.get_dupable_words(s))
             chunks.append(_("Phonetic words: {0}").format(words))
-            return E.p(*join_elems(chunks))
-        return ''
+            return tostring(join_elems(chunks))
+        return mark_safe("")
