@@ -20,17 +20,31 @@ from lino.api.dd import Genders
 from lino.core import fields
 from lino.core import model
 
-name_prefixes1 = set(("HET", "'T", 'VAN', 'DER', 'TER', 'DEN',
-                      'VOM', 'VON', 'OF',
-                      "DE", "DU",
-                      "DA", "DI", "DO", "DOS",
-                      "EL", "AL"))
-name_prefixes2 = set(("VAN DEN", "VAN DER", "VAN DE",
-                      "IN HET", "VON DER", "DE LA"))
-
-NAME_PREFIXES = set([p + ' ' for p in name_prefixes1])
-NAME_PREFIXES |= set([p + ' ' for p in name_prefixes2])
+name_prefixes1 = set(
+    (
+        "HET",
+        "'T",
+        "VAN",
+        "DER",
+        "TER",
+        "DEN",
+        "VOM",
+        "VON",
+        "OF",
+        "DE",
+        "DU",
+        "DA",
+        "DI",
+        "DO",
+        "DOS",
+        "EL",
+        "AL",
+    )
+)
+name_prefixes2 = set(("VAN DEN", "VAN DER", "VAN DE", "IN HET", "VON DER", "DE LA"))
 
+NAME_PREFIXES = set([p + " " for p in name_prefixes1])
+NAME_PREFIXES |= set([p + " " for p in name_prefixes2])
 
 
 def strip_name_prefix(s):
@@ -39,7 +53,7 @@ def strip_name_prefix(s):
 
     def strip_from(s, lst):
         for p in lst:
-            p = p + ' '
+            p = p + " "
             if s.startswith(p):
                 s = s[len(p):]
         return s
@@ -57,7 +71,7 @@ def name2kw(s, last_name_first=True):
     first_name first (e.g. Luc Saffre).
     """
     kw = {}
-    a = s.split(',')
+    a = s.split(",")
     if len(a) == 2:
         if last_name_first:
             return dict(last_name=a[0].strip(), first_name=a[1].strip())
@@ -74,39 +88,28 @@ def name2kw(s, last_name_first=True):
     else:
         # string of more than 3 words
         if last_name_first:
-            a01 = a[0] + ' ' + a[1]
+            a01 = a[0] + " " + a[1]
             if a01.upper() in name_prefixes2:
-                return dict(
-                    last_name=a01 + ' ' + a[2],
-                    first_name=' '.join(a[3:]))
+                return dict(last_name=a01 + " " + a[2], first_name=" ".join(a[3:]))
             elif a[0].upper() in name_prefixes1:
-                return dict(
-                    last_name=a[0] + ' ' + a[1],
-                    first_name=' '.join(a[2:]))
+                return dict(last_name=a[0] + " " + a[1], first_name=" ".join(a[2:]))
             else:
-                return dict(last_name=a[0],
-                            first_name=' '.join(a[1:]))
+                return dict(last_name=a[0], first_name=" ".join(a[1:]))
         else:
             if len(a) >= 4:
-                pc = a[-3] + ' ' + a[-2]  # prefix2 candidate
+                pc = a[-3] + " " + a[-2]  # prefix2 candidate
                 if pc.upper() in name_prefixes2:
-                    return dict(
-                        last_name=pc + ' ' + a[-1],
-                        first_name=' '.join(a[:-3]))
+                    return dict(last_name=pc + " " + a[-1], first_name=" ".join(a[:-3]))
             pc = a[-2]  # prefix candidate
             if pc.upper() in name_prefixes1:
-                return dict(
-                    last_name=pc + ' ' + a[-1],
-                    first_name=' '.join(a[:-2]))
-        return dict(
-            last_name=a[-1],
-            first_name=' '.join(a[:-1]))
+                return dict(last_name=pc + " " + a[-1], first_name=" ".join(a[:-2]))
+        return dict(last_name=a[-1], first_name=" ".join(a[:-1]))
 
     return kw
 
 
 def upper1(s):
-    if ' ' in s:
+    if " " in s:
         return s  # don't change
     return s[0].upper() + s[1:]
 
@@ -122,8 +125,9 @@ def parse_name(text):
     kw = name2kw(text, last_name_first=False)
     if len(kw) != 2:
         raise ValidationError(
-            _("Cannot find first and last name in \"{0}\"").format(text))
-    for k in ('last_name', 'first_name'):
+            _('Cannot find first and last name in "{0}"').format(text)
+        )
+    for k in ("last_name", "first_name"):
         if kw[k] and not kw[k].isupper():
             kw[k] = upper1(kw[k])
     return kw
@@ -145,7 +149,7 @@ def get_salutation(gender, nominative=False):
     the default (accusative or indirect form) "Herrn".
     """
     if not gender:
-        return ''
+        return ""
     if gender == Genders.female:
         return _("Mrs")
     if nominative:
@@ -153,7 +157,6 @@ def get_salutation(gender, nominative=False):
     return pgettext("indirect salutation", "Mr")
 
 
-
 class Human(model.Model):
     """
     Base class for models that represent a human.
@@ -191,17 +194,14 @@ class Human(model.Model):
         abstract = True
 
     title = models.CharField(
-        pgettext("(of a human)", "Title"),
-        max_length=200, blank=True)
+        pgettext("(of a human)", "Title"), max_length=200, blank=True
+    )
 
-    first_name = models.CharField(
-        _('First name'), max_length=200, blank=True)
+    first_name = models.CharField(_("First name"), max_length=200, blank=True)
 
-    middle_name = models.CharField(
-        _("Middle name"), max_length=200, blank=True)
+    middle_name = models.CharField(_("Middle name"), max_length=200, blank=True)
 
-    last_name = models.CharField(
-        _('Last name'), max_length=200, blank=True)
+    last_name = models.CharField(_("Last name"), max_length=200, blank=True)
 
     gender = Genders.field(blank=True)
 
@@ -226,8 +226,10 @@ class Human(model.Model):
 
     def get_salutation(self, **salutation_options):
         return get_salutation(
-            #~ translation.get_language(),
-            self.gender, **salutation_options)
+            # ~ translation.get_language(),
+            self.gender,
+            **salutation_options,
+        )
 
     def get_last_name_prefix(self):
         """
@@ -243,10 +245,9 @@ class Human(model.Model):
         <http://www.linguee.de/englisch-deutsch/uebersetzung/mr.+dr..html>`__
         and `wikipedia.org <https://en.wikipedia.org/wiki/Title>`__
         """
-        return ''
+        return ""
 
-    def get_full_name(
-            self, salutation=True, upper=None, **salutation_options):
+    def get_full_name(self, salutation=True, upper=None, last_name_first=None, **salutation_options):
         """
         Returns a one-line string composed of salutation,
         :attr:`first_name` and :attr:`last_name`.
@@ -270,12 +271,14 @@ class Human(model.Model):
         See :ref:`lino.tutorial.human` for some examples.
         """
         words = []
-
+        if last_name_first is None:
+            last_name_first = settings.SITE.last_name_first
         if salutation:
             words.append(self.get_salutation(**salutation_options))
         if self.title:
             words.append(self.title)
-        words.append(self.first_name)
+        if not last_name_first:
+            words.append(self.first_name)
         prefix = self.get_last_name_prefix()
         if prefix:
             words.append(prefix)
@@ -285,7 +288,11 @@ class Human(model.Model):
             words.append(self.last_name.upper())
         else:
             words.append(self.last_name)
-        return join_words(*words)
+        rv = join_words(*words)
+        if last_name_first and self.first_name:
+            rv += ", " + self.first_name
+        return rv
+
     full_name = property(get_full_name)
 
     def format_family_member(self, ar, other):
@@ -296,14 +303,22 @@ class Human(model.Model):
         if other.last_name == self.last_name:
             text = other.first_name
         else:
-            text = other.first_name + ' ' + other.last_name.upper()
+            text = other.first_name + " " + other.last_name.upper()
         return ar.obj2html(other, text)
 
     @classmethod
     def get_simple_parameters(cls):
         for p in super().get_simple_parameters():
             yield p
-        yield 'gender'
+        yield "gender"
+
+
+Human.set_widget_options("first_name", detail_pointer=True)
+Human.set_widget_options("last_name", detail_pointer=True)
+
+# TODO: the following two calls should generate an error message
+Human.set_widget_options("last_name", invalid_option=True)
+Human.set_widget_options("invalid_field_name", detail_pointer=True)
 
 
 class Born(model.Model):
@@ -324,8 +339,7 @@ class Born(model.Model):
         # app_label = 'cal'
         abstract = True
 
-    birth_date = fields.IncompleteDateField(
-        blank=True, verbose_name=_("Birth date"))
+    birth_date = fields.IncompleteDateField(blank=True, verbose_name=_("Birth date"))
 
     def get_age(self, today=None):
         """
@@ -350,15 +364,16 @@ class Born(model.Model):
             if today is None:
                 today = settings.SITE.today()
             try:
-                return relativedelta(today , self.birth_date.as_date())
+                return relativedelta(today, self.birth_date.as_date())
             except ValueError:
                 pass
 
-    @fields.displayfield(_("Age"), wildcard_data_elem=True)
+    @fields.displayfield(
+        _("Age"), wildcard_data_elem=True, sortable_by=['birth_date'])
     def age(self, ar, today=None):
         a = self.get_exact_age(today)
         if a is None:
-            return str(_('unknown'))
+            return str(_("unknown"))
         years = a.years
         if years == 1:
             s = _("{} year").format(years)
@@ -378,30 +393,38 @@ class Born(model.Model):
     def setup_parameters(cls, fields):
         fields.update(
             aged_from=models.IntegerField(
-                _("Aged from"), blank=True, null=True,
-                help_text=_("Select only persons aged at least "
-                            "the given number of years.")),
+                _("Aged from"),
+                blank=True,
+                null=True,
+                help_text=_(
+                    "Select only persons aged at least " "the given number of years."
+                ),
+            ),
             aged_to=models.IntegerField(
-                _("Aged to"), blank=True, null=True,
-                help_text=_("Select only persons aged at most "
-                            "the given number of years.")))
+                _("Aged to"),
+                blank=True,
+                null=True,
+                help_text=_(
+                    "Select only persons aged at most " "the given number of years."
+                ),
+            ),
+        )
         super().setup_parameters(fields)
 
     @classmethod
     def get_request_queryset(cls, ar, **filter):
         qs = super().get_request_queryset(ar, **filter)
-        pv = ar.param_values
+        if (pv := ar.param_values) is None:
+            return qs
 
         today = settings.SITE.today()
 
         if pv.aged_from:
-            min_date = today - \
-                datetime.timedelta(days=pv.aged_from * 365)
+            min_date = today - datetime.timedelta(days=pv.aged_from * 365)
             qs = qs.filter(birth_date__lte=min_date.strftime("%Y-%m-%d"))
 
         if pv.aged_to:
-            max_date = today - \
-                datetime.timedelta(days=pv.aged_to * 365)
+            max_date = today - datetime.timedelta(days=pv.aged_to * 365)
             qs = qs.filter(birth_date__gte=max_date.strftime("%Y-%m-%d"))
 
         return qs
@@ -412,6 +435,7 @@ class Born(model.Model):
             yield t
         pv = ar.param_values
         if pv.aged_from or pv.aged_to:
-            yield str(_("Aged %(min)s to %(max)s") % dict(
-                min=pv.aged_from or '...',
-                max=pv.aged_to or '...'))
+            yield str(
+                _("Aged %(min)s to %(max)s")
+                % dict(min=pv.aged_from or "...", max=pv.aged_to or "...")
+            )
