@@ -1,16 +1,16 @@
-# Copyright 2014-2021 Rumma & Ko Ltd
+# Copyright 2014-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Defines classes related to date ranges.
 
 """
 
 import datetime
-try:
-    import pytz
-except ImportError:
-    pytz = None
+
+# try:
+#     import pytz
+# except ImportError:
+#     pytz = None
 
 from django.db import models
 from django.conf import settings
@@ -18,6 +18,7 @@ from django.utils.translation import gettext_lazy as _
 from django.utils.translation import pgettext_lazy as pgettext
 from django.core.exceptions import ValidationError
 from django.utils.timezone import is_aware
+from asgiref.sync import sync_to_async
 
 from lino.utils import last_day_of_month
 from lino.api import dd
@@ -29,14 +30,51 @@ from lino.utils.quantities import Duration
 
 
 def rangefmt(r):
-    return fds(r[0]) + '...' + fds(r[1])
+    return fds(r[0]) + "..." + fds(r[1])
+
+
+def is_future(self):
+    return self.start_date and self.start_date > settings.SITE.today()
+
+
+def rangetext(self, fmt=fdl, empty_text=""):
+    """
+    Return a textual representation of the given date range.
+
+    The date range is to be specified as an object that has two attributes
+    `start_date` and `end_date`.
+
+    See tested examples in :ref:`welfare.specs.aids`.
+    Also used by :mod:`lino_xl.lib.vat`.
+    """
+    if self.start_date and self.end_date:
+        if self.start_date == self.end_date:
+            # s = tostring(E.b(fdl(self.start_date)))
+            s = fmt(self.start_date)
+            return pgettext("date", "on %s") % s
+        else:
+            kw = dict()
+            kw.update(a=fmt(self.start_date))
+            kw.update(b=fmt(self.end_date))
+            return pgettext("date range", "between %(a)s and %(b)s") % kw
+    elif self.start_date:
+        s = fmt(self.start_date)
+        if is_future(self):
+            return pgettext("future date range", "from %s") % s
+        else:
+            return pgettext("date range", "from %s") % s
+    elif self.end_date:
+        s = fmt(self.end_date)
+        return pgettext("date range", "until %s") % s
+    return empty_text
 
 
 class CombinedDateTime(dd.Model):
     """
-    Mixin for models which have at least one couple of date and time
-    fields which form a kind of editable timestamp field.
+    Mixin for models having at least one couple of date and time fields that
+    form a kind of editable timestamp field.
     """
+
     class Meta:
         abstract = True
 
@@ -65,11 +103,11 @@ class CombinedDateTime(dd.Model):
             # value = tz.localize(value)
             value = value.astimezone(self.get_time_zone().tzinfo)
 
-        setattr(self, name + '_date', value.date())
+        setattr(self, name + "_date", value.date())
         t = value.time()
         if not t:
             t = None
-        setattr(self, name + '_time', t)
+        setattr(self, name + "_time", t)
 
     def get_datetime(self, name, altname=None):
         """
@@ -83,12 +121,12 @@ class CombinedDateTime(dd.Model):
         In that case you should ask ``get_datetime("end", "start")``.
 
         """
-        d = getattr(self, name + '_date')
-        t = getattr(self, name + '_time')
+        d = getattr(self, name + "_date")
+        t = getattr(self, name + "_time")
         if not d and altname is not None:
-            d = getattr(self, altname + '_date')
+            d = getattr(self, altname + "_date")
             if not t and altname is not None:
-                t = getattr(self, altname + '_time')
+                t = getattr(self, altname + "_time")
         if not d:
             return None
         if t:
@@ -111,16 +149,12 @@ class Started(CombinedDateTime):
     .. attribute:: start_time
 
     """
+
     class Meta:
         abstract = True
 
-    start_date = models.DateField(
-        blank=True, null=True,
-        verbose_name=_("Start date"))  # iCal:DTSTART
-    start_time = dd.TimeField(
-        blank=True, null=True,
-        verbose_name=_("Start time"))  # iCal:DTSTART
-    #~ start = dd.FieldSet(_("Start"),'start_date start_time')
+    start_date = models.DateField(_("Start date"), blank=True, null=True)
+    start_time = dd.TimeField(_("Start time"), blank=True, null=True)
 
     def save(self, *args, **kw):
         """
@@ -128,7 +162,8 @@ class Started(CombinedDateTime):
         """
         if not self.start_date:
             self.start_date = settings.SITE.today()
-        super(Started, self).save(*args, **kw)
+        super().save(*args, **kw)
+
 
 class Ended(CombinedDateTime):
     """
@@ -138,15 +173,12 @@ class Ended(CombinedDateTime):
     .. attribute:: end_date
     .. attribute:: end_time
     """
+
     class Meta:
         abstract = True
 
-    end_date = models.DateField(
-        blank=True, null=True,
-        verbose_name=_("End Date"))
-    end_time = dd.TimeField(
-        blank=True, null=True,
-        verbose_name=_("End Time"))
+    end_date = models.DateField(blank=True, null=True, verbose_name=_("End Date"))
+    end_time = dd.TimeField(blank=True, null=True, verbose_name=_("End Time"))
 
     def get_duration(self):
         """Return the duration in hours."""
@@ -173,6 +205,7 @@ class Ended(CombinedDateTime):
 
 
 class DateRangeObservable(Model):
+
     class Meta(object):
         abstract = True
 
@@ -183,22 +216,30 @@ class DateRangeObservable(Model):
     def setup_parameters(cls, fields):
         fields.update(
             start_date=models.DateField(
-                _("Period from"), blank=True, null=True,
+                _("Date range from"),
+                blank=True,
+                null=True,
                 default=cls.get_default_start_date,
-                help_text=_("Start date of observed period")))
+                help_text=_("Start date of observed date range."),
+            )
+        )
         fields.update(
             end_date=models.DateField(
                 _("until"),
-                blank=True, null=True,
+                blank=True,
+                null=True,
                 default=cls.get_default_end_date,
-                help_text=_("End date of observed period")))
+                help_text=_("End date of observed date range."),
+            )
+        )
         super().setup_parameters(fields)
 
 
 class DateRange(DateRangeObservable):
     """
-    Mixin for models that represent a period defined by a start date and an end
-    date.
+
+    A model mixin that adds two fields `start_date` and `end_date`.
+    :class:`DateRangeObservable`
 
     Designed for usage with
     :class:`lino.modlib.system.PeriodEvents`.
@@ -221,42 +262,31 @@ class DateRange(DateRangeObservable):
         return (self.start_date, self.end_date)
 
     def is_past(self):
-        return (self.end_date and self.end_date <= dd.today())
+        return self.end_date and self.end_date <= dd.today()
 
     def is_future(self):
-        return (self.start_date and self.start_date > settings.SITE.today())
+        return is_future(self)
 
     def get_period_text(self):
-        if self.start_date and self.end_date:
-            if self.start_date == self.end_date:
-                # s = tostring(E.b(fdl(self.start_date)))
-                s = fdl(self.start_date)
-                return pgettext("date", "on %s") % s
-            else:
-                kw = dict()
-                kw.update(a=fdl(self.start_date))
-                kw.update(b=fdl(self.end_date))
-                return pgettext("date range", "between %(a)s and %(b)s") % kw
-        elif self.start_date:
-            s = fdl(self.start_date)
-            if self.is_future():
-                return pgettext("future date range", "from %s") % s
-            else:
-                return pgettext("date range", "from %s") % s
-        elif self.end_date:
-            s = fdl(self.end_date)
-            return pgettext("date range", "until %s") % s
-        return self.empty_period_text
+        return rangetext(self, fdl, self.empty_period_text)
+
+    def covers_date(self, d):
+        if self.start_date and self.start_date > d:
+            return False
+        if self.end_date and self.end_date < d:
+            return False
+        return True
 
 
-DateRange.set_widget_options('start_date', width=10)
-DateRange.set_widget_options('end_date', width=10)
+DateRange.set_widget_options("start_date", width=10)
+DateRange.set_widget_options("end_date", width=10)
+
 
 class ObservedDateRange(ParameterPanel):
     """:class:`lino.core.param_panel.ParameterPanel` with two fields
-    `start_date` and `end_date` which default to empty.
+    `start_date` and `end_date`.
 
-    Note that you must define yourself a get_request_queryset method in order to
+    You must define yourself a :meth:`get_request_queryset` method in order to
     actually use these two parameter fields.
 
     """
@@ -264,19 +294,24 @@ class ObservedDateRange(ParameterPanel):
     get_default_start_date = None
     get_default_end_date = None
 
-    def __init__(self,
-                 verbose_name_start=_("Date from"),
-                 verbose_name_end=_("until"), **kwargs):
+    def __init__(
+        self, verbose_name_start=_("Date from"), verbose_name_end=_("until"), **kwargs
+    ):
         kwargs.update(
             start_date=models.DateField(
-                verbose_name_start, blank=True, null=True,
+                verbose_name_start,
+                blank=True,
+                null=True,
                 default=self.get_default_start_date,
-                help_text=_("Start of observed date range")),
+                help_text=_("Start of observed date range"),
+            ),
             end_date=models.DateField(
                 verbose_name_end,
-                blank=True, null=True,
+                blank=True,
+                null=True,
                 default=self.get_default_end_date,
-                help_text=_("End of observed date range")),
+                help_text=_("End of observed date range"),
+            ),
         )
         super().__init__(**kwargs)
 
@@ -293,7 +328,6 @@ class ObservedDateRange(ParameterPanel):
 
 
 class Yearly(ObservedDateRange):
-
     """An :class:`ObservedDateRange` for which `start_date` defaults to Jan
     1st and `end_date` to Dec 31 of the current year.
 
@@ -311,10 +345,7 @@ class Yearly(ObservedDateRange):
 
 
 class Monthly(ObservedDateRange):
-
-    """An :class:`ObservedDateRange` which defaults to the current month.
-
-    """
+    """An :class:`ObservedDateRange` which defaults to the current month."""
 
     def get_default_start_date(self):
         return dd.today().replace(day=1)
@@ -324,10 +355,7 @@ class Monthly(ObservedDateRange):
 
 
 class Weekly(ObservedDateRange):
-
-    """An :class:`ObservedDateRange` which defaults to the current week.
-
-    """
+    """An :class:`ObservedDateRange` which defaults to the current week."""
 
     def get_default_start_date(self):
         d = dd.today()
@@ -335,7 +363,7 @@ class Weekly(ObservedDateRange):
 
     def get_default_end_date(self):
         d = dd.today()
-        return d + datetime.timedelta(days=6-d.weekday())
+        return d + datetime.timedelta(days=6 - d.weekday())
 
 
 class Today(ParameterPanel):
@@ -343,11 +371,15 @@ class Today(ParameterPanel):
     with a field `today` that defaults to today.
 
     """
+
     def __init__(self, verbose_name=_("Situation on"), **kw):
         kw.update(
             today=models.DateField(
-                verbose_name, blank=True, null=True,
+                verbose_name,
+                blank=True,
+                null=True,
                 default=dd.today,
-                help_text=_("Date of observation")),
+                help_text=_("Date of observation"),
+            ),
         )
-        super(Today, self).__init__(**kw)
+        super().__init__(**kw)
