@@ -1,19 +1,20 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2010-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
-
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 Defines the :class:`Referrable` model mixin.
 """
 
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.db.models.functions import Length
 
-from etgen.html import E
+from lino.utils.html import E
+from lino.utils import nextref
 from lino.core import model
 from lino.core.fields import displayfield
 
+
 class Referrable(model.Model):
     """
     Mixin for things that have a unique reference, i.e. an identifying
@@ -27,6 +28,7 @@ class Referrable(model.Model):
 
         The reference. This must be either empty or unique.
     """
+
     class Meta(object):
         abstract = True
 
@@ -40,13 +42,13 @@ class Referrable(model.Model):
     """
 
     ref = models.CharField(
-        _("Reference"), max_length=200,
-        blank=True, null=True, unique=True)
+        _("Reference"), max_length=200, blank=True, null=True, unique=True
+    )
 
     @classmethod
     def on_analyze(cls, site):
-        cls.set_widget_options('ref', width=cls.ref_max_length)
-        super(Referrable, cls).on_analyze(site)
+        cls.set_widget_options("ref", width=cls.ref_max_length)
+        super().on_analyze(site)
 
     def on_duplicate(self, ar, master):
         """
@@ -54,24 +56,48 @@ class Referrable(model.Model):
         change the :attr:`ref` in order to avoid an IntegrityError.
         """
         if self.ref:
-            self.ref += ' (DUP)'
-        super(Referrable, self).on_duplicate(ar, master)
+            self.ref += " (DUP)"
+        super().on_duplicate(ar, master)
+
+    def __str__(self):
+        return self.ref or super().__str__()
+
+    def get_next_row(self):
+        """Return the next database row, or None if this is the last one.
+        """
+        ref = nextref(self.ref)
+        if ref is None:
+            return None
+        return self.__class__.get_by_ref(ref, None)
+
+    @staticmethod
+    def ref_prefix(obj, ar=None):
+        return obj.as_ref_prefix(ar)
 
     @classmethod
-    def get_by_ref(cls, ref, default=models.NOT_PROVIDED):
+    def get_by_ref(cls, ref, default=models.NOT_PROVIDED, **more):
         """
         Return the object identified by the given reference.
         """
         try:
-            return cls.objects.get(ref=ref)
+            return cls.objects.get(ref=ref, **more)
         except cls.DoesNotExist:
             if default is models.NOT_PROVIDED:
                 raise cls.DoesNotExist(
-                    "No %s with reference %r" % (str(cls._meta.verbose_name), ref))
+                    "There is no {} with reference {!r}".format(
+                        cls._meta.verbose_name, ref))
             return default
 
+        # try:
+        #     return cls.objects.get(ref=ref, **more)
+        # except cls.DoesNotExist:
+        #     if default is models.NOT_PROVIDED:
+        #         raise cls.DoesNotExist(
+        #             "No %s with reference %r" % (str(cls._meta.verbose_name), ref))
+        #     return default
+
     @classmethod
-    def quick_search_filter(cls, search_text, prefix=''):
+    def quick_search_filter(cls, search_text, prefix=""):
         """Overrides the default behaviour defined in
         :meth:`lino.core.model.Model.quick_search_filter`. For
         Referrable objects, when quick-searching for a text containing
@@ -79,11 +105,10 @@ class Referrable(model.Model):
         the primary key.
 
         """
-        #if search_text.isdigit():
-        if search_text.startswith('*'):
-            return models.Q(**{prefix+'ref__icontains': search_text[1:]})
-        return super(Referrable, cls).quick_search_filter(search_text, prefix)
-
+        # if search_text.isdigit():
+        if search_text.startswith("*"):
+            return models.Q(**{prefix + "ref__icontains": search_text[1:]})
+        return super().quick_search_filter(search_text, prefix)
 
 
 class StructuredReferrable(Referrable):
@@ -126,6 +151,7 @@ class StructuredReferrable(Referrable):
 
 
     """
+
     class Meta:
         abstract = True
 
@@ -138,15 +164,15 @@ class StructuredReferrable(Referrable):
 
     @classmethod
     def get_usable_items(cls):
-        return cls.objects.annotate(
-            ref_len=Length('ref')).filter(
-                ref_len=cls.ref_max_length)
+        return cls.objects.annotate(ref_len=Length("ref")).filter(
+            ref_len=cls.ref_max_length
+        )
 
     @classmethod
     def get_heading_objects(cls):
-        return cls.objects.annotate(
-            ref_len=Length('ref')).exclude(
-                ref_len=cls.ref_max_length)
+        return cls.objects.annotate(ref_len=Length("ref")).exclude(
+            ref_len=cls.ref_max_length
+        )
 
     def is_heading(self):
         if self.ref is None:
@@ -159,7 +185,7 @@ class StructuredReferrable(Referrable):
             s = self.get_designation()
         else:
             s = self.ref
-            s = u' ' * (len(s)-1) + s
+            s = " " * (len(s) - 1) + s
             s += " " + self.get_designation()
         if self.is_heading():
             s = E.b(s)
