@@ -8,10 +8,9 @@ import datetime
 # from django.contrib.humanize.templatetags.humanize import naturaltime
 from django.utils.translation import gettext_lazy as _
 from django.utils.translation import gettext
-from django.utils.html import mark_safe
+from django.utils.html import mark_safe, format_html
 from django.conf import settings
 
-
 from lino.utils.report import EmptyTable
 from lino.core.utils import get_models
 from lino.core import constants
@@ -19,7 +18,7 @@ from lino.core import constants
 # from lino.utils.jsgen import js_code
 from lino.utils.code import codetime
 from lino.utils.diag import analyzer
-from etgen.html import E, join_elems, forcetext, tostring
+from lino.utils.html import E, join_elems, forcetext, tostring
 
 from lino.api import rt, dd
 
@@ -32,8 +31,8 @@ def dtfmt(dt):
         dt = datetime.datetime.fromtimestamp(dt)
         # raise ValueError("Expected float, go %r" % dt)
     return gettext("%(date)s at %(time)s") % dict(
-        date=dd.fds(dt.date()),
-        time=settings.SITE.strftime(dt.time()))
+        date=dd.fds(dt.date()), time=settings.SITE.strftime(dt.time())
+    )
 
 
 class About(EmptyTable):
@@ -42,32 +41,40 @@ class About(EmptyTable):
     This window shows information about this :term:`Lino site`.
 
     """
+
     label = _("About")
     help_text = _("Show information about this site.")
     required_roles = set()
     hide_top_toolbar = True
-    detail_layout = dd.DetailLayout("""
+    detail_layout = dd.DetailLayout(
+        """
     about_html
     # server_status
-    """, window_size=(60, 20))
+    """,
+        window_size=(60, 20),
+    )
 
     # @dd.constant()
     # def about_html(cls):
     @dd.htmlbox()
     def about_html(cls, obj, ar=None):
-
         site = settings.SITE
         body = ""
 
         body += "".join([tostring(e) for e in site.welcome_html()])
 
         for p in site.sorted_plugins:
-            for i in p.get_site_info():
+            for i in p.get_site_info(ar):
                 body += i
 
         if site.languages:
-            body += tostring(E.p(str(_("Languages")) + ": " + ', '.join([
-                lng.django_code for lng in site.languages])))
+            body += tostring(
+                E.p(
+                    str(_("Languages"))
+                    + ": "
+                    + ", ".join([lng.django_code for lng in site.languages])
+                )
+            )
 
         # print "20121112 startup_time", site.startup_time.date()
         # showing startup time here makes no sense as this is a constant text
@@ -79,7 +86,8 @@ class About(EmptyTable):
             body += tostring(E.p(gettext(_("This is a Lino demo site."))))
         if site.the_demo_date:
             s = _("We are running with simulated date set to {0}.").format(
-                dd.fdf(site.the_demo_date))
+                dd.fdf(site.the_demo_date)
+            )
             body += tostring(E.p(s))
 
         # features = []
@@ -88,7 +96,7 @@ class About(EmptyTable):
         #         features.append(k)
         # body += tostring(E.p("{} : {}".format(
         #     _("Enabled features:"), ", ".join(features))))
-        
+
         # style = "border: 1px solid black; border-radius: 2px; padding: 5px;"
         #
         # f_table_head = []
@@ -108,13 +116,14 @@ class About(EmptyTable):
         body += tostring(E.p(str(_("Source timestamps:"))))
         items = []
         times = []
-        packages = set(['django'])
+        packages = set(["django"])
 
-        items.append(E.li(gettext("Server timestamp"), ' : ',
-            E.b(dtfmt(site.kernel.code_mtime))))
+        items.append(
+            E.li(gettext("Server timestamp"), " : ",
+                 E.b(dtfmt(site.kernel.lino_version))))
 
         for p in site.installed_plugins:
-            packages.add(p.app_name.split('.')[0])
+            packages.add(p.app_name.split(".")[0])
         for src in packages:
             label = src
             value = codetime(src)
@@ -123,12 +132,18 @@ class About(EmptyTable):
 
         times.sort(key=lambda x: x[1])
         for label, value in times:
-            items.append(E.li(str(label), ' : ', E.b(dtfmt(value))))
+            items.append(E.li(str(label), " : ", E.b(dtfmt(value))))
         body += tostring(E.ul(*items))
-        body += tostring(E.p("{} : {}".format(
-            _("Complexity factors"),
-            ', '.join(analyzer.get_complexity_factors(dd.today())))))
-        body = "<div>{}</div>".format(body)
+        body += tostring(
+            E.p(
+                "{} : {}".format(
+                    _("Complexity factors"),
+                    ", ".join(analyzer.get_complexity_factors(dd.today())),
+                )
+            )
+        )
+        body = mark_safe(body)
+        body = format_html("<div>{}</div>", body)
         # return js_code(rt.html_text(body))
         # return rt.html_text(body)
         # return mark_safe(body)
