@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2011-2014 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """The :xfile:`models.py` module for the
 :mod:`lino.modlib.awesomeuploader` app.
 
@@ -15,16 +14,14 @@ from lino.api import dd, rt
 
 
 class UploaderAction(dd.Action):
-
     extjs_main_panel = "Lino.AwesomeUploader()"
     opens_a_window = True
-    action_name = 'uploader'
-    default_format = 'html'
+    action_name = "uploader"
+    default_format = "html"
     # icon_name = 'calendar'
 
 
 class UploaderPanel(dd.Frame):
-
     help_text = _("Open an window for multiple file uploads.")
     label = _("Uploader")
 
@@ -33,5 +30,4 @@ class UploaderPanel(dd.Frame):
         return UploaderAction()
 
 
-
 # __all__ = ['UploaderPanel']
