@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2017 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """The models module for this plugin.
 
 """
@@ -11,6 +10,7 @@
 from django.conf import settings
 
 from datetime import datetime, timedelta
+
 # from django.db import models
 from django.contrib.humanize.templatetags.humanize import naturaltime
 
@@ -18,27 +18,27 @@ from lino.api import dd, _
 
 from lino.core.auth import get_auth_middleware
 
+
 def format_timestamp(dt):
     if dt is None:
-        return ''
-    return u"{} {} ({})".format(
+        return ""
+    return "{} {} ({})".format(
         dt.strftime(settings.SITE.date_format_strftime),
         dt.strftime(settings.SITE.time_format_strftime),
-        naturaltime(dt))
+        naturaltime(dt),
+    )
 
 
 class Sessions(dd.Table):
-    """
+    """ """
 
-    """
     label = _("Connections")
-    model = 'sessions.Session'
-    
+    model = "sessions.Session"
+
     required_roles = dd.login_required(dd.SiteAdmin)
     column_names = "ip_address:12 login_failures:5 blacklisted_since:12 username:20 last_request:30 last_login:30"
     # window_size = (90, 12)
 
-
     @dd.displayfield(_("IP address"))
     def ip_address(self, obj, ar):
         d = obj.get_decoded()
@@ -65,6 +65,3 @@ class Sessions(dd.Table):
     @dd.displayfield(_("Last login"))
     def last_login(self, obj, ar):
         return format_timestamp(None)
-
-    
-            
