@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2015 Rumma & Ko Ltd
+# Copyright 2015-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """Runs the :manage:`checkdata` management command with `--fix`
 option.
@@ -7,8 +7,10 @@ option.
 """
 
 from django.core.management import call_command
+from lino.api import dd
 
 
 def objects():
-    call_command('checkdata', fix=True)
+    if not dd.is_installed("linod"):
+        call_command("checkdata", fix=True)
     return []
