@@ -1,6 +1,5 @@
 # Copyright 2012-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
-
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """Defines a Loader which is automatically installed to
 `TEMPLATE_LOADERS` by :mod:`lino.core.site`
 
@@ -15,7 +14,6 @@ from lino.core import requests
 
 
 class DjangoJinjaTemplate(object):
-
     """
     used e.g. to render :srcref:`/lino/lino/config/500.html`
     """
@@ -25,27 +23,25 @@ class DjangoJinjaTemplate(object):
 
     def render(self, context):
         # flatten the Django Context into a single dictionary.
-        #~ logger.info("20130118 %s",context)
+        # ~ logger.info("20130118 %s",context)
         context_dict = {}
         for d in context.dicts:
             context_dict.update(d)
         # extend_context(context_dict)
-        ar = requests.BaseRequest(
-            renderer=settings.SITE.plugins.jinja.renderer)
-            # renderer=settings.SITE.kernel.default_renderer)
+        ar = requests.BaseRequest(renderer=settings.SITE.plugins.jinja.renderer)
+        # renderer=settings.SITE.kernel.default_renderer)
         context_dict = ar.get_printable_context(**context_dict)
-        context_dict.setdefault('request', None)
-        #context_dict.setdefault('ar', ar)
-        #~ logger.info("20130118 %s",context_dict.keys())
+        context_dict.setdefault("request", None)
+        # context_dict.setdefault('ar', ar)
+        # ~ logger.info("20130118 %s",context_dict.keys())
         return self.jt.render(context_dict)
 
 
 class Loader(BaseLoader):
-
     is_usable = True
 
     def load_template(self, template_name, template_dirs=None):
-        #~ source, origin = self.load_template_source(template_name, template_dirs)
+        # ~ source, origin = self.load_template_source(template_name, template_dirs)
         env = settings.SITE.plugins.jinja.renderer.jinja_env
 
         try:
