@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2020 Rumma & Ko Ltd
+# Copyright 2011-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 Adds all languages of the world to your database.
@@ -22,16 +22,13 @@ import codecs
 
 from django.conf import settings
 
-# from lino.utils import dblogger as logger
-
 from lino.api import dd
 from lino.modlib.languages.models import Language
 
-
 """
 """
 
-german = u'''
+german = """
 Abchasisch abk
 Aceh-Sprache ace
 Acholi-Sprache ach
@@ -526,8 +523,7 @@ Zenaga zen
 Zhuang zha
 Zulu-Sprache zul
 Zuñi-Sprache zun
-'''
-
+"""
 
 # copied from http://fr.wikipedia.org/wiki/Liste_des_codes_ISO_639-1
 french = """
@@ -719,27 +715,24 @@ zu	zul	zul	Zoulou	isiZulu	Zulu
 """
 
 
-
-
 def objects():
-
     LANGUAGES = {}
 
-    fn = os.path.join(os.path.dirname(__file__), 'iso-639-3_20100707.tab')
+    fn = os.path.join(os.path.dirname(__file__), "iso-639-3_20100707.tab")
     n = 0
-    #~ for ln in file(fn).readlines():
+    # ~ for ln in file(fn).readlines():
     with codecs.open(fn, encoding="UTF-8") as fd:
         for ln in fd.readlines():
             n += 1
             if ln:
-                rec = ln.split('\t')
+                rec = ln.split("\t")
                 if len(rec) != 8:
                     dd.logger.warning("Ignored line %d (len(rec) is %d)", n, len(rec))
                 elif len(rec[0]) != 3:
                     dd.logger.warning("Ignored line %d", n)
                 else:
                     language_type = rec[5]
-                    if language_type == 'L':
+                    if language_type == "L":
                         ref_name = rec[6]
                         if ref_name:
                             code = rec[0]
@@ -747,8 +740,9 @@ def objects():
                                 code = rec[1]
                             LANGUAGES[code] = dict(en=ref_name, iso2=rec[3])
                     else:
-                        dd.logger.debug("Ignored line %d : language type is %r",
-                                     n, language_type)
+                        dd.logger.debug(
+                            "Ignored line %d : language type is %r", n, language_type
+                        )
 
     n = 0
     for ln in german.splitlines():
@@ -756,16 +750,16 @@ def objects():
         if ln:
             ln = ln.strip()
             if not ln[-4:-3].isspace():
-                logger.warning("german %d: syntax", n)
+                dd.logger.warning("german %d: syntax", n)
             else:
                 code = ln[-3:]
                 name = ln[:-4]
                 d = LANGUAGES.get(code, None)
                 if d is None:
                     pass
-                    #~ logger.warning("%r in german but not english",code)
-                    #~ d = dict(en=name)
-                    #~ LANGUAGES[code] = d
+                    # ~ logger.warning("%r in german but not english",code)
+                    # ~ d = dict(en=name)
+                    # ~ LANGUAGES[code] = d
                 else:
                     d.update(de=name)
 
@@ -774,13 +768,12 @@ def objects():
         n += 1
         if ln:
             ln = ln.strip()
-            rec = ln.split('\t')
+            rec = ln.split("\t")
             if len(rec) < 6:
-                logger.warning("Ignored french:%d:%s (len(rec) is %d)",
-                               n, ln, len(rec))
+                dd.logger.warning("Ignored french:%d:%s (len(rec) is %d)", n, ln, len(rec))
             code = rec[1]
             if len(code) == 7:
-                if code[3:4] == '/':
+                if code[3:4] == "/":
                     code = code[:3]
                 else:
                     code = None
@@ -790,17 +783,18 @@ def objects():
                 if d is not None:
                     d.update(fr=name)
 
-
-
     n = 0
     for code, kw in LANGUAGES.items():
-        iso2 = kw['iso2']
-        kw = dd.babel_values('name', **kw)
-        if kw.get('name'):
+        iso2 = kw["iso2"]
+        kw = dd.babel_values("name", **kw)
+        if kw.get("name"):
             n += 1
             kw.update(iso2=iso2)
             yield Language(id=code, **kw)
         else:
-            logger.debug("%r : no name for default site language %s",
-                         code, settings.SITE.DEFAULT_LANGUAGE.django_code)
-    #~ logger.info("Installed %d languages",n)
+            dd.logger.debug(
+                "%r : no name for default site language %s",
+                code,
+                settings.SITE.DEFAULT_LANGUAGE.django_code,
+            )
+    # ~ logger.info("Installed %d languages",n)
