@@ -1,13 +1,10 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2022-2023 Rumma & Ko Ltd
+# Copyright 2022-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-import atexit
 import asyncio
 import json
 import os
-import pickle
-import signal
 import socket
 import logging
 import struct
@@ -21,106 +18,134 @@ except ImportError:
 
 # from asgiref.sync import sync_to_async
 from django.conf import settings
+
 # from django.utils import timezone
-# from channels.db import database_sync_to_async
+from channels.db import database_sync_to_async
 from channels.consumer import AsyncConsumer
 
 # from lino.modlib.linod.tasks import Tasks
-from lino.utils.socks import send_through_socket, get_from_socket
+# from lino.utils.socks import send_through_socket, get_from_socket
 
 from lino import logger
 
+# from lino.api import dd
+from .utils import BROADCAST_CHANNEL
+
 # used for debugging, when no 'log' dir exists
 # if not logger.handlers:
 #     logger.addHandler(logging.StreamHandler())
 #     logger.setLevel(logging.INFO)
 
 
-class LogReceiver(asyncio.Protocol):
+def is_slave(actor): return actor.master is not None
 
-    # def connection_made(self, transport):
-    #     print("20231019 connection_made", transport)
 
-    def data_received(self, data: bytes):
-        data = pickle.loads(data[4:]) # first four bytes gives the size of the rest of the data
-        record = logging.makeLogRecord(data)
-        # print("20231019 data_received", record)
-        # 20231019 server_logger.handle(record)
-        logger.handle(record)
+match_master = (
+    lambda ContentType, MasterModel, Master: Master == MasterModel
+    or ContentType == Master
+    or issubclass(Master, MasterModel)
+)
 
-# from .utils import log_sock_path, worker_sock_path
 
 class LinodConsumer(AsyncConsumer):
-
     # tasks: Tasks
-    clients = set()
+    # clients = set()
 
     async def log_server(self, event=None):
-        asyncio.ensure_future(self._log_server())
+        # 'log.server' in `pm linod`
+        from lino.modlib.linod.mixins import start_log_server
 
-    async def _log_server(self):
-        # 'log.server' in linod.py
-        log_sock_path = settings.SITE.log_sock_path
-        if log_sock_path is None:
-            logger.info("No log server because there is no log directory.")
-            return
-        logger.info("Start the log server...")
-        atexit.register(self.remove_sock_file)
-        loop = asyncio.get_running_loop()
-        # log_sock_path.unlink(True)
-        self.remove_sock_file()
-        server = await loop.create_unix_server(LogReceiver, log_sock_path)
-        # await server.serve_forever()
-        try:
-            async with server:
-                await server.serve_forever()
-        finally:
-            self.remove_sock_file()
-
-    def remove_sock_file(self):
-        settings.SITE.log_sock_path.unlink(missing_ok=True)
-        # worker_sock_path.unlink(True)
+        asyncio.ensure_future(start_log_server())
 
     async def run_background_tasks(self, event: dict):
-        # 'run.background.jobs' in `pm linod`
-        BackgroundTask = settings.SITE.models.linod.BackgroundTask
-        ar = settings.SITE.login()
-        asyncio.ensure_future(BackgroundTask.start_task_runner(ar))
+        # 'run.background.tasks' in `pm linod`
+        from lino.modlib.linod.mixins import start_task_runner
+        # from lino.core.utils import login
+        # ar = login(settings.SITE.plugins.linod.daemon_user)
+        from lino.core.requests import BaseRequest
+        u = await settings.SITE.user_model.objects.aget(
+            username=settings.SITE.plugins.linod.daemon_user)
+        ar = BaseRequest(user=u)
+        asyncio.ensure_future(start_task_runner(ar))
 
     async def send_push(self, event):
         # 'send.push' in notify.send_notification()
         # logger.info("Push to %s : %s", user or "everyone", data)
-        data = event['data']
-        user = event['user_id']
+        data = event["data"]
+        user = event["user_id"]
         if user is not None:
-            user = settings.SITE.models.users.User.objects.get(pk=user)
+            user = await database_sync_to_async(
+                settings.SITE.models.users.User.objects.get
+            )(pk=user)
         kwargs = dict(
             data=json.dumps(data),
             vapid_private_key=settings.SITE.plugins.notify.vapid_private_key,
             vapid_claims={
-                'sub': "mailto:{}".format(settings.SITE.plugins.notify.vapid_admin_email)
-            }
+                "sub": "mailto:{}".format(
+                    settings.SITE.plugins.notify.vapid_admin_email
+                )
+            },
         )
         if user is None:
             subs = settings.SITE.models.notify.Subscription.objects.all()
         else:
-            subs = settings.SITE.models.notify.Subscription.objects.filter(user=user)
-        for sub in subs:
+            subs = settings.SITE.models.notify.Subscription.objects.filter(
+                user=user)
+        async for sub in subs.aiterator():
             sub_info = {
-                'endpoint': sub.endpoint,
-                'keys': {
-                    'p256dh': sub.p256dh,
-                    'auth': sub.auth,
+                "endpoint": sub.endpoint,
+                "keys": {
+                    "p256dh": sub.p256dh,
+                    "auth": sub.auth,
                 },
             }
             try:
                 req = webpush(subscription_info=sub_info, **kwargs)
             except WebPushException as e:
                 if e.response.status_code == 410:
-                    sub.delete()
+                    await database_sync_to_async(sub.delete)()
                 else:
                     raise e
 
+    async def send_panel_update(self, event):
+        from lino.api import rt, dd
+
+        msg = json.loads(event["text"])
+        ups = [rt.models.resolve(aID) for aID in msg["actorIDs"]]
+        data = dict(
+            [
+                (a.actor_id, {"mk": None, "mt": None, "pk": msg["pk"]})
+                for a in ups
+                if not is_slave(a)
+            ]
+        )
+        if msg["mk"] is not None:
+            MasterModel = rt.models.resolve(msg["master_model"])
+            ContentType = rt.models.contenttypes.ContentType
+            mt = (
+                await database_sync_to_async(ContentType.objects.get_for_model)(
+                    MasterModel
+                )
+            ).pk
+            data.update(
+                **dict(
+                    [
+                        (a.actor_id, {"mk": msg["mk"],
+                         "mt": mt, "pk": msg["pk"]})
+                        for a in ups
+                        if is_slave(a)
+                        and match_master(ContentType, MasterModel, a.master)
+                    ]
+                )
+            )
+        await self.channel_layer.group_send(
+            BROADCAST_CHANNEL,
+            {
+                "type": "send.notification",
+                "text": json.dumps({"type": msg["type"], "data": data}),
+            },
+        )
+
     # async def dev_worker(self, event: dict):
     #     # dev.worker in linod
     #     # worker_sock = str(worker_sock_path)
@@ -173,14 +198,3 @@ class LinodConsumer(AsyncConsumer):
     #                 handle_msg(client_sock)
     #     finally:
     #         worker_sock_path.unlink(True)
-
-    # async def job_list(self, event):
-    #     async def do():
-    #         with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as sock:
-    #             sockd = str(settings.SITE.site_dir / 'sockd')
-    #             sock.connect(sockd)
-    #             jobs = await database_sync_to_async(self.tasks.status)()
-    #             data = pickle.dumps(jobs)
-    #             await asyncio.sleep(1)
-    #             send_through_socket(sock, data)
-    #     asyncio.ensure_future(do())
