@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2006-2023 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """See introduction in :doc:`/specs/memo`.
 
 TODO:
@@ -14,19 +13,16 @@ TODO:
   which would be empty for # and @ but "]" for memo commands.
 
 """
-from lino import logger
-
-import re
-import inspect
-from typing import Callable, Any
-
-from bs4 import BeautifulSoup, MarkupResemblesLocatorWarning
-import warnings; warnings.filterwarnings("ignore",
-    category=MarkupResemblesLocatorWarning)
-
-from django.conf import settings
-from django.db.models import Model
 from etgen import etree
+from django.conf import settings
+# from lino import logger
+import re
+import traceback
+from typing import Callable, Any
+from bs4 import BeautifulSoup, MarkupResemblesLocatorWarning
+import warnings
+
+warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)
 
 
 # COMMAND_REGEX = re.compile(r"\[(\w+)\s*((?:[^[\]]|\[.*?\])*?)\]")
@@ -45,47 +41,75 @@ class Suggester:
     Every value of :attr:`Parser.suggesters` is an instance of this.
 
     """
-    def __init__(self, trigger, data, fldname, formatter=str, getter=None):
+
+    def __init__(
+        self,
+        trigger,
+        data,
+        fldname,
+        formatter=str,
+        value=lambda x, y: getattr(x, y),
+        getter=None,
+    ):
+        """
+        `trigger` is a short text, usually one character, like "@" or "#",
+        which will trigger a list of autocomplete suggestions to pop up.
+
+        `func` is a callable expected to yield a series of suggestions to be
+        displayed in text editor.
+
+        Every suggestion is expected to be a tuple `(abbr, text)`, where `abbr`
+        is the abbreviation to come after the trigger (e.g. a username or a
+        ticket number), and text is a full description of this suggestion to be
+        displayed in the list.
+
+        Usage examples: see :mod:`lino_xl.lib.tickets` and :mod:`lino.modlib.users`
+
+        """
         self.trigger = trigger
+        if len(trigger) != 1:
+            raise Exception("Trigger text must be exactly 1 character.")
         self.data = data
         self.fldname = fldname
         self.formatter = formatter
-
-        fld = data.model._meta.get_field(fldname)
-
-        # if isinstance(fld, models.IntegerField):
-        #     searchkw = {}
+        self.value = value
 
         if getter is None:
+
             def getter(abbr):
                 return data.get(**{fldname: abbr})
 
         self.getter = getter
 
-    def get_suggestions(self, query=''):
+    def get_suggestions(self, query=""):
         flt = self.data.model.quick_search_filter(query)
         for obj in self.data.filter(flt)[:5]:
-            yield {'value': getattr(obj, self.fldname), 'title': self.formatter(obj), 'link': self.get_href(obj)}
+            # v = self.formatter(obj)
+            yield {
+                "value": self.value(obj, self.fldname),
+                "title": self.formatter(obj),
+                "link": self.get_href(obj),
+            }
 
     def get_object(self, abbr):
         return self.getter(abbr)
 
     def get_href(self, obj, ar=None):
-        an = obj.get_detail_action(ar)
-        return r"javascript:window.App.runAction({" + \
-            f"'actorId': '{self.formatter(an.actor)}', " + \
-            "'an': 'detail', " + \
-            "'rp': null, " + \
-            r"'status': {" + \
-                f"'record_id': {self.formatter(obj.pk)}" + \
-            r"}" + \
-        r"})"
+        da = obj.get_detail_action(ar)
+        return (
+            r"javascript:window.App.runAction({"
+            + f"'actorId': '{self.formatter(da.actor)}', "
+            + f"'action_full_name': '{da.action.full_name()}', "
+            + "'rp': null, "
+            + r"'status': {"
+            + f"'record_id': {self.formatter(obj.pk)}"
+            + r"}"
+            + r"})"
+        )
 
 
 class Parser:
-    """The memo parser.
-
-    """
+    """The memo parser."""
 
     safe_mode = False
 
@@ -95,21 +119,8 @@ class Parser:
         self.suggesters = dict()
 
     def add_suggester(self, *args, **kwargs):
-
         """
-
-        `trigger` is a short text, usually one character, like "@" or "#",
-        which will trigger a list of autocomplete suggestions to pop up.
-
-        `func` is a callable expected to yield a series of suggestions to be
-        displayed in text editor.
-
-        Every suggestion is expected to be a tuple `(abbr, text)`, where `abbr` is
-        the abbreviation to come after the trigger (e.g. a username or a ticket
-        number), and text is a full description of this suggestion to be displayed
-        in the list.
-
-        Usage examples: see :mod:`lino_xl.lib.tickets` and :mod:`lino.modlib.users`
+        Add a :class:`Suggester` (see there for args and kwargs).
 
         """
 
@@ -119,9 +130,13 @@ class Parser:
         self.suggesters[s.trigger] = s
 
     def compile_suggester_regex(self):
-        triggers = "".join(["\\" if key in "[\^$.|?*+(){}" else "" + key
-            for key in self.suggesters.keys()])
-        return re.compile(r"([^\w])?([" + triggers + "])(\w+)")
+        triggers = "".join(
+            [
+                r"\\" if key in r"[\^$.|?*+(){}" else "" + key
+                for key in self.suggesters.keys()
+            ]
+        )
+        return re.compile(r"([^\w])?([" + triggers + r"])(\w+)")
 
     def register_command(self, cmdname, func: Callable[[Any, str, str, dict], None]):
         """Register a memo command identified by the given text `cmd`.
@@ -136,10 +151,12 @@ class Parser:
             if issubclass(func._for_model, existing_func._for_model):
                 return
             if not issubclass(existing_func._for_model, func._for_model):
-                raise Exception("Duplicate definition of memo command '{}'".format(cmdname))
+                raise Exception(
+                    "Duplicate definition of memo command '{}'".format(cmdname)
+                )
         self.commands[cmdname] = func
 
-    def register_django_model(self, name, model, cmd=None):
+    def register_django_model(self, name, model, cmd=None, rnd=None):
         """
         Register the given string `name` as command for referring to
         database rows of the given Django database model `model`.
@@ -152,7 +169,10 @@ class Parser:
         # if rnd is None:
         #     def rnd(obj):
         #         return "[{} {}] ({})".format(name, obj.id, title(obj))
+        if rnd is None:
+            rnd = model.memo2html
         if cmd is None:
+
             def cmd(ar, s, cmdname, mentions, context):
                 # args = s.split(None, 1)
                 pk, text = split_name_rest(s)
@@ -177,9 +197,10 @@ class Parser:
                 #     return "[{} {}]".format(name, s)
                 # if not caption:
                 #     caption = obj.get_memo_title()
-                    # txt = "#{0}".format(obj.id)
-                    # kw.update(title=title(obj))
-                return obj.memo2html(ar, text)
+                # txt = "#{0}".format(obj.id)
+                # kw.update(title=title(obj))
+                # return obj.memo2html(ar, text)
+                return rnd(obj, ar, text)
                 # e = ar.obj2html(obj, txt, **kw)
                 # # return str(ar)
                 # return etree.tostring(e)
@@ -189,14 +210,14 @@ class Parser:
         #         pass
 
         cmd._for_model = model
-        cmd.__doc__ = """
+        if cmd.__doc__ is None:
+            cmd.__doc__ = rnd.__doc__ or """
 Insert a reference to the specified {}.
 
 The first argument is mandatory and specifies the primary key.
 All remaining arguments are used as the text of the link.
 """.format(model._meta.verbose_name)
 
-
         self.register_command(name, cmd)
         # if manage_usage is not None:
         #     self.register_usage_manager(name, manage_usage)
@@ -213,6 +234,7 @@ All remaining arguments are used as the text of the link.
                 # emails to the admins.
                 # logger.warning(e)
                 return self.handle_error(matchobj, e)
+
         return func
 
     def format_value(self, v):
@@ -221,11 +243,13 @@ All remaining arguments are used as the text of the link.
         return str(v)
 
     def get_referred_objects(self, text):
-        """Yield all database objects referred in the given `text` using a suggester.
+        """
+        Yield all database objects referred in the given `text` using a
+        suggester.
         """
         regex = self.compile_suggester_regex()
-        all_matchs = re.findall(regex, text)
-        for match in all_matchs:
+        all_matches = re.findall(regex, text)
+        for match in all_matches:
             suggester = self.suggesters[match[1]]
             try:
                 yield suggester.get_object(match[2])
@@ -233,24 +257,25 @@ All remaining arguments are used as the text of the link.
                 pass  #
 
     def suggester_match_func(self, ar):
-
         def func(matchobj):
             whitespace = matchobj.group(1)
             whitespace = "" if whitespace is None else whitespace
             trigger = matchobj.group(2)
             abbr = matchobj.group(3)
-            suggester = self.suggesters[trigger] # can't key error as regex is created from the keys
+            suggester = self.suggesters[
+                trigger
+            ]  # can't key error as regex is created from the keys
             try:
                 obj = suggester.get_object(abbr)
-                return whitespace + etree.tostring(ar.obj2html(obj, trigger+abbr, title=str(obj)))
+                return whitespace + ar.obj2htmls(obj, trigger + abbr, title=str(obj))
             except Exception as e:
                 # likely a mismatch or bad pk, return full match
                 # return self.handle_error(matchobj, e)
                 return matchobj.group(0)
+
         return func
 
     def cmd_match_func(self, ar, mentions, context):
-
         def func(matchobj):
             if matchobj.group(1).startswith("\\"):
                 return matchobj.group(0)[1:]
@@ -260,10 +285,10 @@ All remaining arguments are used as the text of the link.
                 return matchobj.group(0)
 
             params = matchobj.group(3)
-            params = params.replace('\\\n', ' ')
-            params = params.replace('\xa0', ' ')
-            params = params.replace('\u200b', ' ')
-            params = params.replace('&nbsp;', ' ')
+            params = params.replace("\\\n", " ")
+            params = params.replace("\xa0", " ")
+            params = params.replace("\u200b", " ")
+            params = params.replace("&nbsp;", " ")
             params = str(params.strip())
             try:
                 return self.format_value(cmdh(ar, params, cmd, mentions, context))
@@ -273,37 +298,48 @@ All remaining arguments are used as the text of the link.
                 # don't log an exception because that might cause lots of
                 # emails to the admins.
                 return self.handle_error(matchobj, e)
+
         return func
 
     def handle_error(self, mo, e):
-        #~ return mo.group(0)
+        # ~ return mo.group(0)
         msg = "[ERROR %s in %r at position %d-%d]" % (
-            e, mo.group(0), mo.start(), mo.end())
+            e,
+            mo.group(0),
+            mo.start(),
+            mo.end(),
+        )
+        # raise Exception(msg) from e
         # logger.debug(msg)
+        # print(msg, ":")
+        traceback.print_exc()
         return msg
 
     def parse_suggestions(self, src):
-        soup = BeautifulSoup(src, 'html.parser')
+        soup = BeautifulSoup(src, "html.parser")
         while True:
-            mention = soup.find('span', attrs={'class': 'mention'})
+            mention = soup.find("span", attrs={"class": "mention"})
             if mention is None:
                 break
             attrs = mention.attrs
-            link = attrs['data-link']
-            title = attrs['data-title']
+            link = attrs["data-link"]
+            title = attrs["data-title"]
             value = mention.text
             sanitized_mention = BeautifulSoup(
-                f'<a href="{link}" title="{title}">{value}</a>', 'html.parser')
-            soup.find('span', attrs={'class': "mention"}).replaceWith(sanitized_mention)
+                f'<a href="{link}" title="{title}">{value}</a>', "html.parser"
+            )
+            soup.find("span", attrs={"class": "mention"}).replaceWith(sanitized_mention)
         return str(soup)
 
     def simplify_suggestions(self, src):
-        soup = BeautifulSoup(src, 'html.parser')
+        soup = BeautifulSoup(src, "html.parser")
         while True:
-            mention = soup.find('span', attrs={'class': 'mention'})
+            mention = soup.find("span", attrs={"class": "mention"})
             if mention is None:
                 break
-            soup.find('span', attrs={'class': 'mention'}).replaceWith(BeautifulSoup(mention.text, 'html.parser'))
+            soup.find("span", attrs={"class": "mention"}).replaceWith(
+                BeautifulSoup(mention.text, "html.parser")
+            )
         return str(soup)
 
     def parse(self, src, ar=None, context=None, mentions=None):
