@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2021 Rumma & Ko Ltd
+# Copyright 2009-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
 from lino import logger
@@ -17,47 +17,36 @@ from lino.api import rt, _
 from lino.core import dbutils
 from lino.core.keyboard import Hotkey
 from lino.core.roles import SiteStaff
-from etgen.html import E
+from lino.utils.html import E
 from lino.utils.media import TmpMediaFile
 from lino.utils.pdf import merge_pdfs
 from lino.utils.xml import validate_xml
 
 from .choicelists import BuildMethods
 
-# davlink = settings.SITE.plugins.get('davlink', None)
-# has_davlink = davlink is not None and settings.SITE.use_java
-has_davlink = False
-
-
 
 class BasePrintAction(Action):
     sort_index = 50
-    url_action_name = 'print'
-    label = _('Print')
+    url_action_name = "print"
+    label = _("Print")
     build_method = None
-    hotkey = Hotkey('p', ctrl=True)
+    hotkey = Hotkey("p", code="KeyP", ctrl=True)
 
     def __init__(self, build_method=None, label=None, **kwargs):
-        super(BasePrintAction, self).__init__(label, **kwargs)
+        super().__init__(label, **kwargs)
         if build_method is not None:
             self.build_method = build_method
 
     def validate_result_file(self, bm, filename):
-        """Validate the generated file.
-        """
+        """Validate the generated file."""
         pass
 
     def attach_to_actor(self, actor, name):
-        if not dbutils.resolve_app('system'):
+        if not dbutils.resolve_app("system"):
             return False
         # if actor.__name__ == 'ExcerptsByProject':
         #     logger.info("20140401 attach_to_actor() %r", self)
-        return super(BasePrintAction, self).attach_to_actor(actor, name)
-
-    # def is_callable_from(self, caller):
-    #     # including ShowEmptyTable which is subclass of
-    #     # ShowDetail. But not callable from ShowInsert.
-    #     return isinstance(caller, (ShowTable, ShowDetail))
+        return super().attach_to_actor(actor, name)
 
     def get_print_templates(self, bm, elem):
         # print("20190506 BasePrintAction.get_print_templates", elem)
@@ -76,17 +65,19 @@ class BasePrintAction(Action):
         """Return the target filename if a document needs to be built,
         otherwise return ``None``.
         """
-        elem.before_printable_build(bm)
-        # raise Exception("20170519 before_build didn't warn")
-        filename = bm.get_target_name(self, elem)
-        if not filename:
+        # elem.before_printable_build(bm)
+        if not elem.must_build_printable(bm):
             return
-        if os.path.exists(filename):
-            logger.debug("%s %s -> overwrite existing %s.", bm, elem, filename)
-            os.remove(filename)
-        else:
-            # logger.info("20121221 makedirs_if_missing %s",os.path.dirname(filename))
-            rt.makedirs_if_missing(os.path.dirname(filename))
+        # raise Exception("20170519 before_build didn't warn")
+        filename = bm.get_target_file(self, elem).path
+        filename.unlink(missing_ok=True)
+        filename.parent.mkdir(exist_ok=True, parents=True)
+        # if filename.exists():
+        #     logger.debug("%s %s -> overwrite existing %s.", bm, elem, filename)
+        #     os.remove(filename)
+        # else:
+        #     # logger.info("20121221 makedirs_if_missing %s",os.path.dirname(filename))
+        #     rt.makedirs_if_missing(os.path.dirname(filename))
         logger.debug("%s : %s -> %s", bm, elem, filename)
         return filename
 
@@ -94,21 +85,19 @@ class BasePrintAction(Action):
         # help_url = ar.get_help_url("print", target='_blank')
         # if bm.use_webdav:
         #     url = ar.build_webdav_uri(url)
-        msg = _("Your printable document ({}) "
-                "should now open in a new browser window. "
-                # "If it doesn't, please consult %(help)s "
-                "If it doesn't, please "
-                "ask your system administrator.")
-        msg = msg.format(
-            etree.tostring(E.a(leaf, href=url), encoding="unicode"))
+        msg = _(
+            "Your printable document ({}) "
+            "should now open in a new browser window. "
+            # "If it doesn't, please consult %(help)s "
+            "If it doesn't, please "
+            "ask your system administrator."
+        )
+        msg = msg.format(etree.tostring(
+            E.a(leaf, href=url), encoding="unicode"))
         # msg %= dict(doc=leaf, help=etree.tostring(
         #     help_url, encoding="unicode"))
         kw.update(message=msg, alert=True)
-        if has_davlink and bm.use_webdav and ar.request is not None:
-            kw.update(
-                open_webdav_url=ar.request.build_absolute_uri(url))
-        else:
-            kw.update(open_url=url)
+        kw.update(open_url=url)
         ar.success(**kw)
         return
 
@@ -118,18 +107,20 @@ class BasePrintAction(Action):
         if isinstance(bm, str):
             bm = BuildMethods.get_by_value(bm)
         bm.build(ar, self, elem)
-        mf = bm.get_target(self, elem)
-        leaf = mf.parts[-1]
-        self.notify_done(ar, bm, leaf, mf.get_url(ar.request), **kw)
+        mf = bm.get_target_file(self, elem)
+        # leaf = mf.parts[-1]
+        leaf = mf.path.name
+        self.notify_done(ar, bm, leaf, mf.url, **kw)
 
 
 class DirectPrintAction(BasePrintAction):
     url_action_name = None
-    icon_name = 'printer'
+    icon_name = "printer"
+    # button_text = "🖶"  # 1F5B6
     tplname = None
 
     def __init__(self, label=None, tplname=None, build_method=None, **kw):
-        super(DirectPrintAction, self).__init__(build_method, label, **kw)
+        super().__init__(build_method, label, **kw)
         if tplname is not None:
             self.tplname = tplname
 
@@ -141,10 +132,9 @@ class DirectPrintAction(BasePrintAction):
 
 
 class WriteXmlAction(DirectPrintAction):
-    """Generate an XML file from this database object.
+    """Generate an XML file from this database object."""
 
-    """
-    combo_group = "writexml"
+    # combo_group = "writexml"
     label = _("XML")
 
     build_method = "xml"
@@ -152,24 +142,26 @@ class WriteXmlAction(DirectPrintAction):
     # show_in_toolbar = False
     xsd_file = None
 
-    def validate_result_file(self, bm, filename):
-        if not self.xsd_file:
-            return
-        logger.info("Validating {} against {} ...", filename, self.xsd_file)
-        try:
-            validate_xml(filename, self.xsd_file)
-        except Exception as e:
-            raise Warning(_(
-                "Oops, the generated XML file {} is invalid: {}").format(
-                    filename, e))
+    def validate_result_file(self, bm, xmlfile):
+        if self.xsd_file:
+            logger.info("Validate %s against %s ...", xmlfile, self.xsd_file)
+            # doc = etree.parse(xmlfile)
+            if True:
+                validate_xml(xmlfile, self.xsd_file)
+            else:
+                try:
+                    validate_xml(xmlfile, self.xsd_file)
+                except Exception as e:
+                    msg = _("XML file {} is invalid: {}").format(xmlfile, e)
+                    # print(msg)
+                    raise Warning(msg)
 
 
 class CachedPrintAction(BasePrintAction):
-
-
     # select_rows = False
-    http_method = 'POST'
-    icon_name = 'printer'
+    http_method = "POST"
+    icon_name = "printer"
+    # button_text = "🖶"  # 1F5B6
 
     def before_build(self, bm, elem):
         if elem.build_time:
@@ -177,29 +169,31 @@ class CachedPrintAction(BasePrintAction):
         return BasePrintAction.before_build(self, bm, elem)
 
     def run_from_ui(self, ar, **kw):
-
         if len(ar.selected_rows) == 1:
             obj = ar.selected_rows[0]
             bm = obj.get_build_method()
-            mf = bm.get_target(self, obj)
-            leaf = mf.parts[-1]
+            mf = bm.get_target_file(self, obj)
+            # leaf = mf.parts[-1]
+            leaf = mf.path.name
             if obj.build_time is None:
                 obj.build_target(ar)
                 ar.debug("%s has been built.", leaf)
             else:
                 ar.debug("Reused %s from cache.", leaf)
 
-            url = mf.get_url(ar.request)
-            self.notify_done(ar, bm, leaf, url, **kw)
+            # url = mf.get_url(ar.request)
+            self.notify_done(ar, bm, leaf, mf.url, **kw)
             ar.set_response(refresh=True)
             return
 
         def ok(ar2):
             # qs = [ar.actor.get_row_by_pk(pk) for pk in ar.selected_pks]
             mf = self.print_multiple(ar, ar.selected_rows)
-            ar2.success(open_url=mf.get_url(ar.request))
+            ar2.success(open_url=mf.url)
+            # ar2.success(open_url=mf.get_url(ar.request))
             # kw.update(refresh_all=True)
             # return kw
+
         msg = _("This will print %d rows.") % len(ar.selected_rows)
         ar.confirm(ok, msg, _("Are you sure?"))
 
@@ -209,36 +203,39 @@ class CachedPrintAction(BasePrintAction):
             # assert isinstance(obj,CachedPrintable)
             if obj.printed_by_id is None:
                 obj.build_target(ar)
-            pdf = obj.get_target_name()
-            assert pdf is not None
-            pdfs.append(pdf)
+            # pdf = obj.get_target_file().name
+            # assert pdf is not None
+            # pdfs.append(pdf)
+            mf = obj.get_target_file()
+            pdfs.append(mf.path)
 
-        mf = TmpMediaFile(ar, 'pdf')
-        rt.makedirs_if_missing(os.path.dirname(mf.name))
-        merge_pdfs(pdfs, mf.name)
+        mf = TmpMediaFile(ar, "pdf")
+        # rt.makedirs_if_missing(os.path.dirname(mf.name))
+        rt.makedirs_if_missing(mf.path.parent)
+        merge_pdfs(pdfs, mf.path)
         return mf
 
 
 class EditTemplate(BasePrintAction):
     sort_index = 51
-    url_action_name = 'edit_tpl'
-    label = _('Edit Print Template')
+    url_action_name = "edit_tpl"
+    label = _("Edit Print Template")
     required_roles = set([SiteStaff])
 
     def attach_to_actor(self, actor, name):
         # if not settings.SITE.is_installed('davlink'):
-        if not (settings.SITE.webdav_protocol or has_davlink):
+        if not settings.SITE.webdav_protocol:
             return False
-        return super(EditTemplate, self).attach_to_actor(actor, name)
+        return super().attach_to_actor(actor, name)
 
     def run_from_ui(self, ar, **kw):
-
         lcd = settings.SITE.confdirs.LOCAL_CONFIG_DIR
         if lcd is None:
             # ar.debug("No local config directory in %s " %
             #         settings.SITE.confdirs)
-            raise Warning("No local config directory. "
-                          "Contact your system administrator.")
+            raise Warning(
+                "No local config directory. " "Contact your system administrator."
+            )
 
         elem = ar.selected_rows[0]
         bm = elem.get_build_method()
@@ -255,15 +252,16 @@ class EditTemplate(BasePrintAction):
             if filename == local_file:
                 break
 
-        parts = ['webdav', 'config'] + parts
+        parts = ["webdav", "config"] + parts
         url = settings.SITE.build_media_url(*parts)
         # url = ar.build_webdav_uri(url)
 
-        if not (settings.SITE.webdav_protocol or has_davlink):
+        if not settings.SITE.webdav_protocol:
             msg = "cp %s %s" % (filename, local_file)
             ar.debug(msg)
-            raise Warning("WebDAV is not enabled. "
-                          "Contact your system administrator.")
+            raise Warning(
+                "WebDAV is not enabled. " "Contact your system administrator."
+            )
 
         def doit(ar):
             ar.debug("Going to open url: %s " % url)
@@ -280,8 +278,8 @@ class EditTemplate(BasePrintAction):
             ar.debug("Gonna copy %s to %s", filename, local_file)
 
             def ok(ar2):
-                logger.info(
-                    "%s made local template copy %s", ar.user, local_file)
+                logger.info("%s made local template copy %s",
+                            ar.user, local_file)
                 rt.makedirs_if_missing(os.path.dirname(local_file))
                 shutil.copyfile(filename, local_file)
                 # shutil.copy() can cause PermissionError if dst exists and is
@@ -291,27 +289,23 @@ class EditTemplate(BasePrintAction):
             msg = _(
                 "Before you can edit this template we must create a "
                 "local copy on the server. "
-                "This will exclude the template from future updates.")
+                "This will exclude the template from future updates."
+            )
             ar.confirm(ok, msg, _("Are you sure?"))
 
 
-class ClearCacheAction(Action):
+class ClearCache(Action):
     sort_index = 51
-    url_action_name = 'clear'
-    label = _('Clear cache')
-    icon_name = 'printer_delete'
-
-    # def disabled_for(self,obj,request):
-    #     if not obj.build_time:
-    #         return True
+    url_action_name = "clear"
+    label = _("Clear cache")
+    icon_name = "printer_delete"
 
     def get_action_permission(self, ar, obj, state):
         # obj may be None when Lino asks whether this action
-        # should be visible in the UI
+        # should be visible in the table toolbar
         if obj is not None and not obj.build_time:
             return False
-        return super(ClearCacheAction, self).get_action_permission(
-            ar, obj, state)
+        return super().get_action_permission(ar, obj, state)
 
     def run_from_ui(self, ar):
         elem = ar.selected_rows[0]
@@ -328,8 +322,14 @@ class ClearCacheAction(Action):
             # not depending on the database backend.
 
             t = datetime.datetime(
-                t.year, t.month, t.day, t.hour,
-                t.minute, t.second, elem.build_time.microsecond)
+                t.year,
+                t.month,
+                t.day,
+                t.hour,
+                t.minute,
+                t.second,
+                elem.build_time.microsecond,
+            )
             if settings.USE_TZ:
                 t = make_aware(t)
             if t != elem.build_time:
@@ -337,5 +337,6 @@ class ClearCacheAction(Action):
                 return ar.confirm(
                     doit,
                     _("This will discard all changes in the generated file."),
-                    _("Are you sure?"))
+                    _("Are you sure?"),
+                )
         return doit(ar)
