@@ -2,7 +2,6 @@
 # Copyright 2016-2017 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
 from .choicelists import BuildMethod, BuildMethods
 from .actions import BasePrintAction
 
@@ -20,8 +19,5 @@ class CustomBuildMethod(BuildMethod):
     @classmethod
     def create_action(cls, *args, **kwargs):
         if cls.label is not None:
-            kwargs.setdefault('label', cls.label)
+            kwargs.setdefault("label", cls.label)
         return BasePrintAction(cls(cls.name), *args, **kwargs)
-
-
-
