@@ -2,10 +2,9 @@
 # Copyright 2014-2017 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
 from django.dispatch import Signal
 
-mail_received = Signal(['msg'])
+mail_received = Signal(["msg"])
 """Sent for every incoming mail.
 
 sender:
@@ -24,4 +23,3 @@ peer, mailfrom, rcpttos, data:
 
 
 """
-
