@@ -17,6 +17,7 @@ from django.utils import translation
 from django.utils.encoding import force_str
 from django.utils.translation import gettext, ngettext
 
+from lino import logger
 from lino.api import dd, _, rt
 from lino.core import constants
 from lino.core.utils import resolve_fields_list, models_by_base
@@ -29,7 +30,7 @@ from .choicelists import Recurrences, Weekdays
 
 def format_time(t):
     if t is None:
-        return ''
+        return ""
     return t.strftime(settings.SITE.time_format_strftime)
 
 
@@ -77,8 +78,13 @@ class Lockable(EditSafe):
     lockable_fields = None
 
     locked_by = dd.ForeignKey(
-        rt.settings.SITE.user_model, null=True, default=None, blank=True,
-        on_delete=dd.SET_NULL)
+        rt.settings.SITE.user_model,
+        verbose_name=_("Locked by"),
+        null=True,
+        default=None,
+        blank=True,
+        on_delete=dd.SET_NULL,
+    )
 
     class Meta(object):
         abstract = True
@@ -99,14 +105,9 @@ class Lockable(EditSafe):
         if cls.lockable_fields is None:
             cls.lockable_fields = cls._meta.get_fields()
         else:
-            resolve_fields_list(cls, 'lockable_fields')
+            resolve_fields_list(cls, "lockable_fields")
         cls.lockable_fields = set([f.name for f in cls.lockable_fields])
 
-    # @dd.action(_("Edit"), sort_index=100, callable_from='d')
-    # def acquire_lock(self, ar):
-    #     self.lock_row(ar)
-    #     ar.success(refresh=True)
-
     def before_ui_edit(self, ar, **kwargs):
         self.lock_row(ar)
         ar.success(refresh=True)
@@ -114,21 +115,18 @@ class Lockable(EditSafe):
     def lock_row(self, ar):
         user = ar.get_user()
         if self.locked_by_id is not None:
-            msg = _("{} is being edited by another user. "
-                    "Please try again later.")
-            raise Warning(msg.format(self))
+            # msg = _("{} is being edited by another user. " "Please try again later.")
+            msg = _("{row} is being edited by {user}. Please try again later.")
+            raise Warning(msg.format(row=self, user=self.locked_by))
         for obj in self.get_lockable_rows(user):
             obj.unlock_row(ar)
         self.locked_by = user
         self.save()
-        rt.settings.SITE.logger.debug("%s locks %s.%s" % (user, self.__class__, self.pk))
+        rt.settings.SITE.logger.debug(
+            "%s locks %s.%s" % (user, self.__class__, self.pk)
+        )
         self._disabled_fields = None  # clear cache
 
-    # @dd.action(_("Abort"), sort_index=100, auto_save=None, callable_from='d')
-    # def release_lock(self, ar):
-    #     self.unlock_row(ar)
-    #     ar.success(refresh=True)
-
     def on_ui_abort(self, ar, **kwargs):
         self.unlock_row(ar)
         ar.success(refresh=True)
@@ -138,7 +136,8 @@ class Lockable(EditSafe):
 
         if self.locked_by == user:
             rt.settings.SITE.logger.debug(
-                "%s releases lock on %s.%s", user, self.__class__, self.pk)
+                "%s releases lock on %s.%s", user, self.__class__, self.pk
+            )
             self.locked_by = None
             self.save()
             self._disabled_fields = None  # clear cache
@@ -148,14 +147,23 @@ class Lockable(EditSafe):
             # locked.  This can happen e.g. when user click Save on a
             # row that wasn't locked.
             rt.settings.SITE.logger.debug(
-                "%s cannot unlock %s.%s because it was not locked", self.user, self.__class__, self.pk)
+                "%s cannot unlock %s.%s because it was not locked",
+                self.user,
+                self.__class__,
+                self.pk,
+            )
             return
 
         else:
             # locked by another user.
             # Should we inlcude a better message?
             rt.settings.SITE.logger.debug(
-                "%s cannot unlock %s.%s because it was locked by %s", self.user, self.__class__, self.pk, self.locked_by)
+                "%s cannot unlock %s.%s because it was locked by %s",
+                self.user,
+                self.__class__,
+                self.pk,
+                self.locked_by,
+            )
             return
 
     def has_row_lock(self, ar=None, user=""):
@@ -178,27 +186,30 @@ class Lockable(EditSafe):
         df = set()
         df.add("locked_by")
         if self.pk is None or self.has_row_lock(ar):
-            df.add('edit_row')
+            df.add("edit_row")
         else:
-            df.add('abort_edit')
+            df.add("abort_edit")
             if ar.bound_action.action.window_type != constants.WINDOW_TYPE_TABLE:
                 df |= self.lockable_fields
-                df.add('submit_detail')
+                df.add("submit_detail")
 
         # dd.logger.info("20181008 lockable_fields %s", self.lockable_fields)
         return df
 
 
 POSITION_TEXTS = {
-    "1": _("first"), "-1": _("last"),
-    "2": _("second"), "-2": _("second last"),
-    "3": _("third"), "-3": _("third last"),
-    "4": _("fourth"), "-4": _("fourth last"),
+    "1": _("first"),
+    "-1": _("last"),
+    "2": _("second"),
+    "-2": _("second last"),
+    "3": _("third"),
+    "-3": _("third last"),
+    "4": _("fourth"),
+    "-4": _("fourth last"),
 }
 
 
 class RecurrenceSet(Started, Ended):
-
     class Meta:
         abstract = True
 
@@ -216,8 +227,8 @@ class RecurrenceSet(Started, Ended):
     sunday = models.BooleanField(Weekdays.sunday.text, default=False)
 
     max_events = models.PositiveIntegerField(
-        _("Number of events"),
-        blank=True, null=True)
+        _("Number of events"), blank=True, null=True
+    )
 
     def build_rrule(self) -> str:
         rule = f"FREQ={self.every_unit.name.upper()};INTERVAL={self.every}"
@@ -232,7 +243,8 @@ class RecurrenceSet(Started, Ended):
     @classmethod
     def on_analyze(cls, site):
         cls.WEEKDAY_FIELDS = dd.fields_list(
-            cls, 'monday tuesday wednesday thursday friday saturday sunday')
+            cls, "monday tuesday wednesday thursday friday saturday sunday"
+        )
         super().on_analyze(site)
 
     @classmethod
@@ -241,23 +253,23 @@ class RecurrenceSet(Started, Ended):
             yield f
         for f in cls.WEEKDAY_FIELDS:
             yield f
-        yield 'every'
-        yield 'every_unit'
-        yield 'max_events'
-        #~ yield 'event_type'
-        yield 'start_date'
-        yield 'end_date'
-        yield 'start_time'
-        yield 'end_time'
+        yield "every"
+        yield "every_unit"
+        yield "max_events"
+        # ~ yield 'event_type'
+        yield "start_date"
+        yield "end_date"
+        yield "start_time"
+        yield "end_time"
 
-    def full_clean(self, *args, **kw):
+    def full_clean(self, *args, **kwargs):
         if self.every_unit == Recurrences.per_weekday:
             self.every_unit = Recurrences.weekly
         # elif self.every_unit == Recurrences.once:
         #     self.max_events = 1
         #     self.positions = ''
         #     self.every = 0
-        super().full_clean(*args, **kw)
+        super().full_clean(*args, **kwargs)
         # if self.positions:
         #     if self.every != 1:
         #         raise ValidationError(
@@ -266,10 +278,10 @@ class RecurrenceSet(Started, Ended):
     def disabled_fields(self, ar):
         rv = super().disabled_fields(ar)
         if self.every_unit == Recurrences.once:
-            rv.add('max_events')
-            rv.add('every')
+            rv.add("max_events")
+            rv.add("every")
         # if self.every_unit != Recurrences.per_weekday:
-            # rv |= self.WEEKDAY_FIELDS
+        # rv |= self.WEEKDAY_FIELDS
         return rv
 
     @dd.displayfield(_("Description"))
@@ -279,9 +291,8 @@ class RecurrenceSet(Started, Ended):
     @dd.displayfield(_("Times"))
     def times_text(self, ar):
         if self.start_time or self.end_time:
-            return "%s-%s" % (format_time(self.start_time),
-                              format_time(self.end_time))
-        return ''
+            return "%s-%s" % (format_time(self.start_time), format_time(self.end_time))
+        return ""
 
     @dd.displayfield(_("When"))
     def weekdays_text(self, ar=None):
@@ -292,17 +303,20 @@ class RecurrenceSet(Started, Ended):
         elif self.every_unit == Recurrences.once:
             if self.end_date and self.end_date != self.start_date:
                 return gettext("{0}-{1}").format(
-                    dd.fds(self.start_date), dd.fds(self.end_date))
+                    dd.fds(self.start_date), dd.fds(self.end_date)
+                )
                 # return _("From {0} until {1}").format(
                 #     dd.fdf(self.start_date), dd.fdf(self.end_date))
-            day_text = self.weekdays_text_(', ')
+            day_text = self.weekdays_text_(", ")
             if day_text:
-                return gettext("Once starting on {0} (only {1})").format(dd.fds(self.start_date), day_text)
+                return gettext("Once starting on {0} (only {1})").format(
+                    dd.fds(self.start_date), day_text
+                )
             return gettext("On {0}").format(dd.fdf(self.start_date))
         elif self.every_unit == Recurrences.daily:
             day_text = gettext("day")
         elif self.every_unit == Recurrences.weekly:
-            day_text = self.weekdays_text_(', ')
+            day_text = self.weekdays_text_(", ")
             if not day_text:
                 return gettext("Every week")
         elif self.every_unit == Recurrences.monthly:
@@ -311,8 +325,11 @@ class RecurrenceSet(Started, Ended):
                 # day_text = " {} ".format(gettext("and")).join(positions) \
                 #     + " " + self.weekdays_text_(gettext(' and '), gettext("day")) \
                 #     + " " + gettext("of the month")
-                day_text = self.positions_text_(" {} ".format(gettext("and"))) \
-                    + " " + self.weekdays_text_(" {} ".format(gettext("and")), gettext("day"))
+                day_text = (
+                    self.positions_text_(" {} ".format(gettext("and")))
+                    + " "
+                    + self.weekdays_text_(" {} ".format(gettext("and")), gettext("day"))
+                )
                 return gettext("Every {day} of the month").format(day=day_text)
             else:
                 s = ngettext("Every month", "Every {count} months", self.every)
@@ -322,17 +339,22 @@ class RecurrenceSet(Started, Ended):
             s = ngettext("Every year", "Every {count} years", self.every)
             return s.format(count=self.every)
         elif self.every_unit == Recurrences.easter:
-            s = ngettext("Every year (with Easter)",
-                "Every {count} years (with Easter)", self.every)
+            s = ngettext(
+                "Every year (with Easter)",
+                "Every {count} years (with Easter)",
+                self.every,
+            )
             return s.format(count=self.every)
         else:
             return "Invalid recurrency unit {}".format(self.every_unit)
         s = ngettext("Every {day}", "Every {ord_count} {day}", self.every)
         # num2words does not have support for every other language; and raises NotImplementedError.
         try:
-            ord_count = num2words(self.every, to='ordinal', lang=translation.get_language())
+            ord_count = num2words(
+                self.every, to="ordinal", lang=translation.get_language()
+            )
         except NotImplementedError:
-            ord_count = num2words(self.every, to='ordinal')
+            ord_count = num2words(self.every, to="ordinal")
         return s.format(ord_count=ord_count, day=day_text)
         # if self.every == 1:
         #     return gettext("Every {what}").format(what=every_text)
@@ -340,10 +362,17 @@ class RecurrenceSet(Started, Ended):
         #     ordinal=ordinal(self.every), what=every_text)
         # return gettext("Every %snd %s") % (self.every, every_text)
 
-    def weekdays_text_(self, sep, any=''):
-        if self.monday and self.tuesday and self.wednesday and self.thursday \
-            and self.friday and not self.saturday and not self.sunday:
-                return gettext("working day")
+    def weekdays_text_(self, sep, any=""):
+        if (
+            self.monday
+            and self.tuesday
+            and self.wednesday
+            and self.thursday
+            and self.friday
+            and not self.saturday
+            and not self.sunday
+        ):
+            return gettext("working day")
         weekdays = []
         for wd in Weekdays.get_list_items():
             if getattr(self, wd.name):
@@ -371,37 +400,46 @@ class RecurrenceSet(Started, Ended):
             ev.end_date = newdate + duration
 
     def get_next_alt_date(self, ar, date):
-        """Currently always returns date + 1.
-
-        """
+        """Currently always returns date + 1."""
         if date is None:
             return None
         return self.find_start_date(date + ONE_DAY)
 
-    def get_next_suggested_date(self, ar, date):
+    def get_next_suggested_date(self, date, logger=logger):
         """Find the next date after the given date, without worrying about
         conflicts.
 
         """
-        if self.every_unit in (Recurrences.once, Recurrences.never):
-            ar.debug("No next date when recurrency is %s.", self.every_unit)
+        if self.every_unit in (Recurrences.once, Recurrences.never, None):
+            logger.debug("No next date when recurrency is %s.", self.every_unit)
             return None
 
         freq = self.every_unit.du_freq
         # Recurrences without du_freq silently ignore positions
         if freq is not None and self.positions:
             bysetpos = [int(i) for i in self.positions.split()]
-            kw = dict(freq=freq,
-                count=2, dtstart=date+ONE_DAY, interval=self.every,
-                bysetpos=bysetpos)
+            kw = dict(
+                freq=freq,
+                count=2,
+                dtstart=date + ONE_DAY,
+                interval=self.every,
+                bysetpos=bysetpos,
+            )
             weekdays = []
-            if self.monday: weekdays.append(0)
-            if self.tuesday: weekdays.append(1)
-            if self.wednesday: weekdays.append(2)
-            if self.thursday : weekdays.append(3)
-            if self.friday : weekdays.append(4)
-            if self.saturday : weekdays.append(5)
-            if self.sunday: weekdays.append(6)
+            if self.monday:
+                weekdays.append(0)
+            if self.tuesday:
+                weekdays.append(1)
+            if self.wednesday:
+                weekdays.append(2)
+            if self.thursday:
+                weekdays.append(3)
+            if self.friday:
+                weekdays.append(4)
+            if self.saturday:
+                weekdays.append(5)
+            if self.sunday:
+                weekdays.append(6)
             if len(weekdays):
                 kw.update(byweekday=weekdays)
             rr = rrule(**kw)
@@ -411,7 +449,7 @@ class RecurrenceSet(Started, Ended):
             try:
                 return rr[0].date()
             except IndexError:
-                ar.debug("No date matches your recursion rule(%s).", kw)
+                logger.debug("No date matches your recursion rule(%s).", kw)
                 return None
 
         if self.every_unit == Recurrences.per_weekday:
@@ -439,12 +477,19 @@ class RecurrenceSet(Started, Ended):
         of this recurrence set.
 
         """
-        if self.monday or self.tuesday or self.wednesday or self.thursday \
-           or self.friday or self.saturday or self.sunday:
+        if (
+            self.monday
+            or self.tuesday
+            or self.wednesday
+            or self.thursday
+            or self.friday
+            or self.saturday
+            or self.sunday
+        ):
             wd = date.isoweekday()  # Monday:1, Tuesday:2 ... Sunday:7
             wd = Weekdays.get_by_value(str(wd))
             rv = getattr(self, wd.name)
-            #~ logger.info('20130529 is_available_on(%s) -> %s -> %s',date,wd,rv)
+            # ~ logger.info('20130529 is_available_on(%s) -> %s -> %s',date,wd,rv)
             return rv
         return True
 
@@ -499,6 +544,7 @@ class RecurrenceSet(Started, Ended):
         else:
             event.end_date = None
 
-dd.update_field(RecurrenceSet, 'start_date', default=dd.today)
 
-RecurrenceSet.set_widget_options('every', hide_sum=True)
+dd.update_field(RecurrenceSet, "start_date", default=dd.today)
+
+RecurrenceSet.set_widget_options("every", hide_sum=True)
