@@ -1,6 +1,5 @@
-# Copyright 2011-2023 Rumma & Ko Ltd
+# Copyright 2011-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the choicelists for :mod:`lino.modlib.users`.
 
 """
@@ -24,14 +23,19 @@ class UserType(Choice):
     # authenticated = True
     # """Whether users with this user_type should be considered authenticated."""
 
-    def __init__(self, value, text, role_class,
-                 name=None,  # authenticated=True,
-                 readonly=False,
-                 **kw):
+    def __init__(
+        self,
+        value,
+        text,
+        role_class,
+        name=None,  # authenticated=True,
+        readonly=False,
+        **kw,
+    ):
         # if value is None:
         #     value = self.__module__.split('.')[-2] + '.' \
         #         + self.__class__.__name__
-        super(UserType, self).__init__(value, text, name)
+        super().__init__(value, text, name)
         if isinstance(role_class, (tuple, list)):
             raise Exception("No longer supported")
             # self.roles = set([rc() for rc in role_class])
@@ -44,12 +48,12 @@ class UserType(Choice):
         return UserTypeContext(self)
 
     def attach(self, cls):
-        super(UserType, self).attach(cls)
-        self.kw.setdefault('hidden_languages', cls.hidden_languages)
-        if 'remark' not in self.kw:
+        super().attach(cls)
+        self.kw.setdefault("hidden_languages", cls.hidden_languages)
+        if "remark" not in self.kw:
             s = self.role.__doc__
             if s is not None:
-                self.kw['remark'] = unindent(s)
+                self.kw["remark"] = unindent(s)
 
         for k, vf in cls.virtual_fields.items():
             if vf.has_default():
@@ -62,7 +66,8 @@ class UserType(Choice):
 
         if self.hidden_languages is not None:
             self.hidden_languages = set(
-                settings.SITE.resolve_languages(self.hidden_languages))
+                settings.SITE.resolve_languages(self.hidden_languages)
+            )
 
         del self.kw
 
@@ -76,31 +81,17 @@ class UserType(Choice):
     #
 
     def mask_notifications(self, *args):
-        """
-        Add the given notification message types to the notification mask.
-        Notifications will not be forwarded to users whose user_type
-        filters them away.
-        """
         for mt in args:
             self.mask_message_types.add(mt)
 
     def has_required_roles(self, required_roles):
-        """
-        Return `True` if this user type's :attr:`role` satisfies the
-        specified requirements.
-        """
         # for role in self.roles:
         return self.role.satisfies_requirement(required_roles)
 
     def find_menu_item(self, bound_action):
-        """
-        Find the item of the main menu for the specified bound action.
-        """
         mnu = settings.SITE.get_site_menu(self)
         return mnu.find_item(bound_action)
 
-##
-
 
 class UserTypes(ChoiceList):
     required_roles = dd.login_required(SiteAdmin)
@@ -117,15 +108,10 @@ class UserTypes(ChoiceList):
     # readonly = models.BooleanField(_("Read-only"), default=False)
 
     hidden_languages = settings.SITE.hidden_languages
-    """Default value for the
-    :attr:`hidden_languages<UserType.hidden_languages>` of newly
-    attached choice item.
 
-    """
-
-    @dd.displayfield(_("User role"))
-    def user_role(cls, obj, ar):
-        return obj.role.__class__.__module__ + '.' + obj.role.__class__.__name__
+    # @dd.displayfield(_("User role"))
+    # def user_role(cls, obj, ar):
+    #     return obj.role.__class__.__module__ + '.' + obj.role.__class__.__name__
 
     _anonymous_user = None
 
@@ -133,12 +119,12 @@ class UserTypes(ChoiceList):
     def get_anonymous_user(cls):
         if cls._anonymous_user is None:
             cls._anonymous_user = AnonymousUser(
-                'anonymous', cls.get_by_name('anonymous', None))
+                "anonymous", cls.get_by_name("anonymous", None)
+            )
         return cls._anonymous_user
         # return AnonymousUser()
 
 
-
 # add = UserTypes.add_item
 # add('000', _("Anonymous"), UserRole, 'anonymous', readonly=True)
 # add('100', _("User"), SiteUser, 'user')
