@@ -1,10 +1,12 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2020 Rumma & Ko Ltd
+# Copyright 2011-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
 from django.db import models
+from django.db.models import Q
 
 from django.utils.translation import gettext_lazy as _
+
 # from django.utils.translation import pgettext_lazy as pgettext
 from django.utils.text import format_lazy
 from django.conf import settings
@@ -15,15 +17,15 @@ from lino.core.exceptions import ChangedAPI
 from lino.core import model
 from lino.core import actions
 from lino.core import dbtables
-from lino.core.roles import SiteStaff
+from lino.core.roles import SiteStaff, SiteAdmin
 from lino.modlib.printing.mixins import Printable
 
 from .roles import Helper, AuthorshipTaker
 
 
-
 class Authored(Printable):
-    class Meta(object):
+
+    class Meta:
         abstract = True
 
     # author_field_name = None
@@ -57,18 +59,19 @@ class Authored(Printable):
         if ar and ba.action.select_rows:
             user = ar.get_user()
             author = self.get_author()
-            if author != ar.user \
-               and (ar.subst_user is None or author != ar.subst_user) \
-               and not user.user_type.has_required_roles(
-                   self.manager_roles_required):
+            if (
+                author != ar.user
+                and (ar.subst_user is None or author != ar.subst_user)
+                and not user.user_type.has_required_roles(self.manager_roles_required)
+            ):
                 return ba.action.readonly
         return True
 
-    @classmethod
-    def on_analyze(cls, site):
-        if hasattr(cls, 'manager_level_field'):
-            raise ChangedAPI("{0} has a manager_level_field".format(cls))
-        super().on_analyze(site)
+    # @classmethod
+    # def on_analyze(cls, site):
+    #     if hasattr(cls, "manager_level_field"):
+    #         raise ChangedAPI("{0} has a manager_level_field".format(cls))
+    #     super().on_analyze(site)
 
     # no longer needed after 20170826
     # @classmethod
@@ -85,7 +88,7 @@ class Authored(Printable):
     def get_simple_parameters(cls):
         for p in super().get_simple_parameters():
             yield p
-        yield 'user'  # cls.author_field_name)
+        yield "user"  # cls.author_field_name)
 
     def get_print_language(self):
         u = self.get_author()
@@ -95,16 +98,19 @@ class Authored(Printable):
 
 
 class UserAuthored(Authored):
-    class Meta(object):
+
+    class Meta:
         abstract = True
 
-    workflow_owner_field = 'user'
+    workflow_owner_field = "user"
     # author_field_name = 'user'
     user = dd.ForeignKey(
-        'users.User',
+        "users.User",
         verbose_name=_("Author"),
         related_name="%(app_label)s_%(class)s_set_by_user",
-        blank=True, null=True)
+        blank=True,
+        null=True,
+    )
 
     def set_author(self, user):
         self.user = user
@@ -137,9 +143,8 @@ class UserAuthored(Authored):
         # return self.user.timezone or settings.TIME_ZONE
 
 
-
 class My(dbtables.Table):
-    """Mixin for tables on :class:`Authored` which sets the requesting
+    """Mixin for tables on :class:`Authored` that sets the requesting
     user as default value for the :attr:`author` filter parameter.
 
     If the table's model does *not* inherit from :class:`Authored`,
@@ -152,16 +157,22 @@ class My(dbtables.Table):
     :mod:`lino_xl.lib.reception`.
 
     """
+
+    abstract = True
     # required_roles = dd.login_required()
 
     @classmethod
     def get_actor_label(self):
+        if self._label is not None:
+            return self._label
         if self.model is None:
-            return self._label or self.__name__
-        # return self._label or \
-        #     _("My %s") % self.model._meta.verbose_name_plural
-        return self._label or \
-            format_lazy(_("My {}"), self.model._meta.verbose_name_plural)
+            return self.__name__
+        return format_lazy(_("My {}"), self.model._meta.verbose_name_plural)
+
+    @classmethod
+    def setup_request(cls, ar):
+        super().setup_request(ar)
+        ar.obvious_fields.add("user")
 
     @classmethod
     def param_defaults(self, ar, **kw):
@@ -170,7 +181,7 @@ class My(dbtables.Table):
         # k = self.author_field_name or self.model.author_field_name
         # kw[k] = ar.get_user()
         # kw[self.model.author_field_name] = ar.get_user()
-        kw['user'] = ar.get_user()
+        kw["user"] = ar.get_user()
         return kw
 
 
@@ -179,7 +190,7 @@ class StartPlan(dd.Action):
     # icon_name = 'basket'
     sort_index = 52
     select_rows = False
-    http_method = 'POST'
+    http_method = "POST"
     update_after_start = False
 
     def get_button_label(self, actor):
@@ -204,18 +215,19 @@ class StartPlan(dd.Action):
     def run_from_ui(self, ar, **kw):
         options = self.get_plan_options(ar)
         pm = self.get_plan_model(ar)
-        plan = pm.run_start_plan(ar.get_user(), **options)
-        # plan = self.defining_actor.model.run_start_plan(
+        plan = pm.create_user_plan(ar.get_user(), **options)
+        # plan = self.defining_actor.model.create_user_plan(
         #     ar.get_user(), **options)
         if self.update_after_start:
             plan.run_update_plan(ar)
-        ar.goto_instance(plan)
+        ar.goto_instance(plan, refresh=True)
 
 
 class UpdatePlan(dd.Action):
     label = _("Update plan")
-    icon_name = 'lightning'
+    icon_name = "lightning"
     sort_index = 53
+
     # button_text = "F"
 
     def run_from_ui(self, ar, **kw):
@@ -228,13 +240,15 @@ class UserPlan(UserAuthored):
     class Meta:
         abstract = True
 
-    today = models.DateField(_("Today"), default=dd.today, editable=False)
+    today = models.DateField(_("Today"), default=dd.today)
+    # 20240621 today is no longer readonly because the user may want to continue
+    # a plan they started yesterday
 
     update_plan = UpdatePlan()
     start_plan = StartPlan()
 
     @classmethod
-    def run_start_plan(cls, user, **options):
+    def create_user_plan(cls, user, **options):
         qs = cls.objects.filter(user=user)
         num = qs.count()
         if num == 1:
@@ -244,7 +258,7 @@ class UserPlan(UserAuthored):
                 if getattr(plan, k) != v:
                     changed = True
                     setattr(plan, k, v)
-            if 'today' not in options:
+            if "today" not in options:
                 if plan.today != dd.today():
                     plan.today = dd.today()
                     changed = True
@@ -252,20 +266,23 @@ class UserPlan(UserAuthored):
                 plan.reset_plan()
         else:
             if num > 1:
-                dd.logger.warning("Got {} {} for {}".format(
-                num, cls._meta.verbose_name_plural, user))
+                dd.logger.warning(
+                    f"Got {num} {cls._meta.verbose_name_plural} for {user}")
                 qs.delete()
             plan = cls(user=user, **options)
         plan.full_clean()
         plan.save()
         return plan
 
+    def __str__(self):
+        return _("{plan} by {user}").format(
+            plan=self._meta.verbose_name, user=self.user)
+
     def run_update_plan(self, ar):
         raise NotImplementedError()
 
     def reset_plan(self):
-        """Delete all cached data for this plan.
-        """
+        """Delete all cached data for this plan."""
         pass
 
 
@@ -276,10 +293,11 @@ class AssignToMe(dd.Action):
     :attr:`Assignable.assigned_to`.
 
     """
+
     label = _("Assign to me")
     show_in_workflow = True
     show_in_toolbar = False  # added 20180515 for noi. possible side
-                          # effects in welfare.
+    # effects in welfare.
 
     # readonly = False
     required_roles = dd.login_required(Helper)
@@ -287,7 +305,7 @@ class AssignToMe(dd.Action):
     # button_text = u"\u2698"  # FLOWER (⚘)
     # button_text = u"\u26d1"  # ⛑
     # button_text = u"\u261D"  # ☝
-    button_text = u"\u270B"  # ✋
+    button_text = "\u270B"  # ✋
 
     # help_text = _("You become assigned to this.")
 
@@ -309,8 +327,7 @@ class AssignToMe(dd.Action):
             obj.save()
             ar.set_response(refresh=True)
 
-        ar.confirm(ok, _("You become assigned to this."),
-                   _("Are you sure?"))
+        ar.confirm(ok, _("You become assigned to this."), _("Are you sure?"))
 
 
 class TakeAuthorship(dd.Action):
@@ -332,6 +349,7 @@ class TakeAuthorship(dd.Action):
     and is invited to "take" the appointment which has been agreed for
     him.
     """
+
     label = _("Take")
     show_in_workflow = True
     show_in_toolbar = False
@@ -343,7 +361,7 @@ class TakeAuthorship(dd.Action):
 
     required_roles = dd.login_required(AuthorshipTaker)
 
-    button_text = u"\u2691"
+    button_text = "\u2691"  # flag (⚑)
 
     # def get_action_permission(self, ar, obj, state):
     #     # new since 20160814
@@ -361,40 +379,43 @@ class TakeAuthorship(dd.Action):
 
     def run_from_ui(self, ar, **kw):
         obj = ar.selected_rows[0]
+
         # obj is an Assignable
 
         def ok(ar):
             obj.set_author(ar.get_user())
             # obj.user = ar.get_user()
             obj.assigned_to = None
-            #~ kw = super(TakeAuthorship,self).run(obj,ar,**kw)
+            # ~ kw = super(TakeAuthorship,self).run(obj,ar,**kw)
             obj.save()
             ar.set_response(refresh=True)
 
-        ar.confirm(ok,
-                   _("You take responsibility for {}.").format(obj),
-                   _("Are you sure?"))
+        ar.confirm(
+            ok, _("You take responsibility for {}.").format(obj), _("Are you sure?")
+        )
 
 
 class Assignable(Authored):
     """.. attribute:: assigned_to
 
-        This field is usually empty.  Setting it to another user means
-        "I am not fully responsible for this item".
+    This field is usually empty.  Setting it to another user means
+    "I am not fully responsible for this item".
 
-        This field is cleared when somebody calls
-        :class:`TakeAuthorship` on the object.
+    This field is cleared when somebody calls
+    :class:`TakeAuthorship` on the object.
 
     """
 
-    class Meta(object):
+    class Meta:
         abstract = True
 
     assigned_to = dd.ForeignKey(
         settings.SITE.user_model,
         verbose_name=_("Assigned to"),
         related_name="%(app_label)s_%(class)s_set_assigned",
-        blank=True, null=True)
+        blank=True,
+        null=True,
+    )
 
     take = TakeAuthorship()
     assign_to_me = AssignToMe()
@@ -413,23 +434,96 @@ class Assignable(Authored):
         s = super().disabled_fields(ar)
         user = ar.get_user()
         if self.assigned_to == user:
-            s.add('assign_to_me')
+            s.add("assign_to_me")
 
         if self.disable_author_assign and user == self.get_author():
-            s.add('assign_to_me')
-            s.add('take')
+            s.add("assign_to_me")
+            s.add("take")
         return s
 
-
     def on_create(self, ar):
         # 20130722 e.g. CreateClientEvent sets assigned_to it explicitly
         if self.assigned_to is None:
             self.assigned_to = ar.subst_user
         super().on_create(ar)
 
-
     @classmethod
     def get_simple_parameters(cls):
         for p in super().get_simple_parameters():
             yield p
-        yield 'assigned_to'
+        yield "assigned_to"
+
+    if dd.is_installed('notify'):
+
+        # Send notification of assignment
+        def assigned_to_changed(self, ar):
+
+            if (self.assigned_to is None):
+                return
+            if ar.user == self.assigned_to and not ar.user.notify_myself:
+                return
+
+            ctx = dict(user=ar.user, what=self.obj2memo())
+
+            def msg():
+                subject = _("{user} assigned you to {what}").format(**ctx)
+                return (subject, tostring(E.span(subject)))
+
+            mt = rt.models.notify.MessageTypes.change
+
+            rt.models.notify.Message.emit_notification(
+                ar, self, mt, msg,
+                [(self.assigned_to, self.assigned_to.mail_mode)])
+
+
+# class Groupwise(dd.Model):
+class PrivacyRelevant(dd.Model):
+
+    class Meta:
+        abstract = True
+
+    private = models.BooleanField(
+        _("Confidential"), default=dd.plugins.users.private_default)
+    group = dd.ForeignKey("groups.Group", blank=True, null=True)
+
+    @classmethod
+    def get_request_queryset(cls, ar, **filter):
+        # Show only rows that belong to a group of which I am a member or which
+        # is public.
+        qs = super().get_request_queryset(ar, **filter)
+        user = ar.get_user()
+        if user.is_anonymous:
+            # if dd.is_installed('groups'):
+            #     qs = qs.filter(group__private=False)
+            return qs.filter(private=False)
+        if user.user_type.has_required_roles([SiteAdmin]):
+            return qs
+        flt = Q(private=False)
+        if issubclass(cls, UserAuthored):
+            flt |= Q(user=user)
+        if dd.is_installed('groups'):
+            # flt |= Q(group__private=False)
+            flt |= Q(group__members__user=user)
+        qs = qs.filter(flt).distinct()
+        return qs
+
+    if dd.is_installed("groups"):
+
+        def on_create(self, ar):
+            # if not ar.is_obvious_field('group'):
+            #     self.group = ar.get_user().current_group
+            super().on_create(ar)
+            if not self.group_id:
+                self.group = ar.get_user().current_group
+            if self.group:
+                self.private = self.group.private
+
+        def get_default_group(self):
+            return None  # dd.plugins.groups.get_default_group()
+
+        def full_clean(self):
+            if not self.group_id:
+                self.group = self.get_default_group()
+            if self.group and self.group.private:
+                self.private = True
+            super().full_clean()
