@@ -1,26 +1,15 @@
-# Copyright 2011-2017 Rumma & Ko Ltd
+# Copyright 2011-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-"""Utilities for managing the :ref:`current_user_type`.
-
-
-"""
-
-
 import threading
+from django.conf import settings
+from lino.utils import camelize
+
 user_profile_rlock = threading.RLock()
 _for_user_profile = None
 
 
 def with_user_profile(profile, func, *args, **kwargs):
-    """Run the given callable `func` with the given user type
-    activated. Optional args and kwargs are forwarded to the callable,
-    and the return value is returned.
-
-    This might get deprecated some day since we now have the
-    :meth:`lino.modlib.users.UserType.context` method
-
-    """
     global _for_user_profile
 
     with user_profile_rlock:
@@ -30,17 +19,20 @@ def with_user_profile(profile, func, *args, **kwargs):
         _for_user_profile = old
         return rv
 
+
 def set_user_profile(profile):
-    """Used in doctests to set a default profile"""
+    # Used in doctests to set a default profile
     global _for_user_profile
     _for_user_profile = profile
 
+
 def get_user_profile():
     return _for_user_profile
 
 
 class UserTypeContext(object):
-    """A context manager which activates a current user type."""
+    # A context manager which activates a current user type.
+
     def __init__(self, user_type):
         self.user_type = user_type
 
@@ -60,8 +52,6 @@ class UserTypeContext(object):
 
 # set_for_user_profile = set_user_profile
 
-from lino.utils import camelize
-from lino.api import rt
 
 def create_user(username, user_type=None, with_person=False, **kw):
     # with_person was used in noi1e demo
@@ -75,7 +65,7 @@ def create_user(username, user_type=None, with_person=False, **kw):
         kw.update(username=username, user_type=user_type)
         kw.update(first_name=first_name)
         # kw.update(partner=person)
-        return rt.models.users.User(**kw)
+        return settings.SITE.models.users.User(**kw)
     else:
         # return dd.plugins.skills.supplier_model(first_name=first_name)
-        return rt.models.contacts.Person(first_name=first_name)
+        return settings.SITE.models.contacts.Person(first_name=first_name)
