@@ -1,6 +1,5 @@
-# Copyright 2009-2022 Rumma & Ko Ltd
+# Copyright 2009-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """This defines Lino default settings. You include this (directly or
 indirectly) into your local :xfile:`settings.py` using::
 
@@ -9,19 +8,16 @@ indirectly) into your local :xfile:`settings.py` using::
 """
 
 from lino.api.ad import Site
-# from lino.api import rt
 
-# from lino.modlib.users.models import User
 
 def TIM2LINO_LOCAL(alias, obj):
-    """Hook for local special treatment on instances that have been
-    imported from TIM.
-
-    """
+    # Hook for local special treatment on instances that have been
+    # imported from TIM. Deprecated. Historic.
     return obj
 
 
 def TIM2LINO_USERNAME(userid):
+    # Deprecated. Historic.
     if userid == "WRITE":
         return None
     return userid.lower()
@@ -44,7 +40,7 @@ MANAGERS = ADMINS
 # system time zone.
 # TIME_ZONE = 'Europe/Brussels'
 # TIME_ZONE = None
-TIME_ZONE = 'UTC'  # 20190925 avoid "pytz.exceptions.UnknownTimeZoneError: None"
+TIME_ZONE = "UTC"  # 20190925 avoid "pytz.exceptions.UnknownTimeZoneError: None"
 # TIME_ZONE = 'America/Chicago'
 
 # Language code for this installation. All choices can be found here:
@@ -61,26 +57,40 @@ USE_I18N = True
 EMAIL_HOST = "mail.example.com"
 # EMAIL_PORT = ""
 
-DATABASES = {
-    'default': {
-        'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': ':memory:'
-    }
-}
+DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}
 
 # Django wants system admins to define their own `SECRET_KEY
-# <https://docs.djangoproject.com/en/5.0/ref/settings/#secret-key>`__
+# <https://docs.djangoproject.com/en/5.2/ref/settings/#secret-key>`__
 # setting.  Hint: as long as you're on a development server you just
 # put some non-empty string and that's okay.
 
 SECRET_KEY = "20227"  # see :djangoticket:`20227`
 
-TEST_RUNNER = 'django.test.runner.DiscoverRunner'
+TEST_RUNNER = "django.test.runner.DiscoverRunner"
 
 # disable migrations:
 # MIGRATION_MODULES = dict(contenttypes='lino.fake_migrations', sessions='lino.fake_migrations')
 
-
-
 # 20161114
-ALLOWED_HOSTS = ['testserver', 'localhost', '127.0.0.1', '::1']
+ALLOWED_HOSTS = ["testserver", "localhost", "127.0.0.1", "::1"]
+
+AUTH_PASSWORD_VALIDATORS = [
+    {
+        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
+    },
+    {
+        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
+    },
+    {
+        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
+    },
+    {
+        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
+    },
+]
+
+# How to set custom options:
+# min_len_validator = AUTH_PASSWORD_VALIDATORS[1]
+# min_len_validator["OPTIONS"] = {
+#     "min_length": 6,
+# }
