@@ -1,6 +1,5 @@
 # Copyright 2013-2016 Luc Saffre
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 This is used by tests/__init__.py
 """
@@ -9,8 +8,9 @@ from .settings import *
 
 
 class Site(Site):
-    languages = 'en de fr et nl pt-br es'
-    project_name = 'lino_std'
+    languages = "en de fr et nl pt-br es"
+    project_name = "lino_std"
+
 
 SITE = Site(globals())
 """
