@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2002-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Some extensions for Sphinx.
 
 .. autosummary::
@@ -19,11 +18,14 @@ from pathlib import Path
 from typing import Any, Dict, List, Tuple
 
 import synodal
-from rstgen import sphinxconf
-from rstgen.sphinxconf import interproject
+
+# from rstgen import sphinxconf
+# from rstgen.sphinxconf import interproject
 from sphinx.ext import autosummary
 from sphinx.ext.autodoc.importer import import_module
-from sphinx.util import logging ; logger = logging.getLogger(__name__)
+from sphinx.util import logging
+
+logger = logging.getLogger(__name__)
 
 if False:
     # temporary patch reports failed imports
@@ -31,7 +33,7 @@ if False:
         """Import a Python object given its full name."""
 
         try:
-            name_parts = name.split('.')
+            name_parts = name.split(".")
 
             # # 20200429 first try whether it is a full module name
             # try:
@@ -46,7 +48,7 @@ if False:
             #     pass
 
             # try first interpret `name` as MODNAME.OBJ
-            modname = '.'.join(name_parts[:-1])
+            modname = ".".join(name_parts[:-1])
             if modname:
                 try:
                     mod = import_module(modname)
@@ -59,7 +61,7 @@ if False:
             modname = None
             for j in reversed(range(1, len(name_parts) + 1)):
                 last_j = j
-                modname = '.'.join(name_parts[:j])
+                modname = ".".join(name_parts[:j])
                 try:
                     import_module(modname)
                 except ImportError as e:
@@ -95,35 +97,29 @@ def configure(globals_dict, django_settings_module=None):
       from lino.sphinxcontrib import configure
       configure(globals())
 
-    This will also call :func:`rstgen.sphinxconf.configure`, and then
-    adds more things specific to the Lino framework.
-
-    This adds the main Lino doctrees to your `intersphinx_mapping
-    <https://www.sphinx-doc.org/en/master/usage/extensions/intersphinx.html#confval-intersphinx_mapping>`__.
+    This configure() does *not* call atelier.sphinxconf.configure(). It can be
+    used with either atelier.sphinxconf.configure() or
+    rstgen.sphinxconf.configure()
 
     You can specify an additional positional argument `django_settings_module`
-    (the name of a Django settings module).  If this argument is specified, call
+    (the name of a Django settings file).  If this argument is specified, call
     :meth:`lino.startup` with it.
 
     """
     if django_settings_module is not None:
         from lino import startup
+
         startup(django_settings_module)
-        print("Django started with DJANGO_SETTINGS_MODULE={}.".format(django_settings_module))
+        print(
+            "Django started with DJANGO_SETTINGS_MODULE={}.".format(
+                django_settings_module
+            )
+        )
         from django.conf import settings
+
         print(settings.SITE.welcome_text())
 
-    # Note that this configure() does *not* call atelier.sc.configure(). It can
-    # be used with either atelier.sc.configure() or rstgen.sc.configure()
     # sphinxconf.configure(globals_dict, project)
 
-    intersphinx_mappings = {}
-
-    for r in synodal.REPOS_LIST:
-        if r.public_url and r.nickname:
-            intersphinx_mappings[r.nickname] = (r.public_url, None)
-
-    # 20230314 interproject.configure(globals_dict, **intersphinx_mappings)
-
-    extlinks = globals_dict.setdefault('extlinks', dict())
+    extlinks = globals_dict.setdefault("extlinks", dict())
     extlinks.update(synodal.SPHINX_EXTLINKS)
