@@ -1,8 +1,7 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2016-2020 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
-'''
+"""
 Causes one or several :xfile:`help_texts.py` files to be generated
 after each complete build of the doctree.
 
@@ -71,40 +70,41 @@ Example of a field description::
         </paragraph>
       </desc_content>
     </desc>
-'''
+"""
 
 from collections import OrderedDict
 
 from docutils import nodes
 from docutils import core
 from sphinx import addnodes
-from sphinx.util import logging ; logger = logging.getLogger(__name__)
+from sphinx.util import logging
+
+logger = logging.getLogger(__name__)
 
 from importlib import import_module
 
 from pathlib import Path
 from lino.modlib.help.utils import simplify_name
 
-
-useless_starts = set(['lino.core'])
-useless_endings = set(['.My', '.ByUser'])
+useless_starts = set(["lino.core", "lino.api"])
+useless_endings = set([".My", ".ByUser"])
 # useless_endings = set(['.VentilatingTable', '.My', '.ByUser',
 #                        '.Table', '.AbstractTable', '.VirtualTable',
 #                        '.Actor'])
 
 HEADER = """# -*- coding: UTF-8 -*-
 # generated by lino.sphinxcontrib.help_text_builder
+# fmt: off
 from django.utils.translation import gettext_lazy as _
 """
 
 
 def node2html(node):
     parts = core.publish_from_doctree(node, writer_name="html")
-    return parts['body']
+    return parts["body"]
 
 
 class HelpTextExtractor(object):
-
     def initialize(self, app):
         # print("20200815 HelpTextExtractor.initialize", app)
         self.name2dict = dict()
@@ -117,14 +117,13 @@ class HelpTextExtractor(object):
         # print(20160725, targets)
         for root, modname in targets.items():
             mod = import_module(modname)
-            htf = Path(mod.__file__).parent / 'help_texts.py'
+            htf = Path(mod.__file__).parent / "help_texts.py"
             # if not htf.exists():
             #     raise Exception("No such file: {}".format(htf))
             self.name2file[root] = str(htf)
             self.name2dict[root] = OrderedDict()
 
-        print("Collecting help texts for {}".format(
-            ' '.join(self.name2file.values())))
+        print("Collecting help texts for {}".format(" ".join(self.name2file.values())))
 
     def extract_help_texts(self, app, doctree):
         # print("20200815 HelpTextExtractor.extract_help_texts")
@@ -136,12 +135,12 @@ class HelpTextExtractor(object):
         # for node in doctree.traverse():
         #     self.node_classes.add(node.__class__)
         for node in doctree.traverse(addnodes.desc):
-            if node['domain'] == 'py':
-                if node['objtype'] == 'class':
+            if node["domain"] == "py":
+                if node["objtype"] == "class":
                     self.store_content(node)
-                elif node['objtype'] == 'attribute':
+                elif node["objtype"] == "attribute":
                     self.store_content(node)
-                elif node['objtype'] == 'method':
+                elif node["objtype"] == "method":
                     self.store_content(node)
             # else:
             #     print("20200815 domain is not py", node['domain'])
@@ -157,12 +156,13 @@ class HelpTextExtractor(object):
         #     len(app.env.found_docs), len(app.env.all_docs),
         #     self.docs_processed))
         if self.docs_processed < len(app.env.found_docs):
-        # if self.docs_processed < len(app.env.all_docs):
+            # if self.docs_processed < len(app.env.all_docs):
             logger.info(
                 "Don't write help_texts.py files because "
                 "only {0} of {1} docs have been processed".format(
-                    self.docs_processed,
-                    len(app.env.found_docs)))
+                    self.docs_processed, len(app.env.found_docs)
+                )
+            )
             return
         # print("20200815 HelpTextExtractor.write_help_texts_files", len(self.name2file))
         for k, fn in self.name2file.items():
@@ -171,8 +171,7 @@ class HelpTextExtractor(object):
                 logger.info("No help texts for {}".format(k))
                 continue
             # fn = os.path.join(self.outdir, 'help_texts.py')
-            logger.info("Writing {} help texts for {} to {}".format(
-                len(texts), k, fn))
+            logger.info("Writing {} help texts for {} to {}".format(len(texts), k, fn))
 
             with open(fn, "w") as fd:
 
@@ -213,7 +212,7 @@ class HelpTextExtractor(object):
 
         if not content:
             return
-        content = '\n'.join(content)
+        content = "\n".join(content)
         if '"""' in content:
             msg = '{} : First paragraph of content may not contain \'"""\'. '
             raise Exception(msg.format(sig))
@@ -228,7 +227,7 @@ class HelpTextExtractor(object):
         #     logger.info("20200815 store_content sig is {}".format(sig))
         # else:
         #     logger.info("20200815 store_content sig['names'] is {}".format(sig['names']))
-        for name in sig['ids']:
+        for name in sig["ids"]:
             # if name == "lino.core.model.Model":
             #     raise Exception(name)
             self.sig2dict(name, content)
@@ -254,7 +253,7 @@ class HelpTextExtractor(object):
 
 def setup(app):
     hte = HelpTextExtractor()
-    app.add_config_value('help_texts_builder_targets', {}, 'env')
-    app.connect('builder-inited', hte.initialize)
-    app.connect('doctree-read', hte.extract_help_texts)
-    app.connect('build-finished', hte.write_help_texts_files)
+    app.add_config_value("help_texts_builder_targets", {}, "env")
+    app.connect("builder-inited", hte.initialize)
+    app.connect("doctree-read", hte.extract_help_texts)
+    app.connect("build-finished", hte.write_help_texts_files)
