@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2023 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """:mod:`lino.utils` (the top-level module) contains a few often-used
 function for general use. It has also many subpackages and submodules.
 
@@ -15,14 +14,17 @@ function for general use. It has also many subpackages and submodules.
     config
     cycler
     dates
-    demonames
     daemoncommand
     dataserializer
     dbfreader
     dblogger
+    dbhash
     diag
     djangotest
     dpy
+    fieldutils
+    format_date
+    html
     html2odf
     html2xhtml
     mytidylib
@@ -41,7 +43,6 @@ function for general use. It has also many subpackages and submodules.
     pyuca
     quantities
     ranges
-    requests
     restify
     screenshots
     sqllog
@@ -62,22 +63,24 @@ import traceback
 from dateutil.relativedelta import relativedelta
 import re
 from decimal import Decimal
-from collections import OrderedDict
 from urllib.parse import urlencode
+
 # import locale
 import dateparser
 from io import StringIO
-import contextlib
-from html2text import HTML2Text
+from contextlib import redirect_stdout, contextmanager
+from pathlib import Path
 
-# import rstgen
 from etgen.utils import join_elems
 
 from lino.utils.cycler import Cycler
 from lino.utils.code import codefiles, codetime
+from .sums import SumCollector
 
 from rstgen.utils import confirm, i2d, i2t
 
+DATE_TO_DIR_TPL = "%Y/%m"
+
 
 def read_exception(excinfo):
     # TODO: why not use traceback.format_exc(excinfo) intead of this?
@@ -97,24 +100,40 @@ def buildurl(root, *args, **kw):
         url += "?" + urlencode(kw, True)
     return url
 
-class AttrDict(dict):
 
+def needs_update(src, dest):
+    if dest.exists() and dest.stat().st_mtime >= src.stat().st_mtime:
+        return False
+    return True
+
+
+class AttrDict(dict):
     """
     Dictionary-like helper object.
 
     See usage example in :doc:`/topics/utils`
 
     """
+
     def __getattr__(self, name):
         try:
             return self[name]
         except KeyError:
             raise AttributeError(
-                "AttrDict instance has no key '%s' (keys are %s)" % (
-                    name, ', '.join(list(self.keys()))))
+                "AttrDict instance has no key '%s' (keys are %s)"
+                % (name, ", ".join(list(self.keys())))
+            )
+
+    def __setattr__(self, name, value):
+        if not name in self:
+            raise AttributeError(
+                "AttrDict instance has no key '%s' (keys are %s)"
+                % (name, ", ".join(list(self.keys())))
+            )
+        self[name] = value
 
     def define2(self, name, value):
-        return self.define(*name.split('.') + [value])
+        return self.define(*name.split(".") + [value])
 
     def define(self, *args):
         "args must be a series of names followed by the value"
@@ -135,7 +154,7 @@ class AttrDict(dict):
         return an attribute with dotted name
         """
         o = self
-        for part in name.split('.'):
+        for part in name.split("."):
             o = getattr(o, part, default)
             # o = o.__getattr__(part)
         return o
@@ -197,6 +216,7 @@ def is_string(s):
     #     return isinstance(s, six.string_types) or isinstance(s, newstr)
     return isinstance(s, str)
 
+
 def isidentifier(s):
     """
     Check whether the given string can be used as a Python identifier.
@@ -240,6 +260,7 @@ def isidentifier(s):
 #     except UnicodeDecodeError as e:
 #         raise Exception("%r is not pure : %s" % (s, e))
 
+
 def join_words(*words):
     """
     Remove any empty item (None or ''), call unicode on each and
@@ -250,7 +271,7 @@ def join_words(*words):
     See examples in :doc:`/topics/utils`.
 
     """
-    return ' '.join([str(x) for x in words if x])
+    return " ".join([str(x) for x in words if x])
 
 
 def d2iso(d):
@@ -273,7 +294,7 @@ def call_optional_super(cls, self, name, *args, **kw):
     Doesn't work. See `20110914`.
     """
     s = super(cls, self)
-    m = getattr(s, 'name', None)
+    m = getattr(s, "name", None)
     if m is not None:
         return m(*args, **kw)
 
@@ -294,18 +315,17 @@ def call_on_bases(cls, name, *args, **kw):
         # which has already been called.
         if m is not None and m.__self__.__class__ is cls:
             m(cls, *args, **kw)
-
     """Note: the following algorithm worked in Python 2.7 but not in 2.6,
     a classmethod object in 2.6 has no attribute `im_func`
     """
 
-    #~ m = cls.__dict__.get(name)
-    #~ if m:
-        #~ func = getattr(m,'im_func',None)
-        #~ if func is None:
-            #~ raise Exception("Oops, %r in %s (%r) has no im_func" % (name,cls,m))
-        #~ func(cls,*args,**kw)
-        # ~ # m.__func__(cls,*args,**kw)
+    # ~ m = cls.__dict__.get(name)
+    # ~ if m:
+    # ~ func = getattr(m,'im_func',None)
+    # ~ if func is None:
+    # ~ raise Exception("Oops, %r in %s (%r) has no im_func" % (name,cls,m))
+    # ~ func(cls,*args,**kw)
+    # ~ # m.__func__(cls,*args,**kw)
 
 
 def str2hex(s):
@@ -316,7 +336,7 @@ def str2hex(s):
 
 
     """
-    r = ''
+    r = ""
     for c in s:
         r += hex(ord(c))[2:]
     return r
@@ -335,21 +355,28 @@ def hex2str(value):
     """
     if len(value) % 2 != 0:
         raise Exception("hex2str got value %r" % value)
-    r = ''
+    r = ""
     for i in range(old_div(len(value), 2)):
-        s = value[i * 2:i * 2 + 2]
+        s = value[i * 2: i * 2 + 2]
         h = int(s, 16)
         r += chr(h)
     return r
 
+
 # http://snippets.dzone.com/posts/show/2375
-curry = lambda func, *args, **kw:\
-    lambda *p, **n:\
-    func(*args + p, **dict(list(kw.items()) + list(n.items())))
+curry = lambda func, *args, **kw: lambda *p, **n: func(
+    *args + p, **dict(list(kw.items()) + list(n.items()))
+)
+
+
+def capture_output(func, *args, **kwargs):
+    s = StringIO()
+    with redirect_stdout(s):
+        func(*args, **kwargs)
+    return s.getvalue()
 
 
 class IncompleteDate(object):
-
     """Naive representation of a potentially incomplete gregorian date.
 
     Where *incomplete* means that maybe only the year is known and day and month
@@ -372,22 +399,20 @@ class IncompleteDate(object):
 
         """
 
-        if s.startswith('-'):
+        if s.startswith("-"):
             bc = True
             s = s[1:]
         else:
             bc = False
         try:
-            y, m, d = list(map(int, s.split('-')))
+            y, m, d = list(map(int, s.split("-")))
         except ValueError:
             # the following is useless because dateparser automatically understands many languages.
             # from django.conf import settings
             # lng = [li.django_code for li in settings.SITE.languages]
-            pd = dateparser.parse(
-                s, settings={'STRICT_PARSING': True})
+            pd = dateparser.parse(s, settings={"STRICT_PARSING": True})
             if pd is None:
-                pd = dateparser.parse(
-                    s, settings={'PREFER_DAY_OF_MONTH': 'first'})
+                pd = dateparser.parse(s, settings={"PREFER_DAY_OF_MONTH": "first"})
                 if pd is None:
                     return None
                 y, m, d = pd.year, pd.month, 0
@@ -395,7 +420,7 @@ class IncompleteDate(object):
                 y, m, d = pd.year, pd.month, pd.day
             # raise Exception("Invalid date value {}".format(s))
         if bc:
-            y = - y
+            y = -y
         return cls(y, m, d)
 
     @classmethod
@@ -423,9 +448,9 @@ class IncompleteDate(object):
         return self.strftime()
 
     def strftime(self, fmt="%Y-%m-%d"):
-        #~ s = fmt.replace("%Y",iif(self.bc,'-','')+str(self.year))
+        # ~ s = fmt.replace("%Y",iif(self.bc,'-','')+str(self.year))
         if self.year == 0:
-            s = fmt.replace("%Y", '0000')
+            s = fmt.replace("%Y", "0000")
         else:  # year might be negative
             s = fmt.replace("%Y", str(self.year))
         s = s.replace("%m", "%02d" % self.month)
@@ -433,10 +458,7 @@ class IncompleteDate(object):
         return s
 
     def as_date(self):
-        return datetime.date(
-            self.year or 1900,
-            self.month or 6,
-            self.day or 15)
+        return datetime.date(self.year or 1900, self.month or 6, self.day or 15)
 
     def get_age(self, today):
         "Return age in years as integer."
@@ -447,15 +469,15 @@ class IncompleteDate(object):
         return today.year - self.year
 
 
-#~ class Warning(Exception):
-    #~ """
-    #~ An Exception whose message is meant to be
-    #~ understandable by the user.
-    #~ """
+# ~ class Warning(Exception):
+# ~ """
+# ~ An Exception whose message is meant to be
+# ~ understandable by the user.
+# ~ """
+
 
 # unmodified copy from http://docs.python.org/library/decimal.html#recipes
-def moneyfmt(value, places=2, curr='', sep=',', dp='.',
-             pos='', neg='-', trailneg=''):
+def moneyfmt(value, places=2, curr="", sep=",", dp=".", pos="", neg="-", trailneg=""):
     """
     Convert Decimal to a money formatted string.
 
@@ -471,7 +493,7 @@ def moneyfmt(value, places=2, curr='', sep=',', dp='.',
     See examples in :doc:`/topics/formatting`.
 
     """
-    q = Decimal(10) ** -places      # 2 places --> '0.01'
+    q = Decimal(10) ** -places  # 2 places --> '0.01'
     sign, digits, exp = value.quantize(q).as_tuple()
     result = []
     digits = list(map(str, digits))
@@ -479,10 +501,10 @@ def moneyfmt(value, places=2, curr='', sep=',', dp='.',
     if sign:
         build(trailneg)
     for i in range(places):
-        build(next() if digits else '0')
+        build(next() if digits else "0")
     build(dp)
     if not digits:
-        build('0')
+        build("0")
     i = 0
     while digits:
         build(next())
@@ -492,7 +514,7 @@ def moneyfmt(value, places=2, curr='', sep=',', dp='.',
             build(sep)
     build(curr)
     build(neg if sign else pos)
-    return ''.join(reversed(result))
+    return "".join(reversed(result))
 
 
 def unicode_string(x):
@@ -503,9 +525,9 @@ def unicode_string(x):
     Because the message of an Exception may not be a unicode string.
 
     """
-    return str(x).encode(sys.getdefaultencoding(), 'backslashreplace')
+    return str(x).encode(sys.getdefaultencoding(), "backslashreplace")
     # Python 2.6.6 said "Error in formatting: encode() takes no keyword arguments"
-    #~ return unicode(x).encode(errors='backslashreplace')
+    # ~ return unicode(x).encode(errors='backslashreplace')
 
 
 ONE_DAY = datetime.timedelta(days=1)
@@ -522,9 +544,9 @@ def workdays(start, end):
     See examples in: :doc:`/topics/datetime`.
 
     """
-    #~ for d in range(start,end,ONE_DAY):
-        #~ if d.isoweekday() <= 5:
-            #~ n += 1
+    # ~ for d in range(start,end,ONE_DAY):
+    # ~ if d.isoweekday() <= 5:
+    # ~ n += 1
     n = 0
     d = start
     while d <= end:
@@ -542,14 +564,16 @@ def camelize(s):
     Usage examples in :doc:`/topics/formatting`.
 
     """
+
     def f(k):
         if k.upper() != k:
             return k
         return k[0].upper() + k[1:].lower()
-    return ' '.join([f(k) for k in s.split()])
+
+    return " ".join([f(k) for k in s.split()])
 
 
-UNCAMEL_RE = re.compile('((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))')
+UNCAMEL_RE = re.compile("((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))")
 
 
 def uncamel(s):
@@ -561,7 +585,7 @@ def uncamel(s):
     Usage examples in :doc:`/topics/formatting`.
 
     """
-    return UNCAMEL_RE.sub(r'_\1', s).lower()
+    return UNCAMEL_RE.sub(r"_\1", s).lower()
 
 
 def puts(s):
@@ -585,48 +609,6 @@ def puts(s):
     print(s)
 
 
-class SumCollector(object):
-    """A dictionary of sums to be collected using an arbitrary key.
-
-    This is also included in the default context used by the Jinja
-    renderer (:mod:`lino.modlib.jinja`) when rendering templates,
-    which makes it a more complete solution for a problem asked also
-    elsewhere, e.g. on `Stackoverflow
-    <https://stackoverflow.com/questions/7537439/how-to-increment-a-variable-on-a-for-loop-in-jinja-template>`__.
-
-    See examples in :doc:`/topics/utils`.
-
-    """
-    def __init__(self):
-        self._sums = OrderedDict()
-
-    def collect(self, k, value):
-        """Add the given value to the sum at the given key k."""
-        if value is None:
-            return
-        if k in self._sums:
-            # print("20230614 a", k, "=", self._sums[k], "+", value)
-            self._sums[k] += value
-        else:
-            # print("20230614 b", k, "=", value)
-            self._sums[k] = value
-
-    def __getattr__(self, k):
-        return self._sums.get(k)
-
-    def __getitem__(self, k):
-        return self._sums.get(k)
-
-    def items(self, *args, **kwargs):
-        return self._sums.items(*args, **kwargs)
-
-    def __str__(self):
-        return str(self._sums)
-
-    def __repr__(self):
-        return repr(self._sums)
-
-
 class SimpleSingleton(object):
     _instance = None
 
@@ -638,11 +620,11 @@ class SimpleSingleton(object):
 
 def get_client_ip_address(request):
     # from https://stackoverflow.com/questions/4581789/how-do-i-get-user-ip-address-in-django
-    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
+    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
     if x_forwarded_for:
-        ip = x_forwarded_for.split(',')[0]
+        ip = x_forwarded_for.split(",")[0]
     else:
-        ip = request.META.get('REMOTE_ADDR')
+        ip = request.META.get("REMOTE_ADDR")
     return ip or "(unknown)"  # see ticket #2605
 
 
@@ -655,6 +637,7 @@ class MissingRow:
     not be found.
 
     """
+
     def __init__(self, msg):
         self.message = msg
 
@@ -665,7 +648,7 @@ class MissingRow:
         return "MissingRow({!r})".format(self.message)
 
 
-@contextlib.contextmanager
+@contextmanager
 def logging_disabled(level):
     try:
         logging.disable(level)
@@ -674,9 +657,21 @@ def logging_disabled(level):
         logging.disable(logging.NOTSET)
 
 
-def html2text(html, **kwargs):
-    text_maker = HTML2Text()
-    text_maker.unicode_snob = True
-    for k, v in kwargs.items():
-        setattr(text_maker, k, v)
-    return text_maker.handle(html)
+def nextref(ref):
+    """
+    Increment the first number found in the string, preserving everything
+    thereafter.
+
+    Tested examples in :doc:`/topics/utils`.
+    """
+    num = ""
+    suffix = ""
+    for i, c in enumerate(ref):
+        if c.isdigit():
+            num += c
+        else:
+            suffix = ref[i:]
+            break
+    if not num:
+        return None
+    return str(int(num)+1) + suffix
