@@ -1,16 +1,15 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2021 Rumma & Ko Ltd
+# Copyright 2011-2024 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
+"""This middleware was automatically being installed on every Lino site until
+20240921. No longer used since then. See :ticket:`5755` (Should we remove
+AjaxExceptionResponse?)
 
-"""This middleware is automatically being installed on every Lino
-site.
+The idea was that when an exception occurs during an AJAX call, Lino should not
+respond with Django's default HTML formatted error report but with a plain-text
+traceback because that's more readable when seen in a browser console.
 
-When an exception occurs during an AJAX call, Lino should not respond
-with Django's default HTML formatted error report but with a
-plain-text traceback because that's more readable when seen in a
-browser console.
-
-Originally inspired by http://djangosnippets.org/snippets/650
+Originally inspired by https://djangosnippets.org/snippets/650
 
 Additions by LS:
 
@@ -27,14 +26,13 @@ Additions by LS:
 
 """
 
-from __future__ import unicode_literals
-from builtins import object
-
 import sys
 import traceback
 from django.conf import settings
+
 # from django.http import HttpResponseServerError
 from django.http import HttpResponse
+
 # from django.http import HttpResponseForbidden, HttpResponseBadRequest
 from django.utils.encoding import smart_str
 from django.core.exceptions import PermissionDenied, ObjectDoesNotExist
@@ -42,6 +40,7 @@ from lino.core.utils import format_request
 
 from django.utils.deprecation import MiddlewareMixin
 
+
 class AjaxExceptionResponse(MiddlewareMixin):
     """The middleware class definition."""
 
@@ -52,27 +51,28 @@ class AjaxExceptionResponse(MiddlewareMixin):
     # it can be helpful to temporarily disable filtering of ObjectDoesNotExist
     # exceptions on a production site in order to debug problems like #2699
 
+    # 20240920 I had a sporadic ObjectDoesNotExist exception on a production
+    # server that took me a while to understand because there was no log message
+    # at all.
+
     def process_exception(self, request, exception):
-        # if request.is_ajax():  # See https://docs.djangoproject.com/en/5.0/releases/3.1/
-        if request.headers.get('x-requested-with') == 'XMLHttpRequest':
+        # if request.is_ajax():  # See https://docs.djangoproject.com/en/5.2/releases/3.1/
+        if request.headers.get("x-requested-with") == "XMLHttpRequest":
             (exc_type, exc_info, tb) = sys.exc_info()
-
             # response to client:
-            response = "%s: " % exc_type.__name__
-            response += "%s" % exc_info
-
-            if not isinstance(exception, self.no_traceback):
-                # message to be logged:
-                msg = "AjaxExceptionResponse {0}".format(response)
-                msg += "\nin request {0}\n".format(format_request(request))
-                if settings.DEBUG:
-                    msg += "TRACEBACK:\n"
-                    for tb in traceback.format_tb(tb):
-                        msg += smart_str(tb)
-                    settings.SITE.logger.warning(msg)
-                else:
-                    settings.SITE.logger.exception(msg)
-
+            response = "AjaxExceptionResponse {}: {}".format(
+                exc_type.__name__, exc_info)
+            # message to be logged:
+            msg = response + "\nin request {0}\n".format(format_request(request))
+            if isinstance(exception, self.no_traceback):
+                settings.SITE.logger.warning(msg)
+            elif settings.DEBUG:
+                msg += "TRACEBACK:\n"
+                for tb in traceback.format_tb(tb):
+                    msg += smart_str(tb)
+                settings.SITE.logger.warning(msg)
+            else:
+                settings.SITE.logger.exception(msg)
             return HttpResponse(response, status=400)
             # if isinstance(exception, ObjectDoesNotExist):
             #     return HttpResponseBadRequest(response)
