@@ -1,6 +1,5 @@
-# Copyright 2009-2021 Rumma & Ko Ltd
+# Copyright 2009-2025 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Extends the possibilities for defining choices for fields of a
 Django model.
 
@@ -14,36 +13,35 @@ Example values in :doc:`/topics/utils`.
 
 """
 
+import re
 import decimal
 import datetime
+
 from dateutil import parser as dateparser
-
-import re
-GFK_HACK = re.compile(r'^<a href="javascript:Lino\.(\w+\.\w+)\.detail\.run\(.*,\{ &quot;record_id&quot;: (\w+) \}\)">.*</a>$')
-
-from django.db import models
-from django.conf import settings
 from django.core.exceptions import BadRequest
+from django.conf import settings
+from django.db import models
+from django.apps import apps
+from lino import logger
+from lino.core.utils import getrqdata
+from lino.core import fields
+from lino.core import constants
+
 try:
     from django.contrib.contenttypes.models import ContentType
 except RuntimeError:
     pass
     # Happens when the site doesn't use contenttypes. Ignore silently.
 
-# from lino.api import rt
-from lino.core import constants
-from lino.core import fields
-from lino.core.utils import getrqdata
-
-
-
+GFK_HACK = re.compile(
+    r'^<a href="javascript:Lino\.(\w+\.\w+)\.detail\.run\(.*,\{ &quot;record_id&quot;: (\w+) \}\)">.*</a>$'
+)
 
 # class DataError(Exception):
 #     pass
 
 
 class Converter(object):
-
     def __init__(self, field):
         self.field = field
 
@@ -52,7 +50,6 @@ class Converter(object):
 
 
 class LookupConverter(Converter):
-
     """
     A Converter for ForeignKey and ManyToManyField.
     If the lookup_field is a BabelField, then it tries all available languages.
@@ -61,7 +58,7 @@ class LookupConverter(Converter):
     def __init__(self, field, lookup_field):
         Converter.__init__(self, field)
         model = field.remote_field.model
-        if lookup_field == 'pk':
+        if lookup_field == "pk":
             self.lookup_field = model._meta.pk
         else:
             self.lookup_field = model._meta.get_field(lookup_field)
@@ -71,23 +68,23 @@ class LookupConverter(Converter):
         model = self.field.remote_field.model
         if isinstance(value, model):
             return value
+        # raise Exception("20240220 {} is not a {}".format(value, model))
         return model.lookup_or_create(self.lookup_field, value, **kw)
 
         # if isinstance(self.lookup_field,babel.BabelCharField):
-            # flt  = babel.lookup_filter(self.lookup_field.name,value,**kw)
+        # flt  = babel.lookup_filter(self.lookup_field.name,value,**kw)
         # else:
-            # kw[self.lookup_field.name] = value
-            # flt = models.Q(**kw)
+        # kw[self.lookup_field.name] = value
+        # flt = models.Q(**kw)
         # try:
-            # return model.objects.get(flt)
+        # return model.objects.get(flt)
         # except MultipleObjectsReturned,e:
-            # raise model.MultipleObjectsReturned("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
+        # raise model.MultipleObjectsReturned("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
         # except model.DoesNotExist,e:
-            # raise model.DoesNotExist("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
+        # raise model.DoesNotExist("%s.objects lookup(%r) : %s" % (model.__name__,value,e))
 
 
 class DateConverter(Converter):
-
     def convert(self, **kw):
         value = kw.get(self.field.name)
         if not isinstance(value, datetime.date):
@@ -101,7 +98,6 @@ class DateConverter(Converter):
 
 
 class ChoiceConverter(Converter):
-
     """Converter for :class:`ChoiceListField
     <lino.core.choicelists.ChoiceListField>`.
 
@@ -111,6 +107,7 @@ class ChoiceConverter(Converter):
 
     def convert(self, **kw):
         value = kw.get(self.field.name)
+        # print(f"20241203 convert {self.field.name}")
 
         if value is not None:
             if not isinstance(value, self.field.choicelist.item_class):
@@ -123,7 +120,6 @@ class ChoiceConverter(Converter):
 
 
 class DecimalConverter(Converter):
-
     def convert(self, **kw):
         value = kw.get(self.field.name)
         if value is not None:
@@ -133,13 +129,12 @@ class DecimalConverter(Converter):
 
 
 class ForeignKeyConverter(LookupConverter):
-
     """Converter for ForeignKey fields."""
 
     def convert(self, **kw):
         value = kw.get(self.field.name)
         if value is not None:
-            if value == '':
+            if value == "":
                 value = None
             else:
                 value = self.lookup(value)
@@ -149,13 +144,12 @@ class ForeignKeyConverter(LookupConverter):
 
 
 class GenericForeignKeyConverter(Converter):
-
     """Converter for GenericForeignKey fields."""
 
     def convert(self, **kw):
         value = kw.get(self.field.name)
         if value is not None:
-            if value == '':
+            if value == "":
                 value = None
             else:
                 mo = GFK_HACK.match(value)
@@ -172,38 +166,46 @@ class GenericForeignKeyConverter(Converter):
 
 
 class ManyToManyConverter(LookupConverter):
-
     """Converter for ManyToMany fields."""
+
     splitsep = None
 
     # def lookup(self,value):
-        # model = self.field.remote_field.model
-        # try:
-            # return model.objects.get(
-              # **{self.lookup_field: value})
-        # except model.DoesNotExist,e:
-            # raise DataError("%s.objects.get(%r) : %s" % (
-              # model.__name__,value,e))
+    # model = self.field.remote_field.model
+    # try:
+    # return model.objects.get(
+    # **{self.lookup_field: value})
+    # except model.DoesNotExist,e:
+    # raise DataError("%s.objects.get(%r) : %s" % (
+    # model.__name__,value,e))
 
     def convert(self, **kw):
         values = kw.get(self.field.name)
         if values is not None:
             del kw[self.field.name]
-            l = [self.lookup(value)
-                 for value in values.split(self.splitsep)]
-            kw['_m2m'][self.field.name] = l
+            l = [self.lookup(value) for value in values.split(self.splitsep)]
+            kw["_m2m"][self.field.name] = l
         return kw
 
 
 def make_converter(f, lookup_fields={}):
     from lino.core.gfks import GenericForeignKey
+    from lino.core.fields import VirtualField
 
+    # if f.name == 'rating_type':
+    #     print(f"20241203 {f.__class__}")
+
+    # selector = f
+
+    if isinstance(f, VirtualField):
+        # print(f"20241203 {f.name} {f.return_type.name}")
+        f = f.return_type
     if isinstance(f, models.ForeignKey):
         return ForeignKeyConverter(f, lookup_fields.get(f.name, "pk"))
     if isinstance(f, GenericForeignKey):
         return GenericForeignKeyConverter(f)
     # if isinstance(f,fields.LinkedForeignKey):
-        # return LinkedForeignKeyConverter(f,lookup_fields.get(f.name,"pk"))
+    # return LinkedForeignKeyConverter(f,lookup_fields.get(f.name,"pk"))
     if isinstance(f, models.ManyToManyField):
         return ManyToManyConverter(f, lookup_fields.get(f.name, "pk"))
     if isinstance(f, models.DateField):
@@ -211,36 +213,31 @@ def make_converter(f, lookup_fields={}):
     if isinstance(f, models.DecimalField):
         return DecimalConverter(f)
     from lino.core import choicelists
+
     if isinstance(f, choicelists.ChoiceListField):
-        # if f.name == 'p_book':
-            # print "20131012 b", f
+        # print(f"20241203c {f.__class__}")
         return ChoiceConverter(f)
 
 
-
-
 class BaseChooser(object):
     pass
 
 
 class FieldChooser(BaseChooser):
-
     def __init__(self, field):
         self.field = field
 
 
 class ChoicesChooser(FieldChooser):
-
     def __init__(self, field):
         FieldChooser.__init__(self, field)
         self.simple_values = type(field.choices[0])
 
 
 class Chooser(FieldChooser):
-    """Holds information about the possible choices of a field.
+    """Holds information about the possible choices of a field."""
 
-    """
-    #~ stored_name = None
+    # ~ stored_name = None
     simple_values = False
     instance_values = True
     force_selection = True
@@ -250,30 +247,46 @@ class Chooser(FieldChooser):
     def __init__(self, model, field, meth):
         FieldChooser.__init__(self, field)
         self.model = model
-        #~ self.field = model._meta.get_field(fldname)
+        # ~ self.field = model._meta.get_field(fldname)
         self.meth = meth
         from lino.core.gfks import is_foreignkey
         from lino.core.choicelists import ChoiceListField
-        if isinstance(field, ChoiceListField):
-            self.simple_values = getattr(meth, 'simple_values', False)
-            self.instance_values = getattr(meth, 'instance_values', True)
+
+        selector = field
+        if isinstance(field, fields.VirtualField):
+            selector = field.return_type
+
+        # TODO: move this to top and fix ImportError cannot import name
+        # 'chooser' from partially initialized module 'lino.utils.choosers'
+        # (most likely due to a circular import)
+        from lino.modlib.gfks.fields import GenericForeignKeyIdField
+
+        if isinstance(selector, ChoiceListField):
+            self.simple_values = getattr(meth, "simple_values", False)
+            self.instance_values = getattr(meth, "instance_values", True)
             self.force_selection = getattr(
-                meth, 'force_selection', self.force_selection)
-        elif is_foreignkey(field):
+                meth, "force_selection", self.force_selection
+            )
+        elif is_foreignkey(selector):
             pass
-        elif isinstance(field, fields.VirtualField) and isinstance(field.return_type, models.ForeignKey):
+        elif isinstance(selector, GenericForeignKeyIdField):
             pass
+        # elif isinstance(field, fields.VirtualField) and isinstance(
+        #     field.return_type, models.ForeignKey
+        # ):
+        #     pass
         else:
-            self.simple_values = getattr(meth, 'simple_values', False)
-            self.instance_values = getattr(meth, 'instance_values', False)
+            self.simple_values = getattr(meth, "simple_values", False)
+            self.instance_values = getattr(meth, "instance_values", False)
             self.force_selection = getattr(
-                meth, 'force_selection', self.force_selection)
-        #~ self.context_params = meth.func_code.co_varnames[1:meth.func_code.co_argcount]
+                meth, "force_selection", self.force_selection
+            )
+        # ~ self.context_params = meth.func_code.co_varnames[1:meth.func_code.co_argcount]
         self.context_params = meth.context_params
-        #~ self.multiple = meth.multiple
-        #~ self.context_params = meth.func_code.co_varnames[:meth.func_code.co_argcount]
-        #~ print '20100724', meth, self.context_params
-        #~ logger.warning("20100527 %s %s",self.context_params,meth)
+        # ~ self.multiple = meth.multiple
+        # ~ self.context_params = meth.func_code.co_varnames[:meth.func_code.co_argcount]
+        # ~ print '20100724', meth, self.context_params
+        # ~ logger.warning("20100527 %s %s",self.context_params,meth)
         self.context_values = []
         self.context_fields = []
         for name in self.context_params:
@@ -284,24 +297,24 @@ class Chooser(FieldChooser):
             if f is None:
                 raise Exception(
                     "No data element '%s' in %s "
-                    "(method %s_choices)" % (
-                        name, self.model, field.name))
-            #~ if name == 'p_book':
-                #~ print 20131012, f
+                    "(method %s_choices)" % (name, self.model, field.name)
+                )
+            # ~ if name == 'p_book':
+            # ~ print 20131012, f
             self.context_fields.append(f)
             self.context_values.append(name + "Hidden")
-            #~ if isinstance(f,models.ForeignKey):
-                #~ self.context_values.append(name+"Hidden")
-            #~ else:
-                #~ self.context_values.append(name)
+            # ~ if isinstance(f,models.ForeignKey):
+            # ~ self.context_values.append(name+"Hidden")
+            # ~ else:
+            # ~ self.context_values.append(name)
         self.converters = []
-        #~ try:
+        # ~ try:
         for f in self.context_fields:
             cv = make_converter(f)
             if cv is not None:
                 self.converters.append(cv)
-        #~ except FieldDoesNotExist as e:
-            #~ print e
+        # ~ except FieldDoesNotExist as e:
+        # ~ print e
 
         if hasattr(model, "create_%s_choice" % field.name):
             # There is a create_FOO_choice method
@@ -313,8 +326,10 @@ class Chooser(FieldChooser):
 
     def __str__(self):
         return "Chooser(%s.%s,%s)" % (
-            self.model.__name__, self.field.name,
-            self.context_params)
+            self.model.__name__,
+            self.field.name,
+            self.context_params,
+        )
 
     def create_choice(self, obj, text, ar=None):
         # Call the create_FOO_choice method
@@ -361,31 +376,39 @@ class Chooser(FieldChooser):
         # 20120202
         if tbl.master_field is not None:
             rqdata = getrqdata(ar.request)
-            if tbl.master is not None:
-                master = tbl.master
-            else:
-                mt = rqdata.get(constants.URL_PARAM_MASTER_TYPE)
+            # if tbl.master is not None:
+            #     master = tbl.master
+            # else:
+            #     mt = rqdata.get(constants.URL_PARAM_MASTER_TYPE)
+            master = tbl.master
+            if (mt := rqdata.get(constants.URL_PARAM_MASTER_TYPE)) is not None:
                 try:
                     master = ContentType.objects.get(pk=mt).model_class()
                 except ContentType.DoesNotExist:
                     # master = None
-                    raise BadRequest("Invalid content type {}={}".format(constants.URL_PARAM_MASTER_TYPE, mt))
+                    raise BadRequest(
+                        "Invalid content type {}={}".format(
+                            constants.URL_PARAM_MASTER_TYPE, mt
+                        )
+                    )
 
             pk = rqdata.get(constants.URL_PARAM_MASTER_PK, None)
             if pk and master:
-                if 'get_master_instance' in tbl.__dict__:
+                if "get_master_instance" in tbl.__dict__:
                     kw[tbl.master_field.name] = tbl.get_master_instance(ar, master, pk)
                 else:
                     try:
                         kw[tbl.master_field.name] = master.objects.get(pk=pk)
                     except ValueError:
                         raise Exception(
-                            "Invalid primary key %r for %s", pk, master.__name__)
+                            "Invalid primary key %r for %s", pk, master.__name__
+                        )
                     except master.DoesNotExist:
-                        raise Exception("There's no %s with primary key %r" %
-                                        (master.__name__, pk))
+                        raise Exception(
+                            "There's no %s with primary key %r" % (master.__name__, pk)
+                        )
 
-        for k, v in list(ar.request.GET.items()):
+        for k, v in ar.request.GET.items():
             kw[str(k)] = v
 
         # logger.info(
@@ -399,39 +422,40 @@ class Chooser(FieldChooser):
             kw.update(tbl.known_values)
 
         if False:  # removed 20120815 #1114
-            #~ ar = tbl.request(ui,request,tbl.default_action)
+            # ~ ar = tbl.request(ui,request,tbl.default_action)
             if ar.create_kw:
                 kw.update(ar.create_kw)
             if ar.known_values:
                 kw.update(ar.known_values)
             if tbl.master_key:
                 kw[tbl.master_key] = ar.master_instance
-            #~ if tbl.known_values:
-                #~ kw.update(tbl.known_values)
+            # ~ if tbl.known_values:
+            # ~ kw.update(tbl.known_values)
         return self.get_choices(**kw)  # 20120918b
 
     def get_text_for_value(self, value, obj):
-        m = getattr(self.field, 'get_text_for_value', None)
+        m = getattr(self.field, "get_text_for_value", None)
         if m is not None:  # e.g. lino.utils.choicelist.ChoiceListField
             return m(value)
-        #~ raise NotImplementedError
-        #~ assert not self.simple_values
+        # ~ raise NotImplementedError
+        # ~ assert not self.simple_values
         m = getattr(obj, "get_" + self.field.name + "_display", str)
-        #~ if m is None:
-            #~ raise Exception("")
+        # ~ if m is None:
+        # ~ raise Exception("")
         return m(value)
-        #~ raise NotImplementedError("%s : Cannot get text for value %r" % (self.meth,value))
+        # ~ raise NotImplementedError("%s : Cannot get text for value %r" % (self.meth,value))
 
 
 def uses_simple_values(holder, fld):
     "used by :class:`lino.core.store`"
     from lino.core.gfks import is_foreignkey
+
     if is_foreignkey(fld):
         return False
     # if isinstance(fld, models.OneToOneRel):
     #     return False
-    if not hasattr(fld, 'name'):
-        raise Exception("DEBUG", holder, fld, fld.display_mode)
+    if not hasattr(fld, "name"):
+        raise Exception("20240925")
     if holder is not None and fld.name is not None:
         ch = holder.get_chooser_for_field(fld.name)
         if ch is not None:
@@ -447,19 +471,21 @@ def uses_simple_values(holder, fld):
 
 
 def _chooser(make, **options):
-    #~ options.setdefault('quick_insert_field',None)
+    # ~ options.setdefault('quick_insert_field',None)
     def chooser_decorator(fn):
         def wrapped(*args):
             return fn(*args)
+
         cp = options.pop(
-            'context_params',
-            fn.__code__.co_varnames[1:fn.__code__.co_argcount])
+            "context_params", fn.__code__.co_varnames[1: fn.__code__.co_argcount]
+        )
         wrapped.context_params = cp
         for k, v in options.items():
             setattr(wrapped, k, v)
         return make(wrapped)
         # return classmethod(wrapped)
         # A chooser on an action must not turn it into a classmethod
+
     return chooser_decorator
 
 
@@ -475,13 +501,15 @@ def noop(x):
 def action_chooser(**options):
     return _chooser(noop, **options)
 
+
 def get_choosers_dict(holder):
-    d = holder.__dict__.get('_choosers_dict', None)
+    d = holder.__dict__.get("_choosers_dict", None)
     if d is None:
         d = dict()
-        setattr(holder, '_choosers_dict', d)
+        setattr(holder, "_choosers_dict", d)
     return d
 
+
 def check_for_chooser(holder, field):
     # holder is either a Model, an Actor or an Action.
     if isinstance(field, fields.DummyField):
@@ -493,18 +521,28 @@ def check_for_chooser(holder, field):
         #     raise Exception("20200425 {} is not {}".format(holder, ch.model))
         return ch
 
+    # print("20250518", holder, ch, d)
+
     methname = field.name + "_choices"
     m = getattr(holder, methname, None)
     if m is not None:
+        # print(f"20250511 chooser for {field.name} in {holder} is {m}")
         # if field.name.endswith('municipality'):
         #     print("20200524 check_for_chooser() found {} on {}", field.name, holder)
         # 20200425 fix theoretical bug
         # if field.name
         # if ch in d:
-            # raise Exception(
-            #     "Duplicate of chooser for {} in {}".format(field, holder))
+        # raise Exception(
+        #     "Duplicate of chooser for {} in {}".format(field, holder))
         ch = Chooser(holder, field, m)
         d[field.name] = ch
         return ch
     # if field.name == 'city':
     #     logger.info("20140822 chooser for %s.%s", holder, field.name)
+
+
+def discover_choosers():
+    logger.debug("Discovering choosers for database fields...")
+    for model in apps.get_models():
+        for field in model._meta.fields:
+            check_for_chooser(model, field)
