@@ -2,7 +2,6 @@
 # Copyright 2012-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 # docs: https://dev.lino-framework.org/topics/utils.html
-
 """
 Defines some utilities to inspect Python code.
 See :doc:`/topics/utils`.
@@ -19,15 +18,14 @@ from pathlib import Path
 import rstgen
 
 
-
-def codefiles_imported(pattern='*'):
-    #~ exp = re.compile(pattern, flags)
+def codefiles_imported(pattern="*"):
+    # ~ exp = re.compile(pattern, flags)
 
     for name, mod in sys.modules.copy().items():
-        #~ if name == 'lino.extjs' and pattern == '*':
-            # ~ logger.info("20130801 %r -> %r", name,mod.__file__)
+        # ~ if name == 'lino.extjs' and pattern == '*':
+        # ~ logger.info("20130801 %r -> %r", name,mod.__file__)
         if fnmatch.fnmatch(name, pattern):
-        #~ if exp.match(name):
+            # ~ if exp.match(name):
             filename = getattr(mod, "__file__", None)
             if filename is not None:
                 if filename.endswith(".pyc") or filename.endswith(".pyo"):
@@ -53,10 +51,9 @@ def codetime(*args, **kw):
     return code_mtime
 
 
-
 # def codefiles(pattern='*'):
 def codefiles(module_name):
-    #~ exp = re.compile(pattern, flags)
+    # ~ exp = re.compile(pattern, flags)
     mod = import_module(module_name)
     filename = getattr(mod, "__file__", None)
     if filename is None:
@@ -76,7 +73,6 @@ def codefiles(module_name):
     for p in recurse(main.parent):
         yield p
 
-
     # if filename.endswith(".pyc") or filename.endswith(".pyo"):
     #     filename = filename[:-1]
     # if filename.endswith("$py.class"):
@@ -117,9 +113,12 @@ def codetime_via_source_files(*args):  # not used
 
 def is_start_of_docstring(line):
     for delim in '"""', "'''":
-        if line.startswith(delim) or line.startswith('u' + delim) \
-           or line.startswith('r' + delim) \
-           or line.startswith('ru' + delim):
+        if (
+            line.startswith(delim)
+            or line.startswith("u" + delim)
+            or line.startswith("r" + delim)
+            or line.startswith("ru" + delim)
+        ):
             return delim
 
 
@@ -139,7 +138,7 @@ class SourceFile(object):
     def analyze(self):
         self.count_code, self.count_total, self.count_blank, self.count_doc = 0, 0, 0, 0
         self.count_comment = 0
-        #~ count_code, count_total, count_blank, count_doc = 0, 0, 0, 0
+        # ~ count_code, count_total, count_blank, count_doc = 0, 0, 0, 0
         skip_until = None
         for line in open(self.filename).readlines():
             self.count_total += 1
@@ -147,45 +146,51 @@ class SourceFile(object):
             if not line:
                 self.count_blank += 1
             else:
-                if line.startswith('#'):
+                if line.startswith("#"):
                     self.count_comment += 1
                     continue
                 if skip_until is None:
                     skip_until = is_start_of_docstring(line)
                     if skip_until is not None:
                         self.count_doc += 1
-                        #~ skip_until = '"""'
+                        # ~ skip_until = '"""'
                         continue
-                    #~ if line.startswith('"""') or line.startswith('u"""'):
-                        #~ count_doc += 1
-                        #~ skip_until = '"""'
-                        #~ continue
-                    #~ if line.startswith("'''") or line.startswith("u'''"):
-                        #~ count_doc += 1
-                        #~ skip_until = "'''"
-                        #~ continue
+                    # ~ if line.startswith('"""') or line.startswith('u"""'):
+                    # ~ count_doc += 1
+                    # ~ skip_until = '"""'
+                    # ~ continue
+                    # ~ if line.startswith("'''") or line.startswith("u'''"):
+                    # ~ count_doc += 1
+                    # ~ skip_until = "'''"
+                    # ~ continue
                     self.count_code += 1
                 else:
                     self.count_doc += 1
-                    #~ if line.startswith(skip_until):
+                    # ~ if line.startswith(skip_until):
                     if skip_until in line:
                         skip_until = None
 
-        #~ self.count_code, count_total, count_blank, count_doc
+        # ~ self.count_code, count_total, count_blank, count_doc
 
 
 def analyze_rst(*packages):
-    fields = 'count_code count_doc count_comment count_total'.split()
-    headers = ["name", "code lines", "doc lines",
-               "comment lines", "total lines", "files"]
+    fields = "count_code count_doc count_comment count_total".split()
+    headers = [
+        "name",
+        "code lines",
+        "doc lines",
+        "comment lines",
+        "total lines",
+        "files",
+    ]
     rows = []
 
     def fmt(n):
         if n < 1000:
             return "{}".format(n)
         elif n < 2000:
-            return "{}k".format(round(n/1000.0, 1))
-        return "{:.0f}k".format(round(n/1000, 0))
+            return "{}k".format(round(n / 1000.0, 1))
+        return "{:.0f}k".format(round(n / 1000, 0))
 
     count_files = len(fields)
     SUMS_LEN = count_files + 1
@@ -201,5 +206,5 @@ def analyze_rst(*packages):
         for i, k in enumerate(fields):
             total_sums[i] += sums[i]
         total_sums[count_files] += sums[count_files]
-    rows.append(['total'] + [fmt(n) for n in total_sums])
+    rows.append(["total"] + [fmt(n) for n in total_sums])
     return rstgen.table(headers, rows)
