@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """This defines the :class:`ConfigDirCache` which Lino instantiates
 and installs as :attr:`SITE.confdirs
 <lino.core.site.Site.confdirs>`.
@@ -31,22 +30,18 @@ fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
 
 from lino.utils import iif
 
-SUBDIR_NAME = 'config'  # we might change this to "templates"
+SUBDIR_NAME = "config"  # we might change this to "templates"
 
 
 class ConfigDir(object):
-
-    """A directory that may contain configuration files.
-
-    """
+    """A directory that may contain configuration files."""
 
     def __init__(self, name, writeable):
         self.name = abspath(name)
         self.writeable = writeable
 
     def __repr__(self):
-        return "ConfigDir %s" % self.name + iif(
-            self.writeable, " (writeable)", "")
+        return "ConfigDir %s" % self.name + iif(self.writeable, " (writeable)", "")
 
 
 class ConfigDirCache(object):
@@ -62,14 +57,16 @@ class ConfigDirCache(object):
     def scan_config_dirs(self):
         """Scan the file system and populate :attr:`config_dirs`.
 
-        Example of a special case is the apc demo project, which has a local
-        config dir.  When the apc tests are run on travis (i.e. LINO_CACHE_ROOT
+        Example of a special case is the cosi1 demo project, which has a local
+        config dir.  When the cosi1 tests are run on travis (i.e. LINO_CACHE_ROOT
         is set), we must not forget to add it (:ticket:`3705`).
 
         """
         config_dirs = []
 
         def add_config_dir(name, mod):
+            if mod.__file__ is None:
+                raise Exception(f"20250206 Module {mod} has no __file__!")
             pth = join(dirname(mod.__file__), SUBDIR_NAME)
             if isdir(pth):
                 # logger.info("add_config_dir %s %s", name, pth)
@@ -121,12 +118,12 @@ class ConfigDirCache(object):
         if os.path.isabs(fn):
             return fn
         if len(groups) == 0:
-            groups = ['']
+            groups = [""]
         for group in groups:
             if group:
-                prefix = join(*(group.split('/')))
+                prefix = join(*(group.split("/")))
             else:
-                prefix = ''
+                prefix = ""
             for cd in self.config_dirs:
                 ffn = join(cd.name, prefix, fn)
                 if os.path.exists(ffn):
@@ -146,9 +143,9 @@ class ConfigDirCache(object):
         files = {}
         for group in groups:
             if group:
-                prefix = os.path.sep + join(*(group.split('/')))
+                prefix = os.path.sep + join(*(group.split("/")))
             else:
-                prefix = ''
+                prefix = ""
             for cd in self.config_dirs:
                 pth = cd.name + prefix
                 if isdir(pth):
@@ -163,14 +160,14 @@ class ConfigDirCache(object):
         e.g. ignore "foo_fr.html" if "foo.html" exists.
         Note: but don't ignore "my_template.html"
         """
-        files = self.find_config_files('*' + template_ext, *groups)
+        files = self.find_config_files("*" + template_ext, *groups)
         l = []
         template_ext
         for name in list(files.keys()):
-            basename = name[:-len(template_ext)]
-            chunks = basename.split('_')
+            basename = name[: -len(template_ext)]
+            chunks = basename.split("_")
             if len(chunks) > 1:
-                basename = '_'.join(chunks[:-1])
+                basename = "_".join(chunks[:-1])
                 if basename + template_ext in files:
                     continue
             l.append(name)
@@ -178,7 +175,9 @@ class ConfigDirCache(object):
         if not l:
             logger.warning(
                 "find_template_config_files() : no matches for (%r, %r)",
-                '*' + template_ext, groups)
+                "*" + template_ext,
+                groups,
+            )
         return l
 
     def load_config_files(self, loader, pattern, *groups):
@@ -197,6 +196,7 @@ class ConfigDirCache(object):
 
         def fcmp(a):
             return a[0][:-4]
+
         files.sort(key=fcmp)
         for group in groups:
             prefix = group.replace("/", os.sep)
@@ -204,7 +204,7 @@ class ConfigDirCache(object):
                 filename = join(prefix, filename)
                 ffn = join(cd.name, filename)
                 logger.debug("Loading %s...", ffn)
-                s = codecs.open(ffn, encoding='utf-8').read()
+                s = codecs.open(ffn, encoding="utf-8").read()
                 loader(s, cd, filename)
 
 
@@ -239,12 +239,12 @@ def make_dummy_messages_file(src_fn, messages):
     translatable messages that getmessages will find.
     """
     raise Exception("Never used")
-    target_fn = src_fn + '.py'
+    target_fn = src_fn + ".py"
     if not must_make(src_fn, target_fn):
         logger.debug("%s is up-to-date.", target_fn)
         return
     try:
-        f = file(target_fn, 'w')
+        f = file(target_fn, "w")
     except IOError as e:
         logger.warning("Could not write file %s : %s", target_fn, e)
         return
