@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2013-2021 by Rumma & Ko Ltd.
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """ Turns a list of items into an endless loop. Useful when generating demo
 fixtures.
 
@@ -38,6 +37,7 @@ class Cycler(object):
         else:
             self.items = args
         self.current = 0
+        self.loop_no = 1
 
     def pop(self):
         if len(self.items) == 0:
@@ -46,20 +46,22 @@ class Cycler(object):
         self.current += 1
         if self.current >= len(self.items):
             self.current = 0
+            self.loop_no += 1
         if isinstance(item, Cycler):
             return item.pop()
         return item
 
-    def __len__(self):
-        return len(self.items)
-
     def reset(self):
         self.current = 0
 
+    def __len__(self):
+        return len(self.items)
 
-def _test():
-    import doctest
-    doctest.testmod()
+    def __iter__(self):
+        return self.items.__iter__()
 
-if __name__ == "__main__":
-    _test()
+    def __repr__(self):
+        return f"Cycler({self.current} of {len(self.items)} in loop {self.loop_no})"
+
+    def __getitem__(self, *args):
+        return self.items.__getitem__(*args)
