@@ -1,7 +1,5 @@
 # Copyright 2009 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
-
 """
 YAML serializer.
 
@@ -26,16 +24,14 @@ from django.core.serializers.python import Deserializer as PythonDeserializer
 
 
 class DjangoSafeDumper(yaml.SafeDumper):
-
     def represent_decimal(self, data):
-        return self.represent_scalar('tag:yaml.org,2002:str', str(data))
+        return self.represent_scalar("tag:yaml.org,2002:str", str(data))
 
-DjangoSafeDumper.add_representer(
-    decimal.Decimal, DjangoSafeDumper.represent_decimal)
+
+DjangoSafeDumper.add_representer(decimal.Decimal, DjangoSafeDumper.represent_decimal)
 
 
 class FakeDeserializedObject(base.DeserializedObject):
-
     """loaddata requires DeserializedObject instances,
     but this Deserializer does *not* bypass pre_save/save methods.
     """
@@ -46,6 +42,7 @@ class FakeDeserializedObject(base.DeserializedObject):
     def save(self, save_m2m=True):
         self.object.save()
 
+
 # my code
 
 from lino.utils.instantiator import Instantiator
@@ -54,30 +51,30 @@ from lino.utils.instantiator import Instantiator
 class Serializer(PythonSerializer):
     internal_use_only = False
 
-    #~ """
-    #~ Convert a queryset to YAML.
-    #~ """
+    # ~ """
+    # ~ Convert a queryset to YAML.
+    # ~ """
 
+    # ~ def handle_field(self, obj, field):
+    # ~ # A nasty special case: base YAML doesn't support serialization of time
+    # ~ # types (as opposed to dates or datetimes, which it does support). Since
+    # ~ # we want to use the "safe" serializer for better interoperability, we
+    # ~ # need to do something with those pesky times. Converting 'em to strings
+    # ~ # isn't perfect, but it's better than a "!!python/time" type which would
+    # ~ # halt deserialization under any other language.
+    # ~ if isinstance(field, models.TimeField) and getattr(obj, field.name) is not None:
+    # ~ self._current[field.name] = str(getattr(obj, field.name))
+    # ~ else:
+    # ~ super(Serializer, self).handle_field(obj, field)
 
-    #~ def handle_field(self, obj, field):
-        # ~ # A nasty special case: base YAML doesn't support serialization of time
-        # ~ # types (as opposed to dates or datetimes, which it does support). Since
-        # ~ # we want to use the "safe" serializer for better interoperability, we
-        # ~ # need to do something with those pesky times. Converting 'em to strings
-        # ~ # isn't perfect, but it's better than a "!!python/time" type which would
-        # ~ # halt deserialization under any other language.
-        #~ if isinstance(field, models.TimeField) and getattr(obj, field.name) is not None:
-            #~ self._current[field.name] = str(getattr(obj, field.name))
-        #~ else:
-            #~ super(Serializer, self).handle_field(obj, field)
+    # ~ def end_serialization(self):
+    # ~ self.options.pop('stream', None)
+    # ~ self.options.pop('fields', None)
+    # ~ yaml.dump(self.objects, self.stream, Dumper=DjangoSafeDumper, **self.options)
 
-    #~ def end_serialization(self):
-        #~ self.options.pop('stream', None)
-        #~ self.options.pop('fields', None)
-        #~ yaml.dump(self.objects, self.stream, Dumper=DjangoSafeDumper, **self.options)
+    # ~ def getvalue(self):
+    # ~ return self.stream.getvalue()
 
-    #~ def getvalue(self):
-        #~ return self.stream.getvalue()
 
 def Deserializer(stream_or_string, **options):
     """
@@ -90,9 +87,9 @@ def Deserializer(stream_or_string, **options):
 
     model_builder = None
     for values in yaml.load_all(stream):
-        if 'model' in values:
-            modelspec = values.pop('model')
-            #model_class = eval(modelspec)
+        if "model" in values:
+            modelspec = values.pop("model")
+            # model_class = eval(modelspec)
             app, model = modelspec.split(".")
             # print app,model
             model_class = models.get_model(app, model)
@@ -104,17 +101,16 @@ def Deserializer(stream_or_string, **options):
         # print model_class
         instance = model_builder.build(**values)
         # yield instance
-        #~ if model_class == User:
-            #~ instance.set_password(yamldict.get('password'))
+        # ~ if model_class == User:
+        # ~ instance.set_password(yamldict.get('password'))
         # data files are required to use "!!python/object:", so the
         # yamldict is a Python object
         # self.add_node(yamldict)
         # print instance.pk, instance
-        #~ instance.save()
-        #~ m2m_data = {}
+        # ~ instance.save()
+        # ~ m2m_data = {}
         yield FakeDeserializedObject(instance)
 
-
-        #~ instance.save()
-        #~ print "Saved:", instance
-        #self.modelspec = modelspec
+        # ~ instance.save()
+        # ~ print "Saved:", instance
+        # self.modelspec = modelspec
