@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2011-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """This module contains mainly a utility function :func:`html2odf`
 which converts an ElementTree object generated using
 :mod:`etgen.html` to a fragment of ODF.
@@ -33,7 +32,8 @@ from lino import logger
 
 from io import StringIO
 from lxml import etree
-from etgen.html import E, tostring
+from lino.utils.html import E, tostring
+
 
 def toxml(node):
     """Convert an ODF node to a string with its XML representation."""
@@ -44,9 +44,8 @@ def toxml(node):
 
 from odf import text
 
-
-#~ PTAGS = ('p','td','li')
-PTAGS = ('p', 'td', 'div', 'table', 'tr')
+# ~ PTAGS = ('p','td','li')
+PTAGS = ("p", "td", "div", "table", "tr")
 
 
 def html2odf(e, ct=None, **ctargs):
@@ -59,106 +58,108 @@ def html2odf(e, ct=None, **ctargs):
 
     """
     sections_counter = 1
-    #~ print "20120613 html2odf()", e.tag, e.text
+    # ~ print "20120613 html2odf()", e.tag, e.text
     if ct is None:
         ct = text.P(**ctargs)
-        #~ if e.tag in PTAGS:
-            #~ oe = text.P(**ctargs)
-        #~ else:
-            #~ oe = text.P(**ctargs)
-            #~ logger.info("20130201 %s", tostring(e))
-            #~ raise NotImplementedError("<%s> without container" % e.tag)
+        # ~ if e.tag in PTAGS:
+        # ~ oe = text.P(**ctargs)
+        # ~ else:
+        # ~ oe = text.P(**ctargs)
+        # ~ logger.info("20130201 %s", tostring(e))
+        # ~ raise NotImplementedError("<%s> without container" % e.tag)
     if isinstance(e, str):
         ct.addText(e)
-        #~ oe = text.Span()
-        #~ oe.addText(e)
-        #~ yield oe
+        # ~ oe = text.Span()
+        # ~ oe.addText(e)
+        # ~ yield oe
         return ct
 
-    if e.tag == 'ul':
-        ct = text.List(stylename='podBulletedList')
-        ctargs = dict(stylename='podBulletItem')
-        #~ ctargs = dict()
+    if e.tag == "ul":
+        ct = text.List(stylename="podBulletedList")
+        ctargs = dict(stylename="podBulletItem")
+        # ~ ctargs = dict()
 
     text_container = None
 
-    if e.tag in ('b', 'strong'):
-        #~ oe = text.Span(stylename='Bold Text')
-        oe = text.Span(stylename='Strong Emphasis')
-    elif e.tag == 'a':
-        oe = text.Span(stylename='Strong Emphasis')
-        #~ oe = text.Span(stylename='Bold Text')
-    elif e.tag in ('i', 'em'):
-        oe = text.Span(stylename='Emphasis')
-    elif e.tag == 'span':
+    if e.tag in ("b", "strong"):
+        # ~ oe = text.Span(stylename='Bold Text')
+        oe = text.Span(stylename="Strong Emphasis")
+    elif e.tag == "a":
+        oe = text.Span(stylename="Strong Emphasis")
+        # ~ oe = text.Span(stylename='Bold Text')
+    elif e.tag in ("i", "em"):
+        oe = text.Span(stylename="Emphasis")
+    elif e.tag == "span":
         oe = text.Span()
-    elif e.tag == 'br':
+    elif e.tag == "br":
         oe = text.LineBreak()
 
-    elif e.tag == 'h1':
+    elif e.tag == "h1":
         """
         <text:h text:style-name="Heading_20_1" text:outline-level="1">
         """
         oe = ct = text.H(stylename="Heading 1", outlinelevel=1)
-    elif e.tag == 'h2':
+    elif e.tag == "h2":
         oe = ct = text.H(stylename="Heading 2", outlinelevel=2)
-    elif e.tag == 'h3':
+    elif e.tag == "h3":
         oe = ct = text.H(stylename="Heading 3", outlinelevel=3)
-    elif e.tag == 'div':
+    elif e.tag == "div":
         oe = ct = text.Section(name="S" + str(sections_counter))
 
-    elif e.tag == 'img':
+    elif e.tag == "img":
         return  # ignore images
-    elif e.tag == 'ul':
+    elif e.tag == "ul":
         oe = ct
-    #~ elif e.tag in ('ul','ol'):
-        #~ oe = text.List(stylename=e.tag.upper())
-        #~ ctargs = dict(stylename=e.tag.upper()+"_P")
-    elif e.tag == 'li':
-        #~ oe = ct
+    # ~ elif e.tag in ('ul','ol'):
+    # ~ oe = text.List(stylename=e.tag.upper())
+    # ~ ctargs = dict(stylename=e.tag.upper()+"_P")
+    elif e.tag == "li":
+        # ~ oe = ct
         oe = text.ListItem()
         text_container = text.P(**ctargs)
         oe.appendChild(text_container)
 
     elif e.tag in PTAGS:
         oe = ct
-        #~ if ct.tagName == 'p':
-            #~ oe = ct
-        #~ else:
-            #~ oe = text.P(**ctargs)
+        # ~ if ct.tagName == 'p':
+        # ~ oe = ct
+        # ~ else:
+        # ~ oe = text.P(**ctargs)
     else:
         logger.info("20130201 %s", tostring(e))
         raise NotImplementedError("<%s> inside <%s>" % (e.tag, ct.tagName))
-        #~ oe = text.Span()
+        # ~ oe = text.Span()
 
     if text_container is None:
         text_container = oe
     if e.text:
         text_container.addText(e.text)
     for child in e:
-        #~ html2odf(child,oe)
+        # ~ html2odf(child,oe)
         html2odf(child, text_container, **ctargs)
-        #~ for oc in html2odf(child,oe):
-            # ~ # oe.addElement(oc)
-            #~ oe.appendChild(oc)
-    #~ if not True:
-        #~ if e.tail:
-            #~ oe.addText(e.tail)
+        # ~ for oc in html2odf(child,oe):
+        # ~ # oe.addElement(oc)
+        # ~ oe.appendChild(oc)
+    # ~ if not True:
+    # ~ if e.tail:
+    # ~ oe.addText(e.tail)
     if oe is not ct:
         ct.appendChild(oe)
-        #~ yield oe
-    #~ if True:
+        # ~ yield oe
+    # ~ if True:
     if e.tail:
-        #~ yield e.tail
-        #~ yield text.Span(text=e.tail)
-        #~ yield Text(e.tail)
+        # ~ yield e.tail
+        # ~ yield text.Span(text=e.tail)
+        # ~ yield Text(e.tail)
         ct.addText(e.tail)
     return ct
 
 
 def _test():
     import doctest
+
     doctest.testmod()
 
+
 if __name__ == "__main__":
     _test()
