@@ -1,6 +1,5 @@
 # Copyright 2009-2020 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the :class:`Instantiator` class and some other utilities
 used for generating database objects in :ref:`python fixtures <dpy>`.
 
@@ -9,7 +8,8 @@ used for generating database objects in :ref:`python fixtures <dpy>`.
 from lino.core.utils import resolve_model, UnresolvedModel
 from lino.core.utils import obj2str
 from lino.utils import i2d  # for backward compatibility of .py fixtures
-from lino.core.fields import make_remote_field, RemoteField
+from lino.core.fields import RemoteField
+from lino.core.atomizer import make_remote_field
 from lino.utils.choosers import make_converter
 
 
@@ -27,17 +27,19 @@ class Instantiator(object):
         if isinstance(self.model, UnresolvedModel):
             raise Exception("Instantiator on unresolved model %s", model)
         if not self.is_active():
+
             def noop(*values, **kw):
                 pass
+
             self.build = noop
             return
         # if self.model._meta.pk is None:
-            # raise Exception("Model %r is not installed (_meta.pk is None)." % self.model)
+        # raise Exception("Model %r is not installed (_meta.pk is None)." % self.model)
         # if type(fieldnames) == str:
         if isinstance(fieldnames, str):
             fieldnames = fieldnames.split()
         self.default_values = kw
-        #self.fieldnames = fieldnames
+        # self.fieldnames = fieldnames
         lookup_fields = {}
         self.converters = []
         if fieldnames is None:
@@ -70,10 +72,9 @@ class Instantiator(object):
             if cv is not None:
                 self.converters.append(cv)
         # for f in model_class._meta.many_to_many:
-            # print "foo", f.name
+        # print "foo", f.name
 
     def is_active(self):
-
         if isinstance(self.model, UnresolvedModel):
             return False
         if self.model._meta.pk is None:
@@ -96,7 +97,7 @@ class Instantiator(object):
         """
         # logger.debug("Instantiator.build(%s,%r,%r)",self.model_class._meta.db_table,values,kw)
         # i = 0
-        kw['_m2m'] = {}
+        kw["_m2m"] = {}
         setters = []
         for i, v in enumerate(values):
             fld = self.fields[i]
@@ -121,9 +122,9 @@ class Instantiator(object):
         for c in self.converters:
             kw = c.convert(**kw)
         # if self.model.__name__ == 'Company':
-            # print 20130212, __file__, kw
-            # logger.info("20130212 field_cache for %s (%s)",self.model,
-              # ' '.join([f.name for f in self.model._meta._field_name_cache]))
+        # print 20130212, __file__, kw
+        # logger.info("20130212 field_cache for %s (%s)",self.model,
+        # ' '.join([f.name for f in self.model._meta._field_name_cache]))
 
         m2m = kw.pop("_m2m")
         # print("20191210 {}({})", self.model, kw)
@@ -146,17 +147,20 @@ class InstanceGenerator(object):
     """
     Usage example see :mod:`lino_xl.lib.humanlinks.fixtures`.
     """
+
     def __init__(self):
         self._objects = []
         self._instantiators = dict()
 
     def add_instantiator(self, name, *args, **kw):
         i = Instantiator(*args, **kw)
+
         # self._instantiators[i.model] = i
 
         def f(*args, **kw):
             o = i.build(*args, **kw)
             return self.on_new(o)
+
         setattr(self, name, f)
 
     def on_new(self, o):
@@ -170,15 +174,14 @@ class InstanceGenerator(object):
 
 
 def create_row(model, **kw):
-    """Instantiate, full_clean, save and return a database object.
-
-    """
+    """Instantiate, full_clean, save and return a database object."""
     # model = resolve_model(model)
     o = model(**kw)
     o.full_clean()
     o.save()
     return o
 
+
 def create_or_update_row(model, lookup_values, new_values):
     """
     Update a single existing row that can be found using param: lookup values, with new_values.
@@ -188,7 +191,9 @@ def create_or_update_row(model, lookup_values, new_values):
         o = model.objects.get(**lookup_values)
         for attr, value in new_values.items():
             setattr(o, attr, value)
-    except model.DoesNotExist: # If get receives more than one item will raise MultipleObjectsReturned:
+    except (
+        model.DoesNotExist
+    ):  # If get receives more than one item will raise MultipleObjectsReturned:
         o = model(**new_values)
     o.full_clean()
     o.save()
@@ -197,6 +202,7 @@ def create_or_update_row(model, lookup_values, new_values):
 
 create = create_row  # backwards-compat
 
+
 def create_and_get(model, **kw):
     """Create and then read back from database (the latter to avoid
     certain Django side effects)
@@ -205,9 +211,42 @@ def create_and_get(model, **kw):
     o = create(model, **kw)
     return model.objects.get(pk=o.pk)
 
+
+def make_if_needed(model, **values):
+    qs = model.objects.filter(**values)
+    if qs.count() == 1:
+        return qs.first()
+        # pass  # ok, nothing to do
+    elif qs.count() == 0:
+        return model(**values)
+    else:
+        raise Exception(f"Multiple {model._meta.verbose_name_plural} for {values}")
+
+
+def get_or_create(model, **kwargs):
+    # similar to Djanop's QuerySet.get_or_create() but calls full_clean
+    try:
+        obj = model.objects.get(**kwargs)
+    except model.DoesNotExist:
+        obj = model(**kwargs)
+        obj.full_clean()
+        obj.save()
+    return obj
+
+
+def update_or_create(m, **kwargs):
+    if (obj := m.objects.filter(id=kwargs['id']).first()) is not None:
+        for k, v in kwargs.items():
+            setattr(obj, k, v)
+        return obj
+    return m(**kwargs)
+
+
 def _test():
     import doctest
+
     doctest.testmod()
 
+
 if __name__ == "__main__":
     _test()
