@@ -1,17 +1,17 @@
 # Copyright 2015-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
-'''This defines the :class:`Counter` class, a utility used in Jinja
+"""This defines the :class:`Counter` class, a utility used in Jinja
 templates to generate self-incrementing counters for sections,
 subsections and any other sequences.
 
 Usages and examples can be found in :doc:`/topics/utils`.
 
-'''
+"""
 
 
 class Counter(object):
-    """Represents a counter. Usage see """
+    """Represents a counter. Usage see"""
+
     def __init__(self, parent=None, start=0, step=1):
         self.children = []
         self.start = start
@@ -48,7 +48,9 @@ class Counter(object):
 
 def _test():
     import doctest
+
     doctest.testmod()
 
+
 if __name__ == "__main__":
     _test()
