@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """A framework for generating Javascript from Python.
 
 See also :doc:`/specs/jsgen`.
@@ -26,19 +25,20 @@ from lino.utils import IncompleteDate
 from lino.utils import MissingRow
 from lino.utils.quantities import Quantity
 from lino.utils import curry
+
 # from lino.utils import DelayedValue
 from lino.core.permissions import Permittable
 from lino.core.permissions import make_static_permission_handler
 from lino.modlib.users.utils import get_user_profile
 from lino.modlib.users.utils import with_user_profile  # backwards compat
 
-
 CONVERTERS = []
 
 
 def dict2js(d):
     return ", ".join(["%s: %s" % (k, py2js(v)) for k, v in d.items()])
 
+
 def obj2dict(o, attrs):
     """
 
@@ -61,7 +61,6 @@ def register_converter(func):
     CONVERTERS.append(func)
 
 
-
 # def key2js(s):
 #     if isinstance(s, str):
 #         return s
@@ -69,21 +68,22 @@ def register_converter(func):
 
 
 def id2js(s):
-    return s.replace('.', '_')
+    return s.replace(".", "_")
 
 
 class js_code(object):
-
     "A string that py2js will represent as is, not between quotes."
 
     def __init__(self, s):
         self.s = s
+
     # def __repr__(self):
-        # return self.s
+    # return self.s
 
 
 def js_line(s, *args):
-    return js_code(s + '\n', *args)
+    return js_code(s + "\n", *args)
+
 
 DECLARE_INLINE = 0
 DECLARE_VAR = 1
@@ -91,7 +91,6 @@ DECLARE_THIS = 2
 
 
 class Value(object):
-
     declare_type = DECLARE_INLINE
     value_template = "%s"
 
@@ -105,18 +104,18 @@ class Value(object):
         return []
 
     # def js_before_body(self):
-        # for v in self.subvars():
-            # for ln in v.js_before_body():
-                # yield ln
+    # for v in self.subvars():
+    # for ln in v.js_before_body():
+    # yield ln
     def js_body(self):
         for v in self.subvars():
             for ln in v.js_body():
                 yield ln
 
     # def js_after_body(self):
-        # for v in self.subvars():
-            # for ln in v.js_after_body():
-                # yield ln
+    # for v in self.subvars():
+    # for ln in v.js_after_body():
+    # yield ln
 
     def as_ext(self):
         return self.value_template % py2js(self.value)
@@ -127,30 +126,28 @@ VARIABLE_COUNTER = 0
 
 class Variable(Value):
     declare_type = DECLARE_INLINE
-    ext_suffix = ''
+    ext_suffix = ""
     name = None
     ext_name = None
 
     def __init__(self, name, value):
-
         global VARIABLE_COUNTER
         VARIABLE_COUNTER += 1
         if name is None:
             self.ext_name = "var%s%d" % (self.ext_suffix, VARIABLE_COUNTER)
         else:
             name = str(name)
-            self.ext_name = "%s%s%d" % (
-                id2js(name), self.ext_suffix, VARIABLE_COUNTER)
+            self.ext_name = "%s%s%d" % (id2js(name), self.ext_suffix, VARIABLE_COUNTER)
         Value.__init__(self, value)
         # assert self.declare_type != DECLARE_INLINE
 
         self.name = name
         # if name is None:
-            # assert self.declare_type == DECLARE_INLINE
-            # ~ #self.name = "unnamed %s" % self.__class__.__name__
+        # assert self.declare_type == DECLARE_INLINE
+        # ~ #self.name = "unnamed %s" % self.__class__.__name__
         # else:
-            # self.name = name
-            # self.ext_name = id2js(name) + self.ext_suffix
+        # self.name = name
+        # self.ext_name = id2js(name) + self.ext_suffix
         # self.ext_name = id2js(name) + self.ext_suffix
 
     def __str__(self):
@@ -175,7 +172,7 @@ class Variable(Value):
         yield "// end js_declare %s" % self
 
     # def js_column_lines(self):
-        # return []
+    # return []
 
     def as_ext(self):
         if self.declare_type == DECLARE_INLINE:
@@ -207,7 +204,7 @@ class Component(Variable):
         return kw
 
     def update(self, **kw):
-        if 'label' in kw:
+        if "label" in kw:
             raise Exception("20181023")
         self.value.update(**kw)
 
@@ -218,7 +215,7 @@ class Component(Variable):
 
     def walk(self):
         """Walk over this component and its children."""
-        items = self.value['items']
+        items = self.value["items"]
         if not isinstance(items, (list, tuple)):
             items = [items]
         for i in items:
@@ -228,8 +225,8 @@ class Component(Variable):
 
 
 class VisibleComponent(Component, Permittable):
-    """A visible component
-    """
+    """A visible component"""
+
     vflex = False
     hflex = True
     width = None
@@ -255,10 +252,12 @@ class VisibleComponent(Component, Permittable):
         :attr:`required_roles`.
 
         """
-        self.allow_read = curry(make_static_permission_handler(
-            self, True,
-            self.debug_permissions,
-            self.required_roles), self)
+        self.allow_read = curry(
+            make_static_permission_handler(
+                self, True, self.debug_permissions, self.required_roles
+            ),
+            self,
+        )
 
     def is_visible(self):
         if self.hidden:
@@ -268,11 +267,16 @@ class VisibleComponent(Component, Permittable):
     def get_view_permission(self, profile):
         return self.allow_read(profile)
 
-    def setup(self, width=None, height=None, label=None,
-              preferred_width=None,
-              # help_text=None,
-              required_roles=NOT_PROVIDED,
-              **kw):
+    def setup(
+        self,
+        width=None,
+        height=None,
+        label=None,
+        preferred_width=None,
+        # help_text=None,
+        required_roles=NOT_PROVIDED,
+        **kw,
+    ):
         self.value.update(kw)
         # Component.__init__(self,name,**kw)
         if preferred_width is not None:
@@ -286,8 +290,8 @@ class VisibleComponent(Component, Permittable):
         if required_roles is not NOT_PROVIDED:
             if not isinstance(required_roles, set):
                 raise Exception(
-                    "20150628 %s has required_roles %s" % (
-                        self, required_roles))
+                    "20150628 %s has required_roles %s" % (self, required_roles)
+                )
             self.required_roles = required_roles
         # if help_text is not None:
         #     self.help_text = help_text
@@ -319,25 +323,35 @@ class VisibleComponent(Component, Permittable):
             yield self
 
     def debug_lines(self):
-        sep = u"</td><td>"
+        sep = "</td><td>"
         cols = """ext_name name parent label __class__.__name__
         elements js_value
         label_align vertical width preferred_width height
         preferred_height vflex""".split()
-        yield '<tr><td>' + sep.join(cols) + '</td></tr>'
+        yield "<tr><td>" + sep.join(cols) + "</td></tr>"
         for e in self.walk():
-            yield '<tr><td>' + sep.join([py2html(e, n) for n in cols]) + '</td></tr>'
+            yield "<tr><td>" + sep.join([py2html(e, n) for n in cols]) + "</td></tr>"
 
     def ext_options(self, **kw):
         kw = Component.ext_options(self, **kw)
         # edge case of invalid extjs layout when user doesn't have permision to view vflex items. Ticket #2916
-        if getattr(self, 'vflex', False) and kw.get('region', "") != "center" and \
-                get_user_profile() and \
-                len([e for e in getattr(getattr(self, 'parent',None), "elements",[])
-                     if e.get_view_permission(get_user_profile())]) == 1:
-            kw['region'] = 'center'
+        if (
+            getattr(self, "vflex", False)
+            and kw.get("region", "") != "center"
+            and get_user_profile()
+            and len(
+                [
+                    e
+                    for e in getattr(getattr(self, "parent", None), "elements", [])
+                    if e.get_view_permission(get_user_profile())
+                ]
+            )
+            == 1
+        ):
+            kw["region"] = "center"
         return kw
 
+
 def declare_vars(v):
     """
     Yields the Javascript lines that declare the given  :class:`Variable` `v`.
@@ -355,7 +369,8 @@ def declare_vars(v):
                 yield ln
         return
     if isinstance(v, VisibleComponent) and not v.get_view_permission(
-            get_user_profile()):
+        get_user_profile()
+    ):
         return
     if isinstance(v, Component):
         for sub in v.ext_options().values():
@@ -378,9 +393,7 @@ def declare_vars(v):
 
 
 def py2js(v, compact=True):
-    """Note that None values are rendered as ``null`` (not ``undefined``).
-
-    """
+    """Note that None values are rendered as ``null`` (not ``undefined``)."""
     # if isinstance(v, DelayedValue):
     #     # v = dict(field=v.field.name, pk=v.obj.pk)
     #     v = dict(delayed_value_url=v.get_url())
@@ -390,7 +403,7 @@ def py2js(v, compact=True):
         v = cv(v)
 
     # if isinstance(v,LanguageInfo):
-        # return v.django_code
+    # return v.django_code
 
     if isinstance(v, SafeString):
         # print("20231228")
@@ -399,7 +412,7 @@ def py2js(v, compact=True):
         return v.as_ext()
     if isinstance(v, Promise):
         # v = force_str(v)
-        return json.dumps(force_str(v.encode('utf8')))
+        return json.dumps(force_str(v.encode("utf8")))
     if isinstance(v, types.GeneratorType):
         return "".join([py2js(x, compact=compact) for x in v])
     if etree.iselement(v):
@@ -410,23 +423,27 @@ def py2js(v, compact=True):
         #     return json.dumps("Failed to render {!r} : {}".format(v, e))
 
     # if type(v) is types.GeneratorType:
-        # raise Exception("Please don't call the generator function yourself")
-        # return "\n".join([ln for ln in v])
+    # raise Exception("Please don't call the generator function yourself")
+    # return "\n".join([ln for ln in v])
     if callable(v):
         # print 20120114, repr(v)
         # raise Exception("Please call the function yourself")
         return "\n".join([ln for ln in v()])
     if isinstance(v, MissingRow):
-        raise Exception("Cannot render {}".format(v))
+        return json.dumps(repr(v))
+        # raise Exception("Cannot render {}".format(v))
     if isinstance(v, js_code):
         return str(v.s)  # v.s might be a unicode
     if v is None:
         # return 'undefined'
-        return 'null'
+        return "null"
     if isinstance(v, (list, tuple)):  # (types.ListType, types.TupleType):
-        elems = [py2js(x, compact=compact) for x in v
-                 if (not isinstance(x, VisibleComponent))
-                 or x.get_view_permission(get_user_profile())]
+        elems = [
+            py2js(x, compact=compact)
+            for x in v
+            if (not isinstance(x, VisibleComponent))
+            or x.get_view_permission(get_user_profile())
+        ]
         sep = ", " if compact else ", \n"
         return "[ %s ]" % sep.join(elems)
 
@@ -434,9 +451,11 @@ def py2js(v, compact=True):
         items = [
             # i for i in sorted(v.items(), key=lambda x: str(x))
             # i for i in sorted(v.items())
-            i for i in v.items()
+            i
+            for i in v.items()
             if (not isinstance(v, VisibleComponent))
-            or v.get_view_permission(get_user_profile())]
+            or v.get_view_permission(get_user_profile())
+        ]
 
         def sortkey(x):
             return x[0]
@@ -448,7 +467,11 @@ def py2js(v, compact=True):
         #     raise TypeError("Failed to sort {0!r} : {1}".format(items, e))
         sep = ", " if compact else ", \n"
         return "{ %s }" % sep.join(
-            ["%s: %s" % (py2js(k, compact=compact), py2js(i, compact=compact)) for k, i in items])
+            [
+                "%s: %s" % (py2js(k, compact=compact), py2js(i, compact=compact))
+                for k, i in items
+            ]
+        )
 
     if isinstance(v, bool):  # types.BooleanType:
         return str(v).lower()
@@ -472,12 +495,10 @@ def py2js(v, compact=True):
         return repr(v)
     # return json.encoder.encode_basestring(v)
     # print repr(v)
-    # https://docs.djangoproject.com/en/5.0/topics/serialization/
+    # https://docs.djangoproject.com/en/5.2/topics/serialization/
     # if not isinstance(v, (str,unicode)):
-        # raise Exception("20120121 %r is of type %s" % (v,type(v)))
-    return json.dumps(v, sort_keys=True,
-                  indent=4, separators=(',', ': ')
-                      )
+    # raise Exception("20120121 %r is of type %s" % (v,type(v)))
+    return json.dumps(v, sort_keys=True, indent=4, separators=(",", ": "))
     # try:
     #     return json.dumps(v)
     # except TypeError as e:
@@ -487,7 +508,9 @@ def py2js(v, compact=True):
 
 def _test():
     import doctest
+
     doctest.testmod()
 
+
 if __name__ == "__main__":
     _test()
