@@ -1,6 +1,5 @@
 # Copyright 2011-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 This is for writing fixtures that import data from an MS-Access
 database (:xfile:`.mdb`) into Lino.
@@ -42,16 +41,16 @@ which we include here to make it usable in Python 2.6 too.
 
 from lino import logger
 
-
 # ~ ENCODING = 'latin1' # the encoding used by the mdb file
-ENCODING = 'utf8'
-#~ MDB_FILE = 'PPv5MasterCopie.mdb'
-MDBTOOLS_EXPORT = 'mdb-export'
+ENCODING = "utf8"
+# ~ MDB_FILE = 'PPv5MasterCopie.mdb'
+MDBTOOLS_EXPORT = "mdb-export"
 
 import os
 import sys
-#~ ENCODING = sys.stdout.encoding
-#~ import csv
+
+# ~ ENCODING = sys.stdout.encoding
+# ~ import csv
 import codecs
 import datetime
 
@@ -60,12 +59,10 @@ from django.conf import settings
 from lino.utils import ucsv
 from lino.utils import dblogger
 
-
 # ~ ENCODING = 'latin1' # the encoding used by the mdb file
-ENCODING = 'utf8'
-#~ MDB_FILE = 'PPv5MasterCopie.mdb'
-MDBTOOLS_EXPORT = 'mdb-export'
-
+ENCODING = "utf8"
+# ~ MDB_FILE = 'PPv5MasterCopie.mdb'
+MDBTOOLS_EXPORT = "mdb-export"
 
 try:
     from subprocess import check_output, STDOUT
@@ -84,11 +81,9 @@ except ImportError:
         The arguments are the same as for the Popen constructor.
         Example in :doc:`/topics/utils`.
         """
-        if 'stdout' in kwargs:
-            raise ValueError(
-                'stdout argument not allowed, it will be overridden.')
-        process = subprocess.Popen(
-            stdout=subprocess.PIPE, *popenargs, **kwargs)
+        if "stdout" in kwargs:
+            raise ValueError("stdout argument not allowed, it will be overridden.")
+        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
         output, unused_err = process.communicate()
         retcode = process.poll()
         if retcode:
@@ -109,19 +104,25 @@ class Loader(object):
         if os.path.exists(fn):
             logger.warning("Not re-extracting %s since it exists.", fn)
         else:
-            args = [MDBTOOLS_EXPORT, '-D', "%Y-%m-%d %H:%M:%S",
-                    self.mdb_file, self.table_name]
-            s = check_output(args, executable=MDBTOOLS_EXPORT,
-                             env=dict(
-                                 MDB_ICONV='utf-8',
-                                 MDB_JET_CHARSET='utf-8'))
-            #~ print ENCODING
+            args = [
+                MDBTOOLS_EXPORT,
+                "-D",
+                "%Y-%m-%d %H:%M:%S",
+                self.mdb_file,
+                self.table_name,
+            ]
+            s = check_output(
+                args,
+                executable=MDBTOOLS_EXPORT,
+                env=dict(MDB_ICONV="utf-8", MDB_JET_CHARSET="utf-8"),
+            )
+            # ~ print ENCODING
 
-            fd = open(fn, 'w')
+            fd = open(fn, "w")
             fd.write(s)
             fd.close()
             logger.info("Extracted file %s", fn)
-        reader = ucsv.UnicodeReader(open(fn, 'r'), encoding=ENCODING)
+        reader = ucsv.UnicodeReader(open(fn, "r"), encoding=ENCODING)
         headers = next(reader)
         if not headers == self.headers:
             raise Exception("%r != %r" % (headers, self.headers))
@@ -132,13 +133,13 @@ class Loader(object):
                 row[h] = values[i]
             n += 1
             if False:
-                if int(row['IDClient']) == 967:
+                if int(row["IDClient"]) == 967:
                     print(row)
                     raise Exception("20110609")
 
             if False:
                 if n < 10:
-                    print(n, ':', row)
+                    print(n, ":", row)
                 else:
                     raise Exception("20110609")
             for obj in self.row2obj(row):
@@ -151,8 +152,8 @@ class Loader(object):
         if len(dt) != 2:
             raise Exception("Unexpected datetime string %r" % s)
         d = dt[0]
-        #~ t = dt[1]
-        a = [int(i) for i in d.split('-')]
+        # ~ t = dt[1]
+        a = [int(i) for i in d.split("-")]
         return datetime.date(year=a[0], month=a[1], day=a[2])
 
     def parsetime(self, s):
@@ -163,5 +164,5 @@ class Loader(object):
             raise Exception("Unexpected datetime string %r" % s)
         t = dt[1]
         return t[:5]
-        #~ a = [int(i) for i in t.split(':')]
-        #~ return datetime.time(hour=a[0],minute=a[1],second=a[2])
+        # ~ a = [int(i) for i in t.split(':')]
+        # ~ return datetime.time(hour=a[0],minute=a[1],second=a[2])
