@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2012-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 r""" OdsReader uses odfpy to extract data from an .ods document
 (OpenOffice.org spreadsheet).
 
@@ -28,6 +27,7 @@ import os
 def srcpath(filename):
     return os.path.join(os.path.dirname(__file__), filename)
 
+
 from odf import opendocument
 from odf.table import Table, TableRow, TableCell
 from odf.text import P
@@ -40,6 +40,7 @@ class SimpleOdsReader(object):
     subclass of this.
 
     """
+
     filename = None
     """The full path name of the .ods file to be read.
 
@@ -78,8 +79,7 @@ class SimpleOdsReader(object):
             for row in sheet.getElementsByType(TableRow):
                 cells = []
                 for i, cell in enumerate(row.getElementsByType(TableCell)):
-
-                    content = ''
+                    content = ""
                     for p in cell.getElementsByType(P):
                         for n in p.childNodes:
                             content += str(n.data)
@@ -92,8 +92,8 @@ class SimpleOdsReader(object):
                     for r in range(repeat):
                         cells.append(content)
 
-                #~ if len(cells):
-                if ''.join(cells).strip():
+                # ~ if len(cells):
+                if "".join(cells).strip():
                     if headers_found:
                         yield self.cells2row(cells)
                     elif cells == self.headers:
@@ -106,7 +106,6 @@ class SimpleOdsReader(object):
 
 
 class OdsReader(SimpleOdsReader):
-
     """
     Like :class:`SimpleOdsReader`, but each row is converted to
     an :class:`lino.utils.AttrDict`. This requires you to specifiy,
@@ -115,6 +114,7 @@ class OdsReader(SimpleOdsReader):
     attribute names.
 
     """
+
     column_names = None
 
     def cells2row(self, cells):
@@ -129,13 +129,15 @@ class OdsReader(SimpleOdsReader):
             if name:
                 d[name] = None
             i += 1
-        #~ logger.info("20121122 cells2row %s -> %s",cells,d)
+        # ~ logger.info("20121122 cells2row %s -> %s",cells,d)
         return AttrDict(d)
 
 
 def _test():
     import doctest
+
     doctest.testmod()
 
+
 if __name__ == "__main__":
     _test()
