@@ -1,6 +1,5 @@
 # Copyright 2013-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """An extended :class:`TestCase` for the plain python tests of a Lino
 project.
 
@@ -40,6 +39,7 @@ class TestCase(TestCase):
     def setUp(self):
         if self.django_settings_module:
             from lino.core.signals import testcase_setup
+
             testcase_setup.send(self)
         super(TestCase, self).setUp()
 
@@ -83,8 +83,7 @@ class TestCase(TestCase):
         self.run_subprocess(args, **kw)
 
     def run_django_admin_command_cd(self, cwd, *cmdargs, **kw):
-        """Run `django-admin.py CMD` in the given directory.
-        """
+        """Run `django-admin.py CMD` in the given directory."""
         kw.update(cwd=cwd)
         args = [sys.executable, "manage.py"]
         # args += ["--top-level-directory=" + os.path.abspath(cwd)]
@@ -107,10 +106,10 @@ class TestCase(TestCase):
         has not been initialized (in that case, run `fab initdb`).
 
         """
-        filename = 'docs/' + filename
-        sys.path.insert(0,  os.path.abspath('docs'))
+        filename = "docs/" + filename
+        sys.path.insert(0, os.path.abspath("docs"))
         import conf  # import Sphinx conf.py which possibly triggers
-                     # Django startup
+        # Django startup
 
         self.check_doctest(filename)
         # res = doctest.testfile(filename, module_relative=False,
@@ -118,21 +117,23 @@ class TestCase(TestCase):
         #                        optionflags=doctest.REPORT_ONLY_FIRST_FAILURE)
 
         del sys.path[0]
-        #~ os.chdir(oldcwd)
+        # ~ os.chdir(oldcwd)
 
         # if res.failed:
         #     self.fail("doctest.testfile() failed. See earlier messages.")
 
     def check_doctest(self, n):
         res = doctest.testfile(
-            n, module_relative=False,
-            encoding='utf-8',
-            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE)
+            n,
+            module_relative=False,
+            encoding="utf-8",
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE,
+        )
         if res.failed:
             self.fail("doctest {0} failed.".format(n))
 
+    demo_projects_root = "."
 
-    demo_projects_root = '.'
     def do_test_demo_project(self, prjname):
         """
 
