@@ -15,7 +15,7 @@ def unlink(file: str) -> None:
 
 
 def get_from_socket(sock: socket.socket) -> bytes:
-    MSGLEN = struct.unpack('>L', sock.recv(4))[0]
+    MSGLEN = struct.unpack(">L", sock.recv(4))[0]
     return sock.recv(MSGLEN)
 
 
