@@ -1,12 +1,19 @@
+# from future import standard_library
+# standard_library.install_aliases()
+# from builtins import str
+# from builtins import object
 # Copied from http://docs.python.org/library/csv.html
 
 import csv
 import codecs
-import cStringIO
+import io
+
+import six
+from builtins import object
+# from builtins import str
 
 
-class UTF8Recoder:
-
+class UTF8Recoder(object):
     """
     Iterator that reads an encoded stream and reencodes the input to UTF-8
     """
@@ -17,12 +24,14 @@ class UTF8Recoder:
     def __iter__(self):
         return self
 
-    def next(self):
-        return self.reader.next().encode("utf-8")
+    def __next__(self):
+        if six.PY2:
+            return self.reader.readline().encode("utf-8")
+        else:
+            return self.reader.readline()
 
 
-class UnicodeReader:
-
+class UnicodeReader(object):
     """
     A CSV reader which will iterate over lines in the CSV file "f",
     which is encoded in the given encoding.
@@ -32,16 +41,18 @@ class UnicodeReader:
         f = UTF8Recoder(f, encoding)
         self.reader = csv.reader(f, dialect=dialect, **kwds)
 
-    def next(self):
-        row = self.reader.next()
-        return [unicode(s, "utf-8") for s in row]
+    def __next__(self):
+        row = next(self.reader)
+        if six.PY2:
+            return [six.text_type(s, "utf-8") for s in row]
+        else:
+            return [s for s in row]
 
     def __iter__(self):
         return self
 
 
-class UnicodeWriter:
-
+class UnicodeWriter(object):
     """
     A CSV writer which will write rows to CSV file "f",
     which is encoded in the given encoding.
@@ -49,7 +60,7 @@ class UnicodeWriter:
 
     def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
         # Redirect output to a queue
-        self.queue = cStringIO.StringIO()
+        self.queue = io.StringIO()
         self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
         self.stream = f
         self.encoder = codecs.getincrementalencoder(encoding)()
