@@ -9,8 +9,9 @@ import datetime
 
 from lino.api import dd
 from lino.utils import ONE_DAY
+
 SEVEN_DAYS = datetime.timedelta(days=7)
-from etgen.html import E
+from lino.utils.html import E
 from lino.modlib.system.choicelists import Weekdays
 from lino.modlib.users.roles import SiteUser
 
